
#pragma once

// This file was auto-generated with uCalc Transform 3.0 using uCalcH.uc on 02/24/20 at 1:19 AM 

// #define uCalcForWindows 1
// #define uCalcForLinux 1
// #define uCalcForAndroid 1
// #define uCalcForMac 1
// #define uCalcForiOS 1

#include <cstdint>
#include <string>
#include <cfenv>
#if defined(_WIN64)
#   pragma comment(lib, "uCalc64.lib")
#elif defined(_WIN32)
#   pragma comment(lib, "uCalc32.lib")
#else
#  include <dlfcn.h>
#endif

namespace uCalcSoftware {
   
   using namespace std;
   
#if defined(uCalcForLinux) || defined(uCalcForMac) || defined(uCalcForiOS)
   #define StdCall
   #define DllImport
#elif defined(uCalcForAndroid)
   #define uCalcLib "./libuCalc.so"
   #define LoadLibraryA(lib) dlopen(lib, RTLD_LAZY)
   #define GetProcAddress(lib_handle, func) dlsym(lib_handle, func)
   #define HINSTANCE void *
#else // uCalcForWindows (implied)
   #define StdCall __stdcall
   #define DllImport __declspec(dllimport)
   
#if defined(_WIN64)
      #define uCalcLib "uCalc64.dll"
   #else // defined(_WIN32)
      #define uCalcLib "uCalc32.dll"
   #endif
#endif
   
   /// <summary> Data types that come with uCalc</summary>
   ///  <remarks>
   /// <para>These are built-in data types that come with uCalc</para>
   /// </remarks>
   enum class BuiltInType : int32_t {
      Unspecified,
      Auto,
      /// <summary> Boolean</summary>
      Boolean,
      /// <summary> Int8</summary>
      Integer_8,
      /// <summary> Byte</summary>
      Integer_8u,
      /// <summary> Int16</summary>
      Integer_16,
      /// <summary> Int16u</summary>
      Integer_16u,
      /// <summary> Int32</summary>
      Integer_32,
      /// <summary> Int32u</summary>
      Integer_32u,
      /// <summary> Int64</summary>
      Integer_64,
      /// <summary> Int64u</summary>
      Integer_64u,
      /// <summary> Int128</summary>
      Integer_128,
      /// <summary> Int128u</summary>
      Integer_128u,
      /// <summary> Single</summary>
      Float_Single,
      /// <summary> Double</summary>
      Float_Double,
      /// <summary> Extended</summary>
      Float_Extended,
      Size_t,
      Complex,
      Decimal,
      BigNum,
      Date,
      String,
      StringView,
      /// <summary> LPCSTR</summary>
      String_LPCSTR,
      /// <summary> FixedStr</summary>
      String_Fixed,
      /// <summary> WideStr</summary>
      String_Wide,
      /// <summary> StackList</summary>
      List_Stack,
      /// <summary> TableList</summary>
      List_Table,
      /// <summary> SortedList</summary>
      List_Sorted,
      Null,
      Void,
      Any,
      Pointer,
      Omni,
      UserDefined,
      DataType,
      Namespace,
      IntPtr,
      ADDR,
      uCalcPtr,
      Item,
      ParsedExpr,
      uCalcCallback,
      Object,
      Intrinsic
   };
   
   /// <summary> Data type constructors</summary>
   enum class DataTypeConstruction : int32_t {
      ByteSize,
      AllocArray,
      AllocScalar,
      AllocByPtr,
      ReleaseArray,
      ReleaseScalar,
      ResetScalar,
      ResetArray,
      SetArrayValue,
      SetScalarValue,
      SwapScalarValues
   };
   
   /// <summary> Error hander responses</summary>
   enum class ErrorHandlerResponse : int32_t {
      /// <summary> Immediately stops parsing</summary>
      Abort,
      /// <summary> Continues parsing (after the error is resolved)</summary>
      Resume,
      /// <summary> Raises the same error again for the next error handler to take care of</summary>
      ReRaise
   };
   
   /// <summary> Error numbers that uCalc may return</summary>
   enum class ErrorMsg : int32_t {
      /// <summary> No error</summary>
      None,
      /// <summary> Inexact result</summary>
      FloatInexact = 1,
      /// <summary> Floating point underflow</summary>
      FloatUnderflow = 2,
      /// <summary> Floating point overflow</summary>
      FloatOverflow = 4,
      /// <summary> Division by 0</summary>
      FloatDivisionByZero = 8,
      /// <summary> Invalid floating point value</summary>
      FloatInvalid = 16,
      /// <summary> Dynamically defined user error</summary>
      Dynamically_Defined,
      /// <summary> Syntax error</summary>
      Syntax_Error,
      /// <summary> Undefined identifier</summary>
      Undefined_Identifier,
      /// <summary> File not found</summary>
      FileNotFound,
      /// <summary> Unrecognized token</summary>
      Unrecognized_Token,
      /// <summary> Unrecognized command</summary>
      Unrecognized_Command,
      /// <summary> Data type mismatch</summary>
      Datatype_Mismatch,
      /// <summary> Invalid number of arguments</summary>
      Invalid_Argument_Count,
      /// <summary> Invalid definition</summary>
      Invalid_Definition,
      /// <summary> Bracket delimiter error</summary>
      Bracket_Error,
      /// <summary> Undefined callback</summary>
      Undefined_Callback,
      /// <summary> Error message already defined</summary>
      ErrorMessageAlreadyDefined,
      /// <summary> This item cannot be modified</summary>
      ItemCannotBeModified,
      /// <summary> Unrecognized class member</summary>
      Unrecognized_Class_Member,
      /// <summary> Unrecognized namespace member</summary>
      Unrecognized_Namespace_Member,
      /// <summary> Unbalanced quote</summary>
      Unbalanced_Quote,
      /// <summary> Value cannot be assigned here</summary>
      ValueCannotBeAssigned,
      /// <summary> Reparsing has passed the specified limit</summary>
      ReParseOverflow,
      /// <summary> Array or list bound exceeded</summary>
      Array_Bounds_Exceeded,
      Customized1 = 100,
      Customized2,
      Customized3,
      Customized4,
      Customized5,
      Customized6,
      Customized7,
      Customized8,
      Customized9,
      Customized10,
      Customized11,
      Customized12,
      Customized13,
      Customized14,
      Customized15,
      Customized16,
      Customized17,
      Customized18,
      Customized19,
      Customized20
   };
   
   /// <summary> Operator grouping associativity</summary>
   enum class Grouping : int32_t {
      LeftToRight,
      RightToLeft
   };
   
   /// <summary> Properties that a uCalc item may have</summary>
   ///  <remarks>
   /// <para>
   /// Items defined with uCalc typically have a combination of properties.  A variable defined
   /// with DefineVariable for instance will have the ItemIs[Variable] property set.  An operator
   /// will have the ItemIs[Operator] property set.  If it's an infix operator, in addition to
   /// having the ItemIs[Operator] property set, ItemIs[Infix] will also be set.</para>
   /// <para>The IsProperty function can determine if a given uCalc.Item has a certain property.</para>
   /// <para>You can use the IsProperty function to cycle through the list of all items that share
   /// certain properties, or to disambiguate between items that share the same name but have
   /// different properties (for instance the + operator with the Prefix property vs another one
   /// with the Infix property).</para>
   /// <para>Some uCalc enum elements are keywords in VB.NET.  To avoid a naming conflict, enclose the member name in brackets.
   /// For instance: [Function], [Operator], [Optional], [ByVal], etc.
   /// </para>
   /// </remarks>
   enum class ItemIs : int32_t {
      SelectAny,
      SelectAll,
      RightToLeft,
      DataType,
      Array,
      Class,
      Namespace,
      Variable,
      Function,
      Operator,
      FunctionOrOperator,
      Prefix,
      Infix,
      Postfix,
      AddressOwner,
      ByVal,
      ByRef,
      ByCopy,
      ByExpr,
      ByHandle,
      Pointer,
      Optional,
      VariableNumberOfArgs,
      Alias,
      LiteralItem,
      Locked,
      Exclude,
      Transform,
      Search,
      FinishedSearch,
      ReturnSelf,
      ErrorHandler,
      ApiCall,
      CaseSensitive,
      QuotedText,
      RewindOnChange,
      TransformArg,
      Active,
      WhitespaceCounts,
      IgnoreQuote,
      IgnoreStatementSep,
      IgnoreBracket,
      SkipEvalTransform,
      ExprUnit,
      EagerArg,
      SkipPatternMatching,
      SkipOver,
      Modified,
      UpdatePending,
      ParentUpdatePending,
      UpToEndOrSeparator,
      UpToCompleted,
      SkipReduce,
      PatternCheck,
      EvalStrOutput,
      SkipOutput,
      NotFound,
      ListReturned,
      Focusable,
      DisconnectedParent,
      Child,
      Parent,
      Range,
      MiscItem
   };
   
   /// <summary> Licensing options for uCalc products</summary>
   enum class LicenseType : int32_t {
      Demo,
      FMP_Free,
      FMP_Standard_32,
      FMP_Standard_64,
      FMP_32_and_64,
      LanguageBuilder,
      GeneralParser,
      Interpreter,
      ConsoleCalc,
      WindowsCalc,
      StringLib,
      Transform,
      TransformLib,
      Search,
      SearchLib,
      Library1,
      Library2,
      Beta,
      Debugger,
      uCalc_All,
      Developer,
      EndUser
   };
   
   /// <summary> Types of tokens</summary>
   enum class TokenType : int32_t {
      None,
      /// <summary> A token defined without any specified property is generic (default)</summary>
      Generic,
      /// <summary> Represents the absence of any character (token not found)</summary>
      Empty,
      /// <summary> A token that behaves as alphanuric (works with word boundaries)</summary>
      AlphaNumeric,
      /// <summary> A token that serves as argument separator (typically a comma)</summary>
      ArgSeparator,
      /// <summary> A token that starts a bracketed group; typically strings like {, [, or (</summary>
      Bracket,
      /// <summary> The closing token that matches a Bracket token. Ex: }, ], or )</summary>
      BracketMatch,
      /// <summary> A token that separates an object from its member, typically a . (a dot)</summary>
      MemberAccess,
      /// <summary> A token that represents a literal value, typically a quoted string or number</summary>
      Literal,
      /// <summary> A token that represents a statement separator, typically a ; (semicolon)</summary>
      StatementSep,
      /// <summary> A token that represents whitespace, typically tab or space characters</summary>
      Whitespace,
      /// <summary> A token that is not delimited by word boundaries (+, -, *, etc), consecutive characters are reduced until they match a value that exists in the symbol table or until 1 character</summary>
      Reducible,
      /// <summary> Non-character that represents the end of a stream of text</summary>
      Eof,
      LocalVar,
      StaticVar,
      /// <summary> Item in parenthesis (or quotes) following this token will be defined while the expression is parsed</summary>
      DefineNow = StaticVar,
      TempDef,
      /// <summary> The expression in parathesis (or quotes) following the token is evaluated and inserted in place of the token in the string</summary>
      EvalInsert,
      EvalDuringParse,
      /// <summary> Works like EvalInsert, except the expression is transformed instead of evaluated</summary>
      TransformInsert,
      /// <summary> The contents of the quoted file name following this token is inserted into the string</summary>
      FileInclude,
      CurrentSpace,
      DefSpace,
      EvalSpace,
      TokenTransform
   };
   
   /// <summary> RegEx grammar option for defining tokens; based on C++ standard library regex flags</summary>
   enum class RegExGrammar : int32_t {
      /// <summary> Default grammar; ECMAScript is the default</summary>
      Default,
      /// <summary> JavaScript-like regex grammar</summary>
      ECMAScript,
      /// <summary> Basic POSIX grammar (BRE)</summary>
      basic,
      /// <summary> Extended POSIX grammar (ERE)</summary>
      extended,
      /// <summary> awk POSIX grammar</summary>
      awk,
      /// <summary> grep POSIX grammar</summary>
      grep,
      /// <summary> egrep POXIX grammar</summary>
      egrep
   };
   
   /// <summary> Transform reset options</summary>
   ///  <remarks>
   /// <para>Use these options with Transform Reset.  The Default is All</para>
   /// </remarks>
   enum class TransformResetOption : int32_t {
      /// <summary> Resets all (Input, Tokens, Rules)</summary>
      All,
      /// <summary> Resets the input to an empty string</summary>
      Input,
      /// <summary> Resets token definitions</summary>
      Tokens,
      /// <summary> Resets the FromTo rules</summary>
      Rules,
      /// <summary> Resets transform results</summary>
      Results,
      /// <summary> Removes passes</summary>
      Passes
   };
   
   enum class Comparison : int32_t {
      LessThan = -1,
      Equal = 0,
      GreaterThan = 1
   };
   
   enum class ChildStringOptions : int32_t {
      ClearAllOnNewObj,
      ClearOverlapOnRead,
      ClearOverlapOnWrite,
      DetachAllOnNewObj,
      DetachOverlapOnRead,
      DetachOverlapOnWrite
   };
   
   enum class StringListType : int32_t {
      None,
      Chars,
      Tokens,
      Elements,
      Statements,
      Lines,
      Matches
   };
   
   enum class MatchesOption : int32_t {
      All,
      SameAsBefore,
      EmptyList,
      MainLevelOnly,
      InnerMostOnly,
      FocusableOnly,
      NoOffset
   };
   
   typedef void (*uCalcPtr)(bool, char, int, long, float, double);
   typedef void (StdCall *uCalcCallback)(uCalcPtr);
   
   extern "C" DllImport const char * StdCall Name_1(void * , size_t & );
   extern "C" DllImport const char * StdCall Text_2(void * , size_t & );
   extern "C" DllImport uCalcPtr StdCall NextOverload_3(void * );
   extern "C" DllImport const char * StdCall Comment_4(void * , size_t & );
   extern "C" DllImport uCalcPtr StdCall Comment_5(void *  , const char *, size_t CommentStr);
   extern "C" DllImport uCalcPtr StdCall DataType_6(void * );
   extern "C" DllImport uCalcPtr StdCall uCalc_7(void * );
   extern "C" DllImport uCalcPtr StdCall IsProperty_8(void *  , ItemIs ItemProperty , int32_t TrueOrFalse);
   extern "C" DllImport int32_t StdCall IsProperty_9(void *  , ItemIs ItemProperty);
   extern "C" DllImport size_t StdCall Count_10(void * );
   extern "C" DllImport uCalcPtr StdCall BracketMatch_11(void *  , void * BracketToken);
   extern "C" DllImport uCalcPtr StdCall BracketMatch_12(void * );
   extern "C" DllImport uCalcPtr StdCall DataType_13(void *  , void * dType);
   extern "C" DllImport uCalcCallback StdCall FunctionAddress_14(void * );
   extern "C" DllImport const char * StdCall Regex_15(void * , size_t & );
   extern "C" DllImport uCalcPtr StdCall Regex_16(void *  , const char *, size_t Pattern , RegExGrammar Grammar);
   extern "C" DllImport size_t StdCall Precedence_17(void * );
   extern "C" DllImport void StdCall Precedence_18(void *  , size_t Level);
   extern "C" DllImport uCalcPtr StdCall Rename_19(void *  , const char *, size_t NewName);
   extern "C" DllImport double StdCall Value_20(void * );
   extern "C" DllImport void StdCall Value_21(void *  , double Value);
   extern "C" DllImport void StdCall Value_22(void *  , const char *, size_t Value);
   extern "C" DllImport int32_t StdCall ValueBool_23(void * );
   extern "C" DllImport void StdCall ValueBool_24(void *  , int32_t Value);
   extern "C" DllImport uint8_t StdCall ValueByte_25(void * );
   extern "C" DllImport void StdCall ValueByte_26(void *  , uint8_t Value);
   extern "C" DllImport int16_t StdCall ValueInt16_27(void * );
   extern "C" DllImport void StdCall ValueInt16_28(void *  , int16_t Value);
   extern "C" DllImport int32_t StdCall ValueInt32_29(void * );
   extern "C" DllImport void StdCall ValueInt32_30(void *  , int32_t Value);
   extern "C" DllImport int64_t StdCall ValueInt64_31(void * );
   extern "C" DllImport void StdCall ValueInt64_32(void *  , int64_t Value);
   extern "C" DllImport float StdCall ValueSng_33(void * );
   extern "C" DllImport void StdCall ValueSng_34(void *  , float Value);
   extern "C" DllImport double StdCall ValueDbl_35(void * );
   extern "C" DllImport void StdCall ValueDbl_36(void *  , double Value);
   extern "C" DllImport const char * StdCall ValueStr_37(void * , size_t & );
   extern "C" DllImport void StdCall ValueStr_38(void *  , const char *, size_t Value);
   extern "C" DllImport void * StdCall ValuePtr_39(void * );
   extern "C" DllImport void StdCall ValuePtr_40(void *  , void * Value);
   extern "C" DllImport void StdCall ValueByPtr_41(void *  , void * Value);
   extern "C" DllImport void * StdCall ValueAddr_42(void * );
   extern "C" DllImport void StdCall Release_43(void * );
   extern "C" DllImport void * StdCall Handle_44(void * );
   extern "C" DllImport double StdCall Evaluate_45(void * );
   extern "C" DllImport double StdCall EvaluateDbl_46(void * );
   extern "C" DllImport int32_t StdCall EvaluateInt32_47(void * );
   extern "C" DllImport int32_t StdCall EvaluateBool_48(void * );
   extern "C" DllImport const char * StdCall EvaluateStr_49(void * , size_t & );
   extern "C" DllImport void * StdCall EvaluateVoid_50(void * );
   extern "C" DllImport void StdCall Execute_51(void * );
   extern "C" DllImport uCalcPtr StdCall DataType_52(void * );
   extern "C" DllImport void StdCall Release_53(void * );
   extern "C" DllImport void * StdCall Handle_54(void * );
   extern "C" DllImport double StdCall Arg_55(void *  , size_t Index);
   extern "C" DllImport double StdCall ArgDbl_56(void *  , size_t Index);
   extern "C" DllImport double StdCall Arg1_57(void * );
   extern "C" DllImport double StdCall Arg2_58(void * );
   extern "C" DllImport int32_t StdCall ArgBool_59(void *  , size_t Index);
   extern "C" DllImport size_t StdCall ArgCount_60(void * );
   extern "C" DllImport uCalcPtr StdCall ArgObj_61(void *  , size_t Index);
   extern "C" DllImport void * StdCall ArgAddr_62(void *  , size_t Index);
   extern "C" DllImport uCalcPtr StdCall ArgExpr_63(void *  , size_t Index);
   extern "C" DllImport int32_t StdCall ArgInt32_64(void *  , size_t Index);
   extern "C" DllImport int64_t StdCall ArgInt64_65(void *  , size_t Index);
   extern "C" DllImport void * StdCall ArgPtr_66(void *  , size_t Index);
   extern "C" DllImport const char * StdCall ArgStr_67(void * , size_t & , size_t Index);
   extern "C" DllImport void StdCall Return_68(void *  , double Value);
   extern "C" DllImport void StdCall ReturnDbl_69(void *  , double Value);
   extern "C" DllImport void StdCall ReturnBool_70(void *  , int32_t Value);
   extern "C" DllImport void StdCall ReturnInt32_71(void *  , int32_t Value);
   extern "C" DllImport void StdCall ReturnInt64_72(void *  , int64_t Value);
   extern "C" DllImport void StdCall ReturnPtr_73(void *  , void * Value);
   extern "C" DllImport void StdCall ReturnStr_74(void *  , const char *, size_t Value);
   extern "C" DllImport void StdCall ErrorSet_75(void *  , ErrorMsg ErrNumber);
   extern "C" DllImport ErrorHandlerResponse StdCall ErrorRaise_76(void *  , ErrorMsg ErrNumber);
   extern "C" DllImport ErrorHandlerResponse StdCall ErrorRaiseMessage_77(void *  , const char *, size_t ErrMessage);
   extern "C" DllImport uCalcPtr StdCall uCalc_78(void * );
   extern "C" DllImport uCalcPtr StdCall Item_79(void * );
   extern "C" DllImport void StdCall Release_80(void * );
   extern "C" DllImport void * StdCall Handle_81(void * );
   extern "C" DllImport size_t StdCall ByteSize_82(void * );
   extern "C" DllImport BuiltInType StdCall Index_83(void * );
   extern "C" DllImport uCalcPtr StdCall Item_84(void * );
   extern "C" DllImport const char * StdCall Name_85(void * , size_t & );
   extern "C" DllImport void StdCall Reset_86(void *  , void * ValuePtr , size_t Size);
   extern "C" DllImport void StdCall Reset_87(void *  , void * ValuePtr);
   extern "C" DllImport void StdCall SetArray_88(void *  , void * Destination , void * Source , size_t Index);
   extern "C" DllImport void StdCall SetScalar_89(void *  , void * Destination , void * Source);
   extern "C" DllImport void StdCall SwapScalarValues_90(void *  , void * ValuePtrA , void * ValuePtrB);
   extern "C" DllImport const char * StdCall ToString_91(void * , size_t & , void * ValuePtr);
   extern "C" DllImport const char * StdCall ToString_92(void * , size_t & , const char *, size_t Value);
   extern "C" DllImport void StdCall Release_93(void * );
   extern "C" DllImport void * StdCall Handle_94(void * );
   extern "C" DllImport uCalcPtr StdCall IsProperty_95(void *  , ItemIs ItemProperty , int32_t TrueOrFalse);
   extern "C" DllImport size_t StdCall IndexOf_96(void *  , size_t StartPos , size_t NearestIndex);
   extern "C" DllImport int32_t StdCall IsProperty_97(void *  , ItemIs ItemProperty);
   extern "C" DllImport size_t StdCall OldEnd_98(void *  , size_t Index);
   extern "C" DllImport size_t StdCall OldLength_99(void *  , size_t Index);
   extern "C" DllImport void StdCall OldLength_100(void *  , size_t Index , size_t value);
   extern "C" DllImport const char * StdCall NewStr_101(void * , size_t & , size_t Index);
   extern "C" DllImport size_t StdCall Start_102(void *  , size_t Index);
   extern "C" DllImport void StdCall Start_103(void *  , size_t Index , size_t value);
   extern "C" DllImport size_t StdCall End_104(void *  , size_t Index);
   extern "C" DllImport size_t StdCall Length_105(void *  , size_t Index);
   extern "C" DllImport void StdCall Length_106(void *  , size_t Index , size_t value);
   extern "C" DllImport size_t StdCall Count_107(void * );
   extern "C" DllImport void StdCall Remove_108(void *  , size_t Index);
   extern "C" DllImport uCalcPtr StdCall Filter_109(void *  , void * NestedRule);
   extern "C" DllImport const char * StdCall Str_110(void * , size_t & , size_t Index);
   extern "C" DllImport const char * StdCall Str_111(void * , size_t & );
   extern "C" DllImport void StdCall Reset_112(void *  , MatchesOption Options);
   extern "C" DllImport uCalcPtr StdCall Rule_113(void *  , size_t Index);
   extern "C" DllImport size_t StdCall Offset_114(void *  , size_t StartingPos);
   extern "C" DllImport void StdCall Release_115(void * );
   extern "C" DllImport void * StdCall Handle_116(void * );
   extern "C" DllImport uCalcPtr StdCall IsProperty_117(void *  , ItemIs ItemProperty , int32_t TrueOrFalse);
   extern "C" DllImport int32_t StdCall IsProperty_118(void *  , ItemIs ItemProperty);
   extern "C" DllImport uCalcPtr StdCall Tokens_119(void * );
   extern "C" DllImport uCalcPtr StdCall FromTo_120(void *  , const char *, size_t FromStr , const char *, size_t ToStr);
   extern "C" DllImport uCalcPtr StdCall uCalc_121(void *  , void * uCalcHandle);
   extern "C" DllImport uCalcPtr StdCall Range_122(void *  , size_t Start , size_t Size);
   extern "C" DllImport uCalcPtr StdCall NewPass_123(void *  , int32_t Share);
   extern "C" DllImport uCalcPtr StdCall NewRule_124(void * );
   extern "C" DllImport uCalcPtr StdCall Pass_125(void *  , size_t Index , int32_t Share);
   extern "C" DllImport size_t StdCall PassCount_126(void * );
   extern "C" DllImport uCalcPtr StdCall Reset_127(void *  , TransformResetOption ResetOption);
   extern "C" DllImport uCalcPtr StdCall Transform_128(void * );
   extern "C" DllImport uCalcPtr StdCall Filter_129(void * );
   extern "C" DllImport uCalcPtr StdCall Filter_130(void *  , const char *, size_t expression);
   extern "C" DllImport uCalcPtr StdCall Find_131(void *  , const char *, size_t Str);
   extern "C" DllImport uCalcPtr StdCall Transform_132(void *  , const char *, size_t expression);
   extern "C" DllImport uCalcPtr StdCall TransformWithSteps_133(void *  , const char *, size_t expression);
   extern "C" DllImport const char * StdCall Str_134(void * , size_t & );
   extern "C" DllImport uCalcPtr StdCall Str_135(void *  , const char *, size_t InputStr);
   extern "C" DllImport const char * StdCall StrData_136(void * );
   extern "C" DllImport const char * StdCall SubStr_137(void * , size_t & , size_t Start , size_t Count);
   extern "C" DllImport size_t StdCall StrLength_138(void * );
   extern "C" DllImport uCalcPtr StdCall Matches_139(void *  , MatchesOption Options);
   extern "C" DllImport uCalcPtr StdCall IndexBase_140(void *  , size_t BaseValue);
   extern "C" DllImport size_t StdCall IndexBase_141(void * );
   extern "C" DllImport int32_t StdCall WasModified_142(void * );
   extern "C" DllImport uCalcPtr StdCall RulesDefault_143(void * );
   extern "C" DllImport uCalcPtr StdCall RulesOverall_144(void * );
   extern "C" DllImport void StdCall Release_145(void * );
   extern "C" DllImport void * StdCall Handle_146(void * );
   extern "C" DllImport uCalcPtr StdCall IsProperty_147(void *  , ItemIs ItemProperty , int32_t TrueOrFalse);
   extern "C" DllImport int32_t StdCall IsProperty_148(void *  , ItemIs ItemProperty);
   extern "C" DllImport int32_t StdCall Active_149(void * );
   extern "C" DllImport uCalcPtr StdCall Active_150(void *  , int32_t TrueFalse);
   extern "C" DllImport uCalcPtr StdCall FromTo_151(void *  , const char *, size_t FromStr , const char *, size_t ToStr);
   extern "C" DllImport int32_t StdCall Focusable_152(void * );
   extern "C" DllImport uCalcPtr StdCall Focusable_153(void *  , int32_t TrueFalse);
   extern "C" DllImport const char * StdCall FindPart_154(void * , size_t & );
   extern "C" DllImport uCalcPtr StdCall FindPart_155(void *  , const char *, size_t Pattern);
   extern "C" DllImport const char * StdCall FromPart_156(void * , size_t & );
   extern "C" DllImport uCalcPtr StdCall FromPart_157(void *  , const char *, size_t Pattern);
   extern "C" DllImport const char * StdCall ToPart_158(void * , size_t & );
   extern "C" DllImport uCalcPtr StdCall ToPart_159(void *  , const char *, size_t Pattern);
   extern "C" DllImport int32_t StdCall CaseSensitive_160(void * );
   extern "C" DllImport uCalcPtr StdCall CaseSensitive_161(void *  , int32_t TrueFalse);
   extern "C" DllImport int32_t StdCall IgnoreInnerPatterns_162(void * );
   extern "C" DllImport uCalcPtr StdCall IgnoreInnerPatterns_163(void *  , int32_t TrueFalse);
   extern "C" DllImport int32_t StdCall WhitespaceSensitive_164(void * );
   extern "C" DllImport uCalcPtr StdCall WhitespaceSensitive_165(void *  , int32_t TrueFalse);
   extern "C" DllImport int32_t StdCall BracketSensitive_166(void * );
   extern "C" DllImport uCalcPtr StdCall BracketSensitive_167(void *  , int32_t TrueFalse);
   extern "C" DllImport int32_t StdCall HasLocalTransform_168(void * );
   extern "C" DllImport uCalcPtr StdCall LocalTransform_169(void * );
   extern "C" DllImport int32_t StdCall StatementSensitive_170(void * );
   extern "C" DllImport uCalcPtr StdCall StatementSensitive_171(void *  , int32_t TrueFalse);
   extern "C" DllImport size_t StdCall Maximum_172(void * );
   extern "C" DllImport uCalcPtr StdCall Maximum_173(void *  , size_t Value);
   extern "C" DllImport size_t StdCall MaximumAND_174(void * );
   extern "C" DllImport uCalcPtr StdCall MaximumAND_175(void *  , size_t Value);
   extern "C" DllImport size_t StdCall Minimum_176(void * );
   extern "C" DllImport uCalcPtr StdCall Minimum_177(void *  , size_t Value);
   extern "C" DllImport size_t StdCall MinimumAND_178(void * );
   extern "C" DllImport uCalcPtr StdCall MinimumAND_179(void *  , size_t Value);
   extern "C" DllImport uCalcPtr StdCall ParentTransform_180(void * );
   extern "C" DllImport size_t StdCall Precedence_181(void * );
   extern "C" DllImport uCalcPtr StdCall Precedence_182(void *  , size_t Value);
   extern "C" DllImport uCalcPtr StdCall Property_183(void *  , ItemIs SelectedProperty , int32_t TrueFalse);
   extern "C" DllImport int32_t StdCall QuoteSensitive_184(void * );
   extern "C" DllImport uCalcPtr StdCall QuoteSensitive_185(void *  , int32_t TrueFalse);
   extern "C" DllImport const char * StdCall ReplacementText_186(void * , size_t & );
   extern "C" DllImport size_t StdCall StartAfter_187(void * );
   extern "C" DllImport uCalcPtr StdCall StartAfter_188(void *  , size_t Value);
   extern "C" DllImport size_t StdCall StopAfter_189(void * );
   extern "C" DllImport uCalcPtr StdCall StopAfter_190(void *  , size_t Value);
   extern "C" DllImport int32_t StdCall SkipOver_191(void * );
   extern "C" DllImport uCalcPtr StdCall SkipOver_192(void *  , int32_t TrueFalse);
   extern "C" DllImport int32_t StdCall RewindOnChange_193(void * );
   extern "C" DllImport uCalcPtr StdCall RewindOnChange_194(void *  , int32_t TrueFalse);
   extern "C" DllImport int32_t StdCall RightToLeft_195(void * );
   extern "C" DllImport uCalcPtr StdCall RightToLeft_196(void *  , int32_t TrueFalse);
   extern "C" DllImport uCalcPtr StdCall Matches_197(void *  , MatchesOption Options);
   extern "C" DllImport int32_t StdCall IsChildRule_198(void * );
   extern "C" DllImport size_t StdCall MatchesCount_199(void * );
   extern "C" DllImport size_t StdCall Index_200(void * );
   extern "C" DllImport uCalcPtr StdCall Index_201(void *  , size_t Value);
   extern "C" DllImport void StdCall Release_202(void * );
   extern "C" DllImport void * StdCall Handle_203(void * );
   extern "C" DllImport uCalcPtr StdCall IsProperty_204(void *  , ItemIs ItemProperty , int32_t TrueOrFalse);
   extern "C" DllImport int32_t StdCall IsProperty_205(void *  , ItemIs ItemProperty);
   extern "C" DllImport uCalcPtr StdCall Add_206(void *  , void * TokenCollection);
   extern "C" DllImport uCalcPtr StdCall Add_207(void *  , void * TokenItem);
   extern "C" DllImport uCalcPtr StdCall Add_208(void *  , const char *, size_t TokenDef , TokenType Type, const char *, size_t CloseBracket, size_t SubMatch, RegExGrammar Grammar, size_t InsertionPoint);
   extern "C" DllImport uCalcPtr StdCall Define_209(void *  , const char *, size_t TokenDef , TokenType Type, size_t SubMatch, RegExGrammar Grammar);
   extern "C" DllImport uCalcPtr StdCall Insert_210(void *  , int32_t InsertionPoint , const char *, size_t TokenDef , TokenType Type, const char *, size_t CloseBracket, size_t SubMatch, RegExGrammar Grammar);
   extern "C" DllImport uCalcPtr StdCall Token_211(void *  , size_t Index);
   extern "C" DllImport int32_t StdCall IndexOf_212(void *  , void * GivenToken);
   extern "C" DllImport void StdCall Remove_213(void *  , void * TokenItem);
   extern "C" DllImport uCalcPtr StdCall ContextSwitch_214(void *  , void * OtherTokenGroup , const char *, size_t StartToken , const char *, size_t EndToken);
   extern "C" DllImport uCalcPtr StdCall ContextSwitch_215(void *  , void * OtherTokenGroup , const char *, size_t StartToken , int32_t InsertionPointA, const char *, size_t EndToken, int32_t InsertionPointB);
   extern "C" DllImport uCalcPtr StdCall ContextSwitch_216(void *  , void * OtherTokenGroup , void * StartToken , int32_t InsertionPointA, const char *, size_t EndToken, int32_t InsertionPointB);
   extern "C" DllImport void StdCall Clear_217(void * );
   extern "C" DllImport size_t StdCall Count_218(void * );
   extern "C" DllImport void StdCall Release_219(void * );
   extern "C" DllImport void * StdCall Handle_220(void * );
   extern "C" DllImport size_t StdCall TokenCharLen_221(void *  , size_t StartIndex , size_t Count);
   extern "C" DllImport size_t StdCall TokenCount_222(void *  , size_t Index);
   extern "C" DllImport TokenType StdCall TypeOfToken_223(void *  , size_t Index);
   extern "C" DllImport Comparison StdCall Compare_224(void *  , const char *, size_t OtherString , size_t IndexA, size_t IndexB, size_t Length);
   extern "C" DllImport size_t StdCall TokenIndexAtChar_225(void *  , size_t CharacterPosition);
   extern "C" DllImport uCalcPtr StdCall Apply_226(void *  , const char *, size_t expression , const char *, size_t Variable);
   extern "C" DllImport uCalcPtr StdCall At_227(void *  , size_t Index);
   extern "C" DllImport uCalcPtr StdCall AtChar_228(void *  , size_t CharacterPosition);
   extern "C" DllImport uCalcPtr StdCall AtToken_229(void *  , size_t Index);
   extern "C" DllImport uCalcPtr StdCall BracketMatch_230(void *  , size_t Index);
   extern "C" DllImport uCalcPtr StdCall BracketedText_231(void *  , size_t Index);
   extern "C" DllImport uCalcPtr StdCall BracketedTextInside_232(void *  , size_t Index);
   extern "C" DllImport size_t StdCall BracketMatchTokenIndex_233(void *  , size_t Index);
   extern "C" DllImport size_t StdCall CharPos_234(void *  , size_t Nth);
   extern "C" DllImport size_t StdCall CharCount_235(void *  , size_t Nth);
   extern "C" DllImport size_t StdCall TokenPos_236(void *  , size_t Nth);
   extern "C" DllImport size_t StdCall TokenCharPos_237(void *  , size_t Index);
   extern "C" DllImport const char * StdCall Str_238(void * , size_t & , size_t Nth, size_t TokenCount);
   extern "C" DllImport uCalcPtr StdCall Str_239(void *  , const char *, size_t StringValue);
   extern "C" DllImport uCalcPtr StdCall Find_240(void *  , const char *, size_t Pattern , size_t Nth);
   extern "C" DllImport uCalcPtr StdCall FindOtherThan_241(void *  , const char *, size_t Pattern , size_t Nth);
   extern "C" DllImport uCalcPtr StdCall Parent_242(void * );
   extern "C" DllImport uCalcPtr StdCall Root_243(void * );
   extern "C" DllImport uCalcPtr StdCall IndexBase_244(void *  , size_t BaseValue);
   extern "C" DllImport size_t StdCall IndexBase_245(void * );
   extern "C" DllImport uCalcPtr StdCall UpTo_246(void *  , const char *, size_t Pattern , size_t Nth);
   extern "C" DllImport uCalcPtr StdCall StartingFrom_247(void *  , const char *, size_t Pattern , size_t Nth);
   extern "C" DllImport uCalcPtr StdCall StartingFromEndingWith_248(void *  , const char *, size_t StartPattern , const char *, size_t EndPattern , size_t StartNth, size_t EndNth);
   extern "C" DllImport uCalcPtr StdCall Before_249(void *  , const char *, size_t Pattern , size_t Nth);
   extern "C" DllImport uCalcPtr StdCall After_250(void *  , const char *, size_t Pattern , size_t Nth);
   extern "C" DllImport uCalcPtr StdCall Between_251(void *  , const char *, size_t StartPattern , const char *, size_t StopPattern , size_t StartNth, size_t StopNth);
   extern "C" DllImport uCalcPtr StdCall ListOfTokens_252(void * );
   extern "C" DllImport uCalcPtr StdCall ListOfChars_253(void * );
   extern "C" DllImport uCalcPtr StdCall SubString_254(void *  , size_t StartPos , size_t Length);
   extern "C" DllImport uCalcPtr StdCall ChildStringOption_255(void *  , ChildStringOptions Option);
   extern "C" DllImport ChildStringOptions StdCall ChildStringOption_256(void * );
   extern "C" DllImport uCalcPtr StdCall ClearDerivedStrings_257(void * );
   extern "C" DllImport uCalcPtr StdCall Remove_258(void *  , const char *, size_t From_);
   extern "C" DllImport uCalcPtr StdCall Replace_259(void *  , const char *, size_t From_ , const char *, size_t To_);
   extern "C" DllImport uCalcPtr StdCall Skip_260(void *  , const char *, size_t Pattern);
   extern "C" DllImport uCalcPtr StdCall ToUpper_261(void *  , const char *, size_t Pattern, size_t Nth);
   extern "C" DllImport uCalcPtr StdCall ToLower_262(void *  , const char *, size_t Pattern, size_t Nth);
   extern "C" DllImport uCalcPtr StdCall List_263(void *  , StringListType ListType);
   extern "C" DllImport uCalcPtr StdCall ListFormat_264(void *  , const char *, size_t Separator , const char *, size_t Prefix, const char *, size_t Postfix, const char *, size_t UserFunc, const char *, size_t Element, const char *, size_t Index, const char *, size_t Count);
   extern "C" DllImport uCalcPtr StdCall ListSeparator_265(void *  , const char *, size_t Separator);
   extern "C" DllImport uCalcPtr StdCall ListPrefix_266(void *  , const char *, size_t Prefix);
   extern "C" DllImport uCalcPtr StdCall ListPostfix_267(void *  , const char *, size_t Postfix);
   extern "C" DllImport uCalcPtr StdCall ListFunction_268(void *  , const char *, size_t UserFunc , const char *, size_t Element, const char *, size_t Index, const char *, size_t MatchCount);
   extern "C" DllImport uCalcPtr StdCall Populate_269(void *  , double first , double last , const char *, size_t expression , double Step, const char *, size_t Condition, const char *, size_t VarIndex, const char *, size_t VarFirst, const char *, size_t VarLast);
   extern "C" DllImport uCalcPtr StdCall Match_270(void *  , size_t Index);
   extern "C" DllImport size_t StdCall MatchIndexAtChar_271(void *  , size_t Index);
   extern "C" DllImport size_t StdCall MatchIndexAtToken_272(void *  , size_t Index);
   extern "C" DllImport uCalcPtr StdCall NonMatches_273(void * );
   extern "C" DllImport size_t StdCall MatchCount_274(void * );
   extern "C" DllImport uCalcPtr StdCall Transform_275(void *  , void * tt);
   extern "C" DllImport void StdCall Release_276(void * );
   extern "C" DllImport void * StdCall Handle_277(void * );
   extern "C" DllImport uCalcPtr StdCall AddErrorHandler_278(void *  , uCalcCallback ErrorHandlerAddress , size_t Position);
   extern "C" DllImport uCalcPtr StdCall Create_279(void *  , void * InheritsFrom);
   extern "C" DllImport uCalcPtr StdCall CreateTempDefault_280(void *  , void * InheritsFrom);
   extern "C" DllImport uCalcPtr StdCall ReleaseTempDefault_281(void * );
   extern "C" DllImport uCalcPtr StdCall Alias_282(void *  , const char *, size_t NewSymbol , const char *, size_t ExistingSymbol);
   extern "C" DllImport uCalcPtr StdCall Alias_283(void *  , const char *, size_t NewSymbol , void * ExistingItem);
   extern "C" DllImport void StdCall AliasForDefine_284(void *  , const char *, size_t NewCommandName , const char *, size_t ExistingCommandName);
   extern "C" DllImport uCalcPtr StdCall DataTypeOf_285(void *  , const char *, size_t expression);
   extern "C" DllImport uCalcPtr StdCall DataTypeOf_286(void *  , BuiltInType Index);
   extern "C" DllImport uCalcPtr StdCall Define_287(void *  , const char *, size_t expression , void * Address, void * dType);
   extern "C" DllImport uCalcPtr StdCall DefineConstant_288(void *  , const char *, size_t Definition , void * VariableAddress);
   extern "C" DllImport uCalcPtr StdCall DefineFunction_289(void *  , const char *, size_t Definition , uCalcCallback FunctionAddr);
   extern "C" DllImport uCalcPtr StdCall DefineOperator_290(void *  , const char *, size_t Definition , size_t PrecedenceLevel , Grouping Direction, uCalcCallback FunctionAddr);
   extern "C" DllImport uCalcPtr StdCall DefineVariable_291(void *  , const char *, size_t Definition , void * VariableAddress);
   extern "C" DllImport const char * StdCall ErrorExpression_292(void * , size_t & );
   extern "C" DllImport void StdCall ErrorExpression_293(void *  , const char *, size_t expression);
   extern "C" DllImport size_t StdCall ErrorLocation_294(void * );
   extern "C" DllImport const char * StdCall ErrorMessage_295(void * , size_t & , ErrorMsg Index);
   extern "C" DllImport ErrorMsg StdCall ErrorNumber_296(void * );
   extern "C" DllImport void StdCall ErrorResponse_297(void *  , ErrorHandlerResponse Response);
   extern "C" DllImport const char * StdCall ErrorSymbol_298(void * , size_t & );
   extern "C" DllImport double StdCall Eval_299(void *  , const char *, size_t expression);
   extern "C" DllImport const char * StdCall EvalStr_300(void * , size_t & , const char *, size_t expression);
   extern "C" DllImport void StdCall RaiseErrorOnDivideByZero_301(void *  , int32_t TrueOrFalse);
   extern "C" DllImport void StdCall RaiseErrorOnInexact_302(void *  , int32_t TrueOrFalse);
   extern "C" DllImport void StdCall RaiseErrorOnUnderflow_303(void *  , int32_t TrueOrFalse);
   extern "C" DllImport void StdCall RaiseErrorOnOverflow_304(void *  , int32_t TrueOrFalse);
   extern "C" DllImport void StdCall RaiseErrorOnInvalid_305(void *  , int32_t TrueOrFalse);
   extern "C" DllImport int32_t StdCall FloatingPointErrorsToCatch_306(void *  , int32_t Value);
   extern "C" DllImport uCalcPtr StdCall ItemOf_307(void *  , const char *, size_t ItemName , int64_t Properties);
   extern "C" DllImport uCalcPtr StdCall ItemOf_308(void *  , ItemIs Property_ , size_t nth , const char *, size_t DataTypeName);
   extern "C" DllImport uCalcPtr StdCall NamespaceSymbol_309(void *  , size_t nth);
   extern "C" DllImport size_t StdCall NamespaceSymbolCount_310(void * );
   extern "C" DllImport uCalcPtr StdCall NewString_311(void *  , const char *, size_t InitialString);
   extern "C" DllImport uCalcPtr StdCall NewTransform_312(void *  , void * TokensToInherit);
   extern "C" DllImport uCalcPtr StdCall ExpressionTransforms_313(void * );
   extern "C" DllImport uCalcPtr StdCall Parse_314(void *  , const char *, size_t expression , void * DataType);
   extern "C" DllImport uCalcPtr StdCall Parse_315(void *  , const char *, size_t expression , const char *, size_t DataType);
   extern "C" DllImport uCalcPtr StdCall Parse_316(void *  , const char *, size_t expression , BuiltInType DataTypeIndex);
   extern "C" DllImport size_t StdCall Precedence_317(void *  , const char *, size_t Operator1 , const char *, size_t Operator2);
   extern "C" DllImport int64_t StdCall Properties_318(void *  , ItemIs p1 , ItemIs p2, ItemIs p3, ItemIs p4);
   extern "C" DllImport uCalcPtr StdCall DefaultItem_319(void * );
   extern "C" DllImport uCalcPtr StdCall DefaultDataType_320(void * );
   extern "C" DllImport void StdCall DefaultDataType_321(void *  , void * dType);
   extern "C" DllImport void StdCall DefaultDataType_322(void *  , BuiltInType DataTypeIndex);
   extern "C" DllImport void StdCall DefaultDataType_323(void *  , const char *, size_t dTypeStr);
   extern "C" DllImport int32_t StdCall IsDefault_324(void * );
   extern "C" DllImport void StdCall IsDefault_325(void *  , int32_t Condition);
   extern "C" DllImport uCalcPtr StdCall Default_326(void * );
   extern "C" DllImport void StdCall Default_327(void *  , void * uCalcObj);
   extern "C" DllImport uCalcPtr StdCall Format_328(void *  , uCalcCallback FunctionAddr , const char *, size_t Properties);
   extern "C" DllImport uCalcPtr StdCall Format_329(void *  , uCalcCallback FunctionAddr , void * dType);
   extern "C" DllImport uCalcPtr StdCall Format_330(void *  , const char *, size_t Definition , void * dType);
   extern "C" DllImport void StdCall FormatRemove_331(void *  , void * dType);
   extern "C" DllImport const char * StdCall ValueAt_332(void * , size_t & , void * ValuePtr , void * dType);
   extern "C" DllImport const char * StdCall ValueAt_333(void * , size_t & , void * ValuePtr , const char *, size_t ValueDataType);
   extern "C" DllImport void StdCall License_334(void *  , LicenseType Type , size_t L1, size_t L2, size_t L3, size_t L4, size_t L5);
   extern "C" DllImport void StdCall Release_335(void * );
   extern "C" DllImport void * StdCall Handle_336(void * );
   
   //Product: Fast Math Parser
   
   /// <summary> uCalc main class</summary> 
   struct  uCalc {
      
      // CS ImportStatements
      
      struct Expression;
      struct Item;
      struct Callback;
      struct DataType;
      struct Transforms;
      struct Rule;
      struct Tokens;
      struct String;
      struct Matches;
      struct Base;
      
      /// <summary> Base class</summary> 
      struct  Base {
         
         /// <summary> Handle for Base Class</summary> 
         uCalcPtr hb;
         
         /// <summary> Returns the name of the uCalc Item</summary> 
         /// <remarks>
         /// <para>Items such as functions, operators, variables, etc generally are given a name when they are defined.
         ///          That name can be retried with this function.</para>
         /// </remarks> 
         string Name() {
            size_t ReturnStrSize;
            const char * ReturnStr = Name_1((void *)hb, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Returns the text data associated with a uCalc.Item</summary> 
         /// <remarks>
         /// <para>Certain items may have a text property that can be retrieved with a call to this function.  The literal text
         ///          used to define an item is usually stored here.
         /// </para>
         /// </remarks> 
         string Text() {
            size_t ReturnStrSize;
            const char * ReturnStr = Text_2((void *)hb, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Next uCalc Item object that shares the same name</summary> 
         /// <returns> Returns a uCalc Item object, or a null (0) if there is none</returns>
         /// <remarks>
         /// <para>If several functions, operators, parameters, etc are overloaded -- that is they are defined with different
         ///          parameters but share the same name -- or if they (or variables) overshadow each other, this function returns
         ///          the next uCalc.Item object that shares the same name.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item NextOverload() { return NextOverload_3((void *)hb);          }
         
         /// <summary> Documentation/help/comment/tooltip type info associated with an item</summary> 
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///          Each item can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file system, or as a tool tip.
         /// </para>
         /// </remarks> 
         string Comment() {
            size_t ReturnStrSize;
            const char * ReturnStr = Comment_4((void *)hb, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Sets documentation/help/comment/tooltip type info for an item</summary> 
         /// <param name="CommentStr"> Text of the comment</param>
         /// <returns> Rule</returns>
         /// <remarks>
         /// <para>
         ///          Each item can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file system, or as a tool tip.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule Comment(const string & CommentStr) { return Comment_5((void *)hb, CommentStr.data(), CommentStr.size());          }
         
         /// <summary> Data type of a uCalc Item</summary> 
         /// <returns> DataType object</returns>
         /// <remarks>
         /// <para>The returned data type object can be used to determine the type name, size, etc.</para>
         /// </remarks> 
         uCalcSoftware::uCalc::DataType DataType() { return DataType_6((void *)hb);          }
         
         /// <summary> Returns the uCalc object this object belongs to</summary> 
         /// <returns> uCalc object</returns>
         /// <remarks>
         /// <para>If your transform inovlves calcuations, then they will be performed using the
         ///          uCalc object returned by this function.  Each uCalc object can have its own sets of
         ///          variables, functions, and other definitions.</para>
         /// <para>         A uCalc.Item is defined in the context of a uCalc object.  You can later use this function to determine
         ///          the uCalc object the defined item belongs to.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc uCalc() { return uCalc_7((void *)hb);          }
      };
      
      /// <summary> Class for handling uCalc items</summary> 
      struct  Item : Base {
         
         /// <summary> Handle for a uCalc Item</summary> 
         uCalcPtr h;
         
         /// <summary> Sets a given property</summary> 
         /// <param name="ItemProperty"> The property you want to set</param>
         /// <param name="TrueOrFalse"> Value to set the property to</param>
         /// <returns> Returns a uCalc.Item</returns>
         /// <remarks>
         /// <para>
         ///          Properties are selected from the ItemIs enum
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item IsProperty(ItemIs ItemProperty ,bool TrueOrFalse) { return IsProperty_8((void *)h, ItemProperty , TrueOrFalse);          }
         
         /// <summary> Determines if a uCalc Item has the given properties</summary> 
         /// <param name="ItemProperty"> The property you want to check for</param>
         /// <returns> Returns Boolean True if the uCalc Item matches the given property, or False otherwise.</returns>
         /// <remarks>
         /// <para>This helps you determine whether the uCalc Item is a function, operator, variable, data type,
         /// literal value, etc, or has certain other special properties.  Note: Some items may have multiple
         /// properties.  For instance the Function property may be set to true for arrays.
         /// </para>
         /// </remarks> 
         bool IsProperty(ItemIs ItemProperty) { return IsProperty_9((void *)h, ItemProperty);          }
         
         /// <summary> Returns the number of elements in a uCalc Item</summary> 
         /// <remarks>
         /// <para>Depending on the kind of object, this may represent the number of elements in an array, the number of parameters in a function,
         ///          the number of operands in an operator, the number of parameters in a syntax construct, the number of parts in a transform,
         ///          the number of parts in a compound token, or the value explicitlely set in the Count property when calling Define.</para>
         /// <para>         When determining the number of elements, optional parameters in a function count as an element.  If the function
         ///          has a variable number of arguments, then count is -1.</para>
         /// <para>         The value returned for a syntax construct may not seem obvious, especially if it has optional and/or alternative
         ///          parts or some other complexity.
         /// </para>
         /// </remarks> 
         size_t Count() { return Count_10((void *)h);          }
         
         /// <summary> Associates another token with this one as the second part of a bracket pair</summary> 
         /// <param name="BracketToken"> Bracket token</param>
         /// <returns> Same token item that was passed as an argument</returns>
         /// <remarks>
         /// <para>
         ///          Bracket tokens are defined in pairs.  For instance, if you define "[" as a bracket token, you will want to define "]" as a bracket
         ///          match token.  "(" typically goes with bracket matching token ")", "{" with "}" etc.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item BracketMatch(Item BracketToken) { return BracketMatch_11((void *)h, (void *)BracketToken.h );          }
         
         /// <summary> Returns the bracket token that matches this token.</summary> 
         /// <returns> Returns the bracket token that matches this token.</returns>
         /// <remarks>
         /// <para>
         ///          Bracket tokens are defined in pairs.  For instance, "[" typically would have "]" as a bracket
         ///          match token.  "(" typically goes with bracket matching token ")", "{" with "}" etc.  Use this to retrieve the
         ///          matching bracket token item.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item BracketMatch() { return BracketMatch_12((void *)h);          }
         
         /// <summary> Sets the data type of a uCalc Item</summary> 
         /// <param name="dType"> Data type you want to assign to this item</param>
         /// <returns> Returns the current item</returns>
         /// <remarks>
         /// <para>
         ///          This function is intended for use with DefaultItem() or tokens.</para>
         /// <para>         If you want to define a default type other than Double precision for math parser operations, then use this on DafaultItem().</para>
         /// <para>         Some tokens, such as literal strings, or numeric values, generally require an underlying data type.</para>
         /// <para>         The data type for other items is usually defined at the same time the item itself is defined, instead of after
         ///          the fact with this function.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item DataType(uCalc::DataType dType) { return DataType_13((void *)h, (void *)dType.h );          }
         
         using Base::DataType;
         
         /// <summary> Returns the callback address of a defined function (or operator)</summary> 
         /// <returns>  Returns the callback address of a defined function (or operator)</returns>
         /// <remarks>
         /// <para>The returned value is the actual pointer address of the function as used by uCalc internally, and not
         ///          a delegate, even in .NET.  As such, in .NET this value can be used in the more general Define function but
         ///          not functions like DefineFunction or DefineneOperator.</para>
         /// </remarks> 
         uCalcCallback FunctionAddress() { return FunctionAddress_14((void *)h);          }
         
         /// <summary> Retrieves the regex pattern assigned to this token type</summary> 
         /// <returns> String</returns>
         /// <remarks>
         /// <para>
         ///          Tokens are defined with a string that represents the regex for the token.  This function can return that string.
         /// </para>
         /// </remarks> 
         string Regex() {
            size_t ReturnStrSize;
            const char * ReturnStr = Regex_15((void *)h, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Sets a new regex pattern for this token</summary> 
         /// <param name="Pattern"> Regex pattern</param>
         /// <param name="Grammar"> Type of RegEx grammar</param>
         /// <returns> Returns the current Item</returns>
         /// <remarks>
         /// <para>
         ///          Typically a regex pattern is assigned to a token item at the same time the token item is defined.  However, the
         ///          regex pattern and/or type of regex grammar can later be changed.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item Regex(const string & Pattern ,RegExGrammar Grammar  = RegExGrammar::Default) { return Regex_16((void *)h, Pattern.data(), Pattern.size() , Grammar);          }
         
         /// <summary> Returns the precedence level of the uCalc.Item</summary> 
         /// <returns> Returns the precedence level of the uCalc.Item</returns>
         /// <remarks>
         /// <para>Operators, and some syntax constructs are defined with a precedence level.  When defining a new operator you
         ///          may sometimes want to give it the same level as a pre-existing operator, which can be determined with this function.</para>
         /// </remarks> 
         size_t Precedence() { return Precedence_17((void *)h);          }
         
         /// <summary> Sets the precedence level of the uCalc Item</summary> 
         /// <param name="Level"> Precedence level you want to set for this item</param>
         /// <remarks>
         /// <para>
         ///          Operators, and some syntax constructs are defined with a precedence level.  The actual level is an arbitrary numeric
         ///          value that derives its meaning based on whether this value is greater or less than other operators.</para>
         /// <para>         For use mainly with DefaultItem.  Precedence levels are usually set at the time an item is defined instead of using
         ///          this function.
         /// </para>
         /// </remarks> 
         void Precedence(size_t Level) { Precedence_18((void *)h, Level);          }
         
         /// <summary> Renames an item</summary> 
         /// <param name="NewName"> The new name of the item</param>
         /// <returns> Returns this object</returns>
         /// <remarks>
         /// <para>
         /// Any defined item that has a name (such as a variable, constant, function, operator, etc) can be renamed with this function.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item Rename(const string & NewName) { return Rename_19((void *)h, NewName.data(), NewName.size());          }
         
         /// <summary> Returns the numeric value associated with a uCalc.Item</summary> 
         /// <returns> A double precision value</returns>
         /// <remarks>
         /// <para>Use this to retrieve the Double precision value associated with a variable.  Other functions are provided for items
         ///          that have another data type such as 16-bit integer, 32-bit integer, string, etc.
         /// </para>
         /// </remarks> 
         double Value() { return Value_20((void *)h);          }
         
         /// <summary> Sets the value of a floating point variable</summary> 
         /// <param name="Value"> Double precision floating point value that the variable will be set to</param>
         /// <remarks>
         /// <para>When a floating point variable is defined with DefineVariable, you can later set a value for this variable by passing an
         ///            argument to Value.  Note: If your compiler supports pointers, you can define a uCalc variable that is directly attached
         ///            to the address of a variable in your source code, and not have to use Value.
         /// </para>
         /// </remarks> 
         void Value(double Value) { Value_21((void *)h, Value);          }
         
         /// <summary> Sets the value of a variable of any data type by passing a string</summary> 
         /// <param name="Value"> String containing the value that will be converted and stored in the variable</param>
         /// <remarks>
         /// <para>This lets you set the value of a variable defined with DefineVariable (or Define) regardless of the variable's data type.
         ///          The value is passed as a string, and internally converted to the appropriate data type.  Note: Because a conversion takes
         ///          place with ValueAny, it is typically slower than calling Value for floating point, or one of the
         ///          other versions of Value specific to a data type.</para>
         /// </remarks> 
         void Value(const string & Value) { Value_22((void *)h, Value.data(), Value.size());          }
         
         /// <summary> Returns the Boolean value associated with a uCalc.Item</summary> 
         /// <returns> Returns a Boolean value</returns>
         /// <remarks>
         /// <para>Use this to retrieve the Boolean value associated with a variable.</para>
         /// </remarks> 
         bool ValueBool() { return ValueBool_23((void *)h);          }
         
         /// <summary> Sets the value of a Boolean variable</summary> 
         /// <param name="Value"> Boolean value that the variable will be set to</param>
         /// <remarks>
         /// <para>This lets you set the value of a Boolean variable defined with DefineVariable (or Define)</para>
         /// </remarks> 
         void ValueBool(bool Value) { ValueBool_24((void *)h, Value);          }
         
         /// <summary> Returns the uint8_t value associated with a uCalc.Item</summary> 
         /// <returns> Returns an 8-bit number</returns>
         /// <remarks>
         /// <para>Use this to retrieve the uint8_t value associated with a variable.</para>
         /// </remarks> 
         uint8_t ValueByte() { return ValueByte_25((void *)h);          }
         
         /// <summary> Sets the value of a uint8_t variable</summary> 
         /// <param name="Value"> uint8_t value that the variable will be set to</param>
         /// <remarks>
         /// <para>This lets you set the value of a uint8_t variable defined with DefineVariable (or Define)</para>
         /// </remarks> 
         void ValueByte(uint8_t Value) { ValueByte_26((void *)h, Value);          }
         
         /// <summary> Returns the 16-bit integer value associated with a uCalc.Item</summary> 
         /// <returns> Returns a 16-bit integer</returns>
         /// <remarks>
         /// <para>Use this to retrieve the Int16 value associated with a variable.</para>
         /// </remarks> 
         int16_t ValueInt16() { return ValueInt16_27((void *)h);          }
         
         /// <summary> Sets the value of a 16-bit integer variable</summary> 
         /// <param name="Value"> 16-bit integer value that the variable will be set to</param>
         /// <remarks>
         /// <para>This lets you set the value of a 16-bit integer variable defined with DefineVariable (or Define).  This works for
         ///          both signed and unsigned 16-bit integers.</para>
         /// </remarks> 
         void ValueInt16(int16_t Value) { ValueInt16_28((void *)h, Value);          }
         
         /// <summary> Returns the 32-bit integer value associated with a uCalc.Item</summary> 
         /// <returns> Returns a 32-bit integer</returns>
         /// <remarks>
         /// <para>Use this to retrieve the Int32 value associated with a variable.</para>
         /// </remarks> 
         int32_t ValueInt32() { return ValueInt32_29((void *)h);          }
         
         /// <summary> Sets the value of a 32-bit integer variable</summary> 
         /// <param name="Value"> 32-bit integer value that the variable will be set to</param>
         /// <remarks>
         /// <para>This lets you set the value of a 32-bit integer variable defined with DefineVariable (or Define).  This works for
         ///          both signed and unsigned 32-bit integers.</para>
         /// </remarks> 
         void ValueInt32(int32_t Value) { ValueInt32_30((void *)h, Value);          }
         
         /// <summary> Returns the 64-bit integer value associated with a uCalc.Item</summary> 
         /// <returns> Returns a 64-bit value</returns>
         /// <remarks>
         /// <para>Use this to retrieve the Int64 value associated with a variable.</para>
         /// </remarks> 
         int64_t ValueInt64() { return ValueInt64_31((void *)h);          }
         
         /// <summary> Sets the value of a 64-bit integer variable</summary> 
         /// <param name="Value"> 64-bit integer value that the variable will be set to</param>
         /// <remarks>
         /// <para>This lets you set the value of a 64-bit integer variable defined with DefineVariable (or Define).  This works for
         ///          both signed and unsigned 64-bit integers.</para>
         /// </remarks> 
         void ValueInt64(int64_t Value) { ValueInt64_32((void *)h, Value);          }
         
         /// <summary> Returns the Single precision value associated with a uCalc.Item</summary> 
         /// <returns> Returns a Single precision value</returns>
         /// <remarks>
         /// <para>Use this to retrieve the Single precision value associated with a variable.</para>
         /// </remarks> 
         float ValueSng() { return ValueSng_33((void *)h);          }
         
         /// <summary> Sets the value of a single precision floating point variable</summary> 
         /// <param name="Value"> Single precision value that the variable will be set to</param>
         /// <remarks>
         /// <para>This lets you set the value of a single precision variable defined with DefineVariable (or Define).</para>
         /// </remarks> 
         void ValueSng(float Value) { ValueSng_34((void *)h, Value);          }
         
         /// <summary> Returns the Double precision value associated with a uCalc.Item</summary> 
         /// <returns> Returns a double precision value</returns>
         /// <remarks>
         /// <para>Synonymous with Value</para>
         /// </remarks> 
         double ValueDbl() { return ValueDbl_35((void *)h);          }
         
         /// <summary> Sets the value of a double precision floating point variable</summary> 
         /// <param name="Value"> Double precision floating point value that the variable will be set to</param>
         /// <remarks>
         /// <para>ValueDbl is synonymous with Value</para>
         /// </remarks> 
         void ValueDbl(double Value) { ValueDbl_36((void *)h, Value);          }
         
         /// <summary> Returns the String value associated with a uCalc.Item</summary> 
         /// <returns> Returns a string</returns>
         /// <remarks>
         /// <para>Use this to retrieve the String value associated with a variable.</para>
         /// <para>         ValueStr has a special property beyond the Value function for other data types in that it can return the value
         ///          of an item regardless of what data type it has.  The value is returned as a string, after being converted if necessary.
         /// </para>
         /// </remarks> 
         string ValueStr() {
            size_t ReturnStrSize;
            const char * ReturnStr = ValueStr_37((void *)h, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Sets the value of a string variable</summary> 
         /// <param name="Value"> String value that the variable will be set to</param>
         /// <remarks>
         /// <para>This lets you set the value of a string variable defined with DefineVariable (or Define).</para>
         /// </remarks> 
         void ValueStr(const string & Value) { ValueStr_38((void *)h, Value.data(), Value.size());          }
         
         /// <summary> Returns the Pointer value associated with a uCalc.Item</summary> 
         /// <remarks>
         /// <para>Use this to retrieve the Pointer value associated with a variable.
         ///          Note: If the uCalc.Item is a Variable of type Pointer, then ValuePtr returns a pointer value
         ///          that is stored in that variable, whereas ValueAddr returns the Pointer of the variable itself.  In other words
         ///          ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns any value you may
         ///          have set as the address the variable points to.</para>
         /// </remarks> 
         void * ValuePtr() { return ValuePtr_39((void *)h);          }
         
         /// <summary> Sets the value of a pointer variable</summary> 
         /// <param name="Value"> Pointer that the variable will be set to</param>
         /// <remarks>
         /// <para>This lets you set the value of a pointer variable defined with DefineVariable (or Define).  Note: This does not
         ///          change the pointer of the variable.  It simply stores a pointer value in the variable.</para>
         /// </remarks> 
         void ValuePtr(void * Value) { ValuePtr_40((void *)h, Value);          }
         
         /// <summary> Sets the value of a variable to a value who's Pointer is passed</summary> 
         /// <param name="Value"> Pointer to the value that the variable will be set to</param>
         /// <remarks>
         /// <para>This lets you set the value of a variable by passing the pointer to the value you want to set the variable to.</para>
         /// </remarks> 
         void ValueByPtr(void * Value) { ValueByPtr_41((void *)h, Value);          }
         
         /// <summary> Returns the Pointer address of the value associated with a uCalc.Item</summary> 
         /// <returns> Pointer (or Integer)</returns>
         /// <remarks>
         /// <para>Use this to retrieve the Pointer address of the value associated with a variable (instead of
         ///          the value itself).  Note: If the uCalc.Item is a Variable of type Pointer, then ValuePtr returns a pointer value
         ///          that is stored in that variable, whereas ValueAddr returns the Pointer of the variable itself.  In other words
         ///          ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns any given address
         ///          you may have stored as the value of the variable.</para>
         /// </remarks> 
         void * ValueAddr() { return ValueAddr_42((void *)h);          }
         
         Item() { h = nullptr; hb = h;          }
         Item(const string & Definition) { h = Default().Define(Definition).h;          }
         
         /// <summary> Releases an Item defined with Define or other related function</summary> 
         /// <remarks>
         /// <para>When a function, operator, variable, or other item is defined with such functions as
         ///          Define, DefineFunction, DefineOperator, DefineVariable, these items remain in memory, until they are
         ///          released with this function.  Items should generally be released when they are no longer needed, in order
         ///          to preserve memory.</para>
         /// <para>         Note: A uCalc Item object must explicitely be released with Release unless the entire uCalc
         ///          object it is a part of is released.  An Item object will not be released automatically when it
         ///          simply goes out of scope.
         /// </para>
         /// </remarks> 
         void Release() { if ((void *)h != nullptr) Release_43((void *)h); h = nullptr;          }
         
         /// <summary> Returns the handle of an object</summary> 
         /// <remarks>
         /// <para>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload
         ///    (when applicable), you may check Handle() to see when it becomes NULL (or 0) to know when to stop.</para>
         /// </remarks> 
         void * Handle() { return Handle_44((void *)h);          }
         
         Item(uCalcPtr PassedHandle) { h = PassedHandle; hb = h;          }
      };
      
      /// <summary> Class for evaluating an already parsed expression or executing parsed code</summary> 
      struct  Expression {
         
         /// <summary> Handle for a uCalc expression</summary> 
         uCalcPtr h;
         
         // IMPORTANT: must have at least 2 blank lines between each EXPORT (for proper transform)
         
         /// <summary> Rapidly evaluates a floating point expression that was parsed with Parse() or passed ByExpression</summary> 
         /// <returns> Double precision numeric result of the evaluated expression</returns>
         /// <remarks>
         /// <para>Evaluate can be used instead of Eval for faster results in time-critical loops where the same expression
         ///          is repeatedly re-evaluated, with the only change in each iteration typically being the value(s) of one or
         ///          more variables.  Evaluate works in pair with Parse, the relatively slower part, which is called first before
         ///          going into the time-critical loop.  The Expression object returned by Parse is what calls the Evaluate function.</para>
         /// <para>         Although Evaluate() is similar to EvaluateDbl(), it is slightly different.  Both return a double precision value,
         ///          however, EvaluateDbl() can only return the value of a double precision expression, whereas Evaluate() can return the value
         ///          of other numeric types (single precision, various integers, Boolean), in which case the value is first converted to Double
         ///          precision before being returned.  Evaluate() is less prone to data type mixups.  On the other hand when a conversion
         ///          does take place, there is a slight speed penalty.</para>
         /// <para>         Evaluate can also be called by an Expression object that was passed to a callback ByExpression.</para>
         /// <para>         If the expression evaluates to something other than Double precision floating point (such as various integers, complex,
         ///          single precision, etc), the value is converted before being returned.  In some cases (such as string, complex, etc), the
         ///          return result may or may not have a meaningful value.  However, an error is not raised.
         /// </para>
         /// </remarks> 
         double Evaluate() { return Evaluate_45((void *)h);          }
         
         /// <summary> Rapidly evaluates a floating point expression that was parsed with Parse or passed ByExpression</summary> 
         /// <returns> The result of the evaluated floating point expression</returns>
         /// <remarks>
         /// <para>EvaluateDbl is similar to Evaluate, but is intended strictly for expressions that return a Double
         ///          precision value (determined either by the operators used in the expression, a default double precision type, or an explicit
         ///          double precision).  It will not work with Integers or Single precision expressions for instance.</para>
         /// <para>         If the expression evaluates to a non-numeric value (such as a string) or non-scalar numeric value (such as a complex number), or if a numeric
         ///          type other than the default was explicitely specified with Parse, the value returned will be invalid.  However, an error will not be raised.
         /// </para>
         /// </remarks> 
         double EvaluateDbl() { return EvaluateDbl_46((void *)h);          }
         
         /// <summary> Rapidly evaluates expression that was parsed as an integer with Parse or passed ByExpression</summary> 
         /// <returns> Returns a 32-bit integer; the result of the evaluated expression</returns>
         /// <remarks>
         /// <para>
         ///          This is similar to Evaluate, but applies only to expressions that return a 32-bit integer.
         /// </para>
         /// </remarks> 
         int32_t EvaluateInt32() { return EvaluateInt32_47((void *)h);          }
         
         /// <summary> Rapidly evaluates an expression that was parsed as Boolean with Parse or passed ByExpression</summary> 
         /// <returns> The result of the evaluated Boolean expression</returns>
         /// <remarks>
         /// <para>
         ///          This is similar to Evaluate, but applies only to expressions that return a Boolean.
         ///          Certain operators, such as comparison operators like <, >, ==, etc. return a Boolean.  If the final operation of an
         ///          expression is Boolean, for instance in Parse("x + 1 > 3"), then you can use EvaluateBool.  In earlier versions
         ///          Evaluate could not be used for types other than floating point, unless Double was specified in Parse so that a
         ///          conversion could take place (from Bool in this case) to Double before being returned.  Now you can either use
         ///          EvaluateBool (slightly more efficient) or else Evaluate as a catch all for any type including Boolean.
         /// </para>
         /// </remarks> 
         bool EvaluateBool() { return EvaluateBool_48((void *)h);          }
         
         /// <summary> Rapidly evaluates an expression of any data type that was parsed with Parse or passed ByExpression returning a string</summary> 
         /// <returns> Returns a string which is the result of the evaluated expression</returns>
         /// <remarks>
         /// <para>EvaluateStr is similar to Evaluate.  However, EvaluateStr returns a value as a string.  Unlike Evaluate, which can only
         ///          return a floating point value, EvaluateStr can return a value of any data type.  Non-string values are converted to a string
         ///          before being returned.
         /// </para>
         /// </remarks> 
         string EvaluateStr() {
            size_t ReturnStrSize;
            const char * ReturnStr = EvaluateStr_49((void *)h, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Rapidly evaluates an expression that was parsed with Parse or passed ByExpression and returns a Pointer to the result</summary> 
         /// <returns> Returns a pointer to the result instead of the result itself</returns>
         /// <remarks>
         /// <para>Use this when you are interested in a pointer to the result instead of the result itself.  For instance, given an integer
         ///          result, you might use a pointer in order to retrieve the hi uint8_t and lo uint8_t values.
         /// </para>
         /// </remarks> 
         void * EvaluateVoid() { return EvaluateVoid_50((void *)h);          }
         
         /// <summary> Rapidly executes code that was parsed with Parse() or passed ByExpression for which a result is not required</summary> 
         /// <remarks>
         /// <para>This can be used in cases where the reutrn value is not needed, such as statements that represent code to be executed
         ///          rather than a math expression.
         /// </para>
         /// </remarks> 
         void Execute() { Execute_51((void *)h);          }
         
         /// <summary> Returns the data type of a parsed expression</summary> 
         /// <returns> DataType of the expression</returns>
         /// <remarks>
         /// <para>
         ///          Every parsed expression has a return type, which may be set explicity, or deduced implicitly based on the context; for instance comparison operations would return
         ///          a Boolean, text within quotes would return as a string, and dividing two numeric values would return a Double precision floating point.  Sometimes it might not
         ///          be clear what data type an expression has.  This function can return the data type.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::DataType DataType() { return DataType_52((void *)h);          }
         
         Expression() { h = nullptr;          }
         
         /// <summary> Releases an expression that was parsed with Parse</summary> 
         /// <remarks>
         /// <para>Whenever an expression is parsed with Parse, the parsed expression remains in memory until released.
         /// When the parsed expression is no longer needed it should be released with Release to preserve memory.
         /// Note: An expression parsed with Parse must explicitely be released with Release unless the uCalc
         /// object it is in is released.  A parsed expression will not be released automatically when it simply
         /// goes out of scope.
         /// </para>
         /// </remarks> 
         void Release() { if ((void *)h != nullptr) Release_53((void *)h); h = nullptr;          }
         
         /// <summary> Returns the handle of an object</summary> 
         /// <remarks>
         /// <para>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload
         ///    (when applicable), you may check Handle() to see when it becomes NULL (or 0) to know when to stop.</para>
         /// </remarks> 
         void * Handle() { return Handle_54((void *)h);          }
         
         Expression(uCalcPtr PassedHandle) { h = PassedHandle;          }
      };
      
      /// <summary> Class for retrieving arguments passed to a callback, returning a value, and more</summary> 
      struct  Callback {
         
         /// <summary> Handle of the expression part the current callback instance is refering to</summary> 
         uCalcPtr h;
         
         /// <summary> Returns the value of an argument passed to a uCalc.Callback function</summary> 
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <returns> Returns a Double</returns>
         /// <remarks>
         /// <para>When a callback function is defined with DefineFunction or DefineOperator, that
         ///          callback function will receive arguments that uCalc passes to it through Arg().  Arg() handles values
         ///          of the default Double floating point data type.  For other data types, use ArgInt32, ArgStr, etc instead.</para>
         /// <para>         Note that Index is 1-based.  So if there are 3 arguments, the Index of the first will be
         ///          1 (not 0), and the second and third will be 2 and 3 respectively.</para>
         /// <para>         If you are defining a binary operator, 1 is the index for the left operand, and 2 is for the right operand.
         ///          For a unary operator, the index is 1.
         /// </para>
         /// </remarks> 
         double Arg(size_t Index) { return Arg_55((void *)h, Index);          }
         
         /// <summary> Returns the value of an argument passed to a uCalc.Callback function</summary> 
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>
         /// <para>When a callback function is defined with DefineFunction or DefineOperator, that
         ///          callback function will receive arguments that uCalc passes to it through the Arg().  Arg() handles values
         ///          of the default Double floating point data type.  For other data types, use ArgInt32, ArgStr, etc instead.</para>
         /// <para>         Note that Index is 1-based.  So if there are 3 arguments, the Index of the first will be
         ///          1 (not 0), and the second and third will be 2 and 3 respectively.</para>
         /// <para>         If you are defining a binary operator, 1 is the index for the left operand, and 2 is for the right operand.
         ///          For a unary operator, the index is 1.
         /// </para>
         /// </remarks> 
         double ArgDbl(size_t Index) { return ArgDbl_56((void *)h, Index);          }
         
         /// <summary> Returns the first argument of a callback</summary> 
         /// <returns> Returns a Double</returns>
         /// <remarks>
         /// <para>
         ///          Similar to Arg[1], but slightly more efficient.
         /// </para>
         /// </remarks> 
         double Arg1() { return Arg1_57((void *)h);          }
         /// <summary> Returns the second argument of a callback</summary> 
         /// <remarks>
         /// <para>
         ///          Similar to Arg[2], but slightly more efficient.
         /// </para>
         /// </remarks> 
         double Arg2() { return Arg2_58((void *)h);          }
         
         /// <summary> Returns the Boolean integer argument passed to a uCalc.Callback function</summary> 
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <returns> Returns a Boolean</returns>
         /// <remarks>
         /// <para>If a function parameter (or operator operand) was explicitely defined as Int32, then the callback function can
         ///          recieve it through ArgBool.  If the argument is of a different but compatible numeric type, a conversion takes
         ///          place first before being passed.</para>
         /// </remarks> 
         bool ArgBool(size_t Index) { return ArgBool_59((void *)h, Index);          }
         
         /// <summary> Returns the number of arguments being passed to a uCalc.Callback function</summary> 
         /// <returns> 0 if no arg is passed; or a positive integer representing the number of args being passed.</returns>
         /// <remarks>
         /// <para>This is useful for functions that have a variable number of arguments</para>
         /// </remarks> 
         size_t ArgCount() { return ArgCount_60((void *)h);          }
         
         /// <summary> Returns an object instance of the argument being passed to a uCalc.Callback function</summary> 
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <returns> Returns a uCalc.Item; The returned object can be used (where relevent) just like any other uCalc.Item</returns>
         /// <remarks>
         /// <para>With this object, you can do such things as determine the argument's data type, determine which instance of uCalc
         ///          it belongs to, retrieve the pointer of the value of the argument, etc.</para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item ArgObj(size_t Index) { return ArgObj_61((void *)h, Index);          }
         
         /// <summary> Returns the address of the argument value that is being passed</summary> 
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <returns> Returns a pointer</returns>
         /// <remarks>
         /// <para>In addition to Arg() which returns arguments that are of the default Double floating point numeric type, there
         ///          are specialized functions for other datatypes such as ArgInt32, ArgInt64, etc.  However, if the argument is of
         ///          a data type that does not have a specialized function, you can retrieve the address of the argument value as
         ///          a pointer, and use that to retrieve the value.</para>
         /// </remarks> 
         void * ArgAddr(size_t Index) { return ArgAddr_62((void *)h, Index);          }
         
         /// <summary> Returns the object instance of the expression that's passed as an argument to a uCalc.Callback function</summary> 
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <returns> Returns a parsed expression object with which you can call functions like Evaluate or Execute</returns>
         /// <remarks>
         /// <para>uCalc allows you to not only pass arguments by value or by reference.  But it also lets you pass an argument
         ///          by expression.  When an expression is passed ByExpression, the expresison is not evaluated before being passed.
         ///          Instead a handle to the parsed expression is passed, allowing the callback to perform the evaluation.  This is
         ///          very useful for things like "if" and "while" loop constructs etc.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Expression ArgExpr(size_t Index) { return ArgExpr_63((void *)h, Index);          }
         
         /// <summary> Returns the Int32 integer argument passed to a uCalc.Callback function</summary> 
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>
         /// <para>If a function parameter (or operator operand) was explicitely defined as Int32, then the callback function can
         ///          recieve it through ArgInt32.  If the argument is of a different but compatible numeric type, a conversion takes
         ///          place first before being passed.</para>
         /// </remarks> 
         int32_t ArgInt32(size_t Index) { return ArgInt32_64((void *)h, Index);          }
         
         /// <summary> Returns the Int64 integer argument being passed to a uCalc.Callback function</summary> 
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>
         /// <para>If a function parameter (or operator operand) was explicitely defined as Int64, then the callback function can
         ///          recieve it through ArgInt64.  If the argument is of a different but compatible numeric type, a conversion takes
         ///          place first before being passed.</para>
         /// </remarks> 
         int64_t ArgInt64(size_t Index) { return ArgInt64_65((void *)h, Index);          }
         
         /// <summary> Returns a pointer value being passed as an argument to a uCalc.Callback function</summary> 
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>
         /// <para>If a function parameter (or operator operand) was explicitely defined as a Pointer, then the callback function can
         ///          recieve it through ArgPtr.  Note that ArgPtr returns any address value that is being sent, whereas ArgAddr returns the
         ///          address of the value being passed.</para>
         /// </remarks> 
         void * ArgPtr(size_t Index) { return ArgPtr_66((void *)h, Index);          }
         
         /// <summary> Returns the value being passed as argument in to a uCalc.Callback function in the form of a string</summary> 
         /// <param name="Index"> Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>
         /// <para>Returns data of any type to a string.
         ///          In addition to returning an argument that was passed as a string, ArgStr has the special property of returning an argument that
         ///          was passed as any data type.  If the argument was not a string, ArgStr first converts it to a string before returning its value.</para>
         /// </remarks> 
         string ArgStr(size_t Index) {
            size_t ReturnStrSize;
            const char * ReturnStr = ArgStr_67((void *)h, ReturnStrSize , Index);
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Sets the return value (floating point) of a uCalc.Callback function</summary> 
         /// <param name="Value"> Floating point value that your callback returns</param>
         /// <remarks>
         /// <para>When you define a callback function with DefineFunction and you want to return a floating point number,
         ///          do it with Return.
         /// </para>
         /// </remarks> 
         void Return(double Value) { Return_68((void *)h, Value);          }
         
         /// <summary> Sets the return value (default floating point) of a uCalc.Callback function</summary> 
         /// <param name="Value"> Floating point value that your callback returns</param>
         /// <remarks>
         /// <para>When you define a callback function with DefineFunction and you want to return a floating point number,
         ///          do it with Return.</para>
         /// <para>         ReturnDbl() is identical to Return(); you may want to use this if "Return" is a reserved keyword in the compiler you're using.
         /// </para>
         /// </remarks> 
         void ReturnDbl(double Value) { ReturnDbl_69((void *)h, Value);          }
         
         /// <summary> Sets the return value of a uCalc.Callback function as an Boolean integer</summary> 
         /// <param name="Value"> Boolean value that your callback returns</param>
         /// <remarks>
         /// <para>When you define a callback function with DefineFunction and you want to return a Boolean,
         ///          do it with ReturnBool.</para>
         /// </remarks> 
         void ReturnBool(bool Value) { ReturnBool_70((void *)h, Value);          }
         
         /// <summary> Sets the return value of a uCalc.Callback function as an Int32 integer</summary> 
         /// <param name="Value"> Integer (32-bit) that your callback returns</param>
         /// <remarks>
         /// <para>When you define a callback function with DefineFunction and you want to return a Int32,
         ///          do it with ReturnInt32.</para>
         /// </remarks> 
         void ReturnInt32(int32_t Value) { ReturnInt32_71((void *)h, Value);          }
         
         /// <summary> Sets the return value of a uCalc.Callback function as an Int64 integer</summary> 
         /// <param name="Value"> Integer (64-bit) that your callback returns</param>
         /// <remarks>
         /// <para>When you define a callback function with DefineFunction and you want to return a Int64,
         ///          do it with ReturnInt64.</para>
         /// </remarks> 
         void ReturnInt64(int64_t Value) { ReturnInt64_72((void *)h, Value);          }
         
         /// <summary> Sets the return value of a uCalc.Callback function as a Pointer</summary> 
         /// <param name="Value"> Pointer value that your callback returns</param>
         /// <remarks>
         /// <para>When you define a callback function with DefineFunction that returns a pointer value,
         ///          do it with ReturnPtr.</para>
         /// </remarks> 
         void ReturnPtr(void * Value) { ReturnPtr_73((void *)h, Value);          }
         
         /// <summary> Sets the return value of a uCalc.Callback function as a String</summary> 
         /// <param name="Value"> String value that your callback returns</param>
         /// <remarks>
         /// <para>When you define a callback function with DefineFunction that returns a string,
         ///          do it with ReturnStr.</para>
         /// </remarks> 
         void ReturnStr(const string & Value) { ReturnStr_74((void *)h, Value.data(), Value.size());          }
         
         /// <summary> Sets a numerical value for an error</summary> 
         /// <param name="ErrNumber"> Index number of error to raise</param>
         /// <remarks>
         /// <para>
         ///          The index corresponds with an ErrorMsg enum.
         /// </para>
         /// </remarks> 
         void ErrorSet(ErrorMsg ErrNumber) { ErrorSet_75((void *)h, ErrNumber);          }
         
         /// <summary> Raises an error in a uCalc.Callback function</summary> 
         /// <param name="ErrNumber"> Index number of error to raise</param>
         /// <returns> Returns ErrorHandlerResponse::Abort, ErrorHandlerResponse::Resume, or ErrorHandlerResponse::ReRaise.</returns>
         /// <remarks>
         /// <para>Your callback can raise an error by calling this funciton with one of the values of the
         ///          ErrorMsg enum.  End-user function definitions can also raise an error by calling Error().
         ///          The return value is determined by error handlers that you may have added.  ErrorHandlerResponse::Abort (value of 0)
         ///          is returned by default if there are no error handlers.</para>
         /// </remarks> 
         ErrorHandlerResponse ErrorRaise(ErrorMsg ErrNumber) { return ErrorRaise_76((void *)h, ErrNumber);          }
         
         /// <summary> Raises an error with a user-defined message in a uCalc.Callback function</summary> 
         /// <param name="ErrMessage"> Text of custom error message to return</param>
         /// <returns> Returns ErrorHandlerResponse::Abort, ErrorHandlerResponse::Resume, or ErrorHandlerResponse::ReRaise.</returns>
         /// <remarks>
         /// <para>Your callback can raise a custom error message (one not listed as a ErrorMsg enum)
         ///          by simply passing the text of the message to this function.</para>
         /// </remarks> 
         ErrorHandlerResponse ErrorRaiseMessage(const string & ErrMessage) { return ErrorRaiseMessage_77((void *)h, ErrMessage.data(), ErrMessage.size());          }
         
         /// <summary> Returns the uCalc instance the current expression part belongs to</summary> 
         /// <remarks>
         /// <para>A callback is designed to work with all instances of uCalc.  However, at times you may need to know
         ///          which particular instance of uCalc the expression part being passed belongs to.</para>
         /// </remarks> 
         uCalcSoftware::uCalc uCalc() { return uCalc_78((void *)h);          }
         
         /// <summary> Returns the function or operator Item of the current expression part</summary> 
         /// <remarks>
         /// <para>
         ///          In a callback, you can not only get the arguments that were passed, but also the associated function or operator
         ///          that sent the arguments, using this function.</para>
         /// <para>         If several functions and/or operators are mapped to the same callback function name, you can use this to
         ///          let you determine which one is currently being called, based on properties you can check.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item Item() { return Item_79((void *)h);          }
         
         Callback() { h = nullptr;          }
         
         /// <summary> Releases a uCalc callback object</summary> 
         /// <remarks>
         /// <para>
         ///          Release is not meant to be called directly for Callback.
         /// </para>
         /// </remarks> 
         void Release() { if ((void *)h != nullptr) Release_80((void *)h); h = nullptr;          }
         
         /// <summary> Returns the handle of an object</summary> 
         /// <remarks>
         /// <para>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload
         ///    (when applicable), you may check Handle() to see when it becomes NULL (or 0) to know when to stop.</para>
         /// </remarks> 
         void * Handle() { return Handle_81((void *)h);          }
         
         Callback(uCalcPtr PassedHandle) { h = PassedHandle;          }
      }; // Callback
      
      /// <summary> Class for working with uCalc data types</summary> 
      struct  DataType {
         
         /// <summary> Handle for uCalc Data Type</summary> 
         uCalcPtr h;
         
         /// <summary> Returns the size (in bytes) of one unit of data for this data type</summary> 
         /// <returns> Returns an integer</returns>
         size_t ByteSize() { return ByteSize_82((void *)h);          }
         
         /// <summary> Returns the index of the data type</summary> 
         /// <returns> Returns a BuiltInType enum</returns>
         /// <remarks>
         /// <para>The return value is a BuiltInType enum value that represents the data type of the
         ///          current object.  For instance, if the data type is a string, the return value would be BuiltInType::String.
         /// </para>
         /// </remarks> 
         BuiltInType Index() { return Index_83((void *)h);          }
         
         /// <summary> Returns the uCalc.Item handle of the data type</summary> 
         /// <remarks>
         /// <para>Each data type object can also be represented as a uCalc Item object by using this return value.</para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item Item() { return Item_84((void *)h);          }
         
         /// <summary> Returns the name of the data type</summary> 
         /// <returns> Returns a string</returns>
         string Name() {
            size_t ReturnStrSize;
            const char * ReturnStr = Name_85((void *)h, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary> 
         /// <param name="ValuePtr"> Pointer of the uCalc array to be reset</param>
         /// <param name="Size"> Number of elements to reset</param>
         /// <remarks>
         /// <para>Resets all the elements of the array to the default value for that variable's data type.  This should only
         ///          be used for arrays that were explicitly allocated with uCalc's AllocArray.
         /// </para>
         /// </remarks> 
         void Reset(void * ValuePtr ,size_t Size) { Reset_86((void *)h, ValuePtr , Size);          }
         
         /// <summary> Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary> 
         /// <param name="ValuePtr"> Pointer of the uCalc scalar to be reset</param>
         /// <remarks>
         /// <para>
         ///           Resets a scalar variable to the default value for that variable's data type.
         /// </para>
         /// </remarks> 
         void Reset(void * ValuePtr) { Reset_87((void *)h, ValuePtr);          }
         
         /// <summary> Sets the value of an array at a given memory location that was allocated by AllocArray</summary> 
         /// <param name="Destination"> Pointer of array (base address) of which the element value is to be set</param>
         /// <param name="Source"> Pointer to a value of the same data type that will be copied to the element</param>
         /// <param name="Index"> Index of the array element who's value needs to be set</param>
         /// <remarks>
         /// <para>
         ///          Sets the value of an array at a given memory location that was allocated by AllocArray
         /// </para>
         /// </remarks> 
         void SetArray(void * Destination ,void * Source ,size_t Index) { SetArray_88((void *)h, Destination , Source , Index);          }
         
         /// <summary> Sets the value of a scalar at a given memory location that was allocated by AllocScalar</summary> 
         /// <param name="Destination"> Pointer of scalar who's value is to be set</param>
         /// <param name="Source"> Pointer to a value of the same data type that will be copied to the element</param>
         /// <remarks>
         /// <para>
         ///          Sets the value of a scalar at a given memory location that was allocated by AllocScalar
         /// </para>
         /// </remarks> 
         void SetScalar(void * Destination ,void * Source) { SetScalar_89((void *)h, Destination , Source);          }
         
         /// <summary> Swaps the pointers of two variables that were allocated with AllocScalar or AllocArray</summary> 
         /// <param name="ValuePtrA"> Pointer of variable 1</param>
         /// <param name="ValuePtrB"> Pointer of variable 2</param>
         /// <remarks>
         /// <para>Swaps the pointers of two variables that were allocated with AllocScalar or AllocArray</para>
         /// </remarks> 
         void SwapScalarValues(void * ValuePtrA ,void * ValuePtrB) { SwapScalarValues_90((void *)h, ValuePtrA , ValuePtrB);          }
         
         /// <summary> Retuns a string representing the value at the given address formatted for the item's data type</summary> 
         /// <param name="ValuePtr"> Pointer of the data containing the value you want to display</param>
         /// <remarks>
         /// <para>If you have a pointer to data of the given type, you can display this data as a string with this function.  Special formatting
         ///          associated with a data type is used, if such formatting was defined.  For instance, if you're dealing with a Boolean, then
         ///          you might get: True, Yes, On, etc, depending on how the formatting was configured with Format.</para>
         /// </remarks> 
         string ToString(void * ValuePtr) {
            size_t ReturnStrSize;
            const char * ReturnStr = ToString_91((void *)h, ReturnStrSize , ValuePtr);
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Retuns a formatted string of the given data type representing the value that was passed as text</summary> 
         /// <param name="Value"> String representing data of any type that is to be represented as data of the given type</param>
         /// <remarks>
         /// <para>If you pass a string, it will be converted to the given data type, and return using the formatting
         ///          for that data type.</para>
         /// </remarks> 
         string ToString(const string & Value) {
            size_t ReturnStrSize;
            const char * ReturnStr = ToString_92((void *)h, ReturnStrSize , Value.data(), Value.size());
            return string(ReturnStr, ReturnStrSize);
         }
         
         string ToString(const char * Value ) { return ToString(string(Value));          }
         
         DataType() { h = nullptr;          }
         
         /// <summary> Releases a data type</summary> 
         /// <remarks>
         /// <para>
         ///          This releases a Data type from memory.
         /// </para>
         /// </remarks> 
         void Release() { if ((void *)h != nullptr) Release_93((void *)h); h = nullptr;          }
         
         /// <summary> Returns the handle of an object</summary> 
         /// <remarks>
         /// <para>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload
         ///    (when applicable), you may check Handle() to see when it becomes NULL (or 0) to know when to stop.</para>
         /// </remarks> 
         void * Handle() { return Handle_94((void *)h);          }
         
         DataType(uCalcPtr PassedHandle) { h = PassedHandle;          }
      }; // DataType
      
      //Product: uCalc Transform
      
      /// <summary> Class for returning a list of matches</summary> 
      struct  Matches : Base {
         
         /// <summary> Handle for Matches object</summary> 
         uCalcPtr h;
         
         /// <summary> Sets a given property</summary> 
         /// <param name="ItemProperty"> The property you want to set</param>
         /// <param name="TrueOrFalse"> Value to set the property to</param>
         /// <returns> Returns the same Matches object</returns>
         /// <remarks>
         /// <para>Sets a property for a Matches object
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Matches IsProperty(ItemIs ItemProperty ,bool TrueOrFalse) { return IsProperty_95((void *)h, ItemProperty , TrueOrFalse);          }
         
         /// <summary> Returns nth match index given a starting character position </summary> 
         /// <param name="StartPos"> Starting character position of match you're interested in</param>
         /// <param name="NearestIndex"> Hint of nearest index</param>
         /// <returns> Integer representing nth index</returns>
         /// <remarks>
         /// <para>
         /// This gives you the index of the match that starts at a given character position.  For instance, given this string:
         /// "This is a test", which might contain the matches "This", "is", "a", and "test".  If you know that "is" starts at
         /// character position 5, and want to know which match it is, IndexOf(5) will let you know that "is" reprents match 1
         /// (where match 0 is "This").</para>
         /// <para>This is useful for finding the relative nth index in a list of matches, when you have more than one match lists for the same set of data and one
         /// is a subset of the other.
         /// </para>
         /// </remarks> 
         size_t IndexOf(size_t StartPos ,size_t NearestIndex  = 0) { return IndexOf_96((void *)h, StartPos , NearestIndex);          }
         
         /// <summary> Determines if an Item has the given property</summary> 
         /// <param name="ItemProperty"> The property you want to check for</param>
         /// <returns> Returns True if the Item matches the given property, or False otherwise.</returns>
         bool IsProperty(ItemIs ItemProperty) { return IsProperty_97((void *)h, ItemProperty);          }
         
         /// <summary> Returns the old ending character position of a match before it was changed</summary> 
         /// <param name="Index"> Index of match</param>
         /// <returns> Returns an integer</returns>
         /// <remarks>
         /// <para>
         ///    In a search/replace transform type of operation where string matches are changed to new strings, this function returns
         ///    what the ending character position of the match used to be before it was modified.
         /// </para>
         /// </remarks> 
         size_t OldEnd(size_t Index) { return OldEnd_98((void *)h, Index);          }
         
         /// <summary> Returns the old length of a match before it was changed</summary> 
         /// <param name="Index"> Index of match</param>
         /// <returns> Returns an integer</returns>
         /// <remarks>
         /// <para>
         ///    In a search/replace transform type of operation where string matches are changed to new strings, this function returns
         ///    what the length of the matching string used to be before it was modified.
         /// </para>
         /// </remarks> 
         size_t OldLength(size_t Index) { return OldLength_99((void *)h, Index);          }
         
         /// <summary> Sets the OldLength value of a match to something else</summary> 
         /// <param name="Index"> Index of match</param>
         /// <param name="value"> Sets OldLength to this value</param>
         /// <remarks>
         /// <para>
         ///    In a search/replace transform type of operation where string matches are changed to new strings, the one argument version
         ///    of OldLength() returns what the length of the matching string used to be before it was modified.  However, in some cases,
         ///    the reported OldLength() value may need to be adjusted due to Unicode conversions, such as from utf-8 (default mode uCalc uses internally)
         ///    to utf-16.  This function lets you change the OldLength() value of matches.
         /// </para>
         /// </remarks> 
         void OldLength(size_t Index ,size_t value) { OldLength_100((void *)h, Index , value);          }
         
         /// <summary> Returns the new string a match was changed to</summary> 
         /// <param name="Index"> Index of match</param>
         /// <returns> String value</returns>
         /// <remarks>
         /// <para>
         /// This returns the new string that a match was changed to.  For instance if you do a search/replace transform operation that
         /// changes all occurences of "This" in a string to "That", then NewStr(n) would return "That".
         /// </para>
         /// </remarks> 
         string NewStr(size_t Index) {
            size_t ReturnStrSize;
            const char * ReturnStr = NewStr_101((void *)h, ReturnStrSize , Index);
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Rerturns the starting character position of a match</summary> 
         /// <param name="Index"> Index of match</param>
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Rerturns the starting character position of a match
         /// </para>
         /// </remarks> 
         size_t Start(size_t Index) { return Start_102((void *)h, Index);          }
         
         /// <summary> Changes the reported starting position of a match</summary> 
         /// <param name="Index"> Index of match</param>
         /// <param name="value"> Value of starting position</param>
         /// <remarks>
         /// <para>
         ///    This changes the reported starting position of a match.  Useful in cases when a string is converted from Unicode utf-8
         ///    (default mode uCalc uses internally) to utf-16.
         /// </para>
         /// </remarks> 
         void Start(size_t Index ,size_t value) { Start_103((void *)h, Index , value);          }
         
         /// <summary> Returns the ending character position of a match</summary> 
         /// <param name="Index"> Index of match</param>
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the ending character position of a match
         /// </para>
         /// </remarks> 
         size_t End(size_t Index) { return End_104((void *)h, Index);          }
         
         /// <summary> Returns the length of a match</summary> 
         /// <param name="Index"> Index of match</param>
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the length of a match
         /// </para>
         /// </remarks> 
         size_t Length(size_t Index) { return Length_105((void *)h, Index);          }
         
         /// <summary> Adjusts the reported length of a match</summary> 
         /// <param name="Index"> Index of match</param>
         /// <param name="value"> New length of a match</param>
         /// <remarks>
         /// <para>
         ///    This adjusts the reported length of a match.  Updating the length of the matches is necessary for instance if you are
         ///    converting text from utf-8 (default mode uCalc uses internally) to utf-16.
         /// </para>
         /// </remarks> 
         void Length(size_t Index ,size_t value) { Length_106((void *)h, Index , value);          }
         
         /// <summary> Number of matches</summary> 
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    This returns the number of matches that were found after a Transform operation
         /// </para>
         /// </remarks> 
         size_t Count() { return Count_107((void *)h);          }
         
         /// <summary> Removes a match from the list of matches </summary> 
         /// <param name="Index"> Index of match</param>
         void Remove(size_t Index) { Remove_108((void *)h, Index);          }
         
         /// <summary> Filters a list of matches</summary> 
         /// <param name="NestedRule"> Rule to restrict list of matches to</param>
         /// <returns> Returns the same Matches object</returns>
         /// <remarks>
         /// <para>
         /// Filters a list of matches so that only the ones associated with the given Rule will be in the new list.  In a transform with
         /// multiple rules, sometimes you may want to zero in on the just the matches of a particular rule.  This function lets you do that.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Matches Filter(Rule NestedRule) { return Filter_109((void *)h, (void *)NestedRule.h );          }
         
         /// <summary> Returns the matching string</summary> 
         /// <param name="Index"> Index of match</param>
         /// <returns> Returns a string</returns>
         /// <remarks>
         /// <para>
         /// This returns the text string of the nth match.
         /// </para>
         /// </remarks> 
         string Str(size_t Index) {
            size_t ReturnStrSize;
            const char * ReturnStr = Str_110((void *)h, ReturnStrSize , Index);
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Returns a concatenated list of matches</summary> 
         /// <returns> String</returns>
         /// <remarks>
         /// <para>
         ///    This returns all of the matches, concatenated as one string.
         /// </para>
         /// </remarks> 
         string Str() {
            size_t ReturnStrSize;
            const char * ReturnStr = Str_111((void *)h, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Resets the list of matches</summary> 
         /// <param name="Options"> Option for subset of matches to return</param>
         /// <remarks>
         /// <para>
         ///    This resets the list of matches.  The Options argument lets you filter the list in certain ways.
         /// </para>
         /// </remarks> 
         void Reset(MatchesOption Options  = MatchesOption::SameAsBefore) { Reset_112((void *)h, Options);          }
         
         /// <summary> Returns the rule associated with a given match</summary> 
         /// <param name="Index"> Index of match</param>
         /// <returns> Rule object</returns>
         /// <remarks>
         /// <para>
         ///    This returns the rule associated with a given match.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule Rule(size_t Index) { return Rule_113((void *)h, Index);          }
         
         /// <summary> Sets an offset for matches</summary> 
         /// <param name="StartingPos"> Index of match</param>
         /// <returns> Returns the number of matches that were displaced</returns>
         /// <remarks>
         /// <para>
         ///    This sets an offset that the matches' starting and ending character positions will be relaitve to.
         /// </para>
         /// </remarks> 
         size_t Offset(size_t StartingPos  = 0) { return Offset_114((void *)h, StartingPos);          }
         
         Matches() { h = nullptr;          }
         
         /// <summary> Releases a Matches object</summary> 
         /// <remarks>
         /// <para>
         ///    This releases a Matches object from memory.
         /// </para>
         /// </remarks> 
         void Release() { if ((void *)h != nullptr) Release_115((void *)h); h = nullptr;          }
         
         /// <summary> Returns the handle of an object</summary> 
         /// <remarks>
         /// <para>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload
         ///    (when applicable), you may check Handle() to see when it becomes NULL (or 0) to know when to stop.</para>
         /// </remarks> 
         void * Handle() { return Handle_116((void *)h);          }
         
         Matches(uCalcPtr PassedHandle) { h = PassedHandle;          }
      };
      
      /// <summary> uCalc transform class</summary> 
      struct  Transforms : Base {
         
         /// <summary> Handle for uCalc Transform instance</summary> 
         uCalcPtr h;
         
         /// <summary> Sets a given property</summary> 
         /// <param name="ItemProperty"> The property you want to set</param>
         /// <param name="TrueOrFalse"> Value to set the property to</param>
         /// <returns> Returns the same Transforms object</returns>
         /// <remarks>
         /// <para>This sets a given property
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms IsProperty(ItemIs ItemProperty ,bool TrueOrFalse) { return IsProperty_117((void *)h, ItemProperty , TrueOrFalse);          }
         
         /// <summary> Determines if a uCalc Item has the given properties</summary> 
         /// <param name="ItemProperty"> The property you want to check for</param>
         /// <returns> Returns True if the uCalc Item matches the given property, or False otherwise.</returns>
         /// <remarks>
         /// <para>This helps you determine if an item has the given property
         /// </para>
         /// </remarks> 
         bool IsProperty(ItemIs ItemProperty) { return IsProperty_118((void *)h, ItemProperty);          }
         
         /// <summary> Returns the Tokens object for the current transform</summary> 
         /// <returns> Token list object</returns>
         /// <remarks>
         /// <para>
         ///    Returns the Tokens object, which consists of a list of tokens for the current transform.
         ///    You can use this to add, modify, delete, or reset the tokens.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Tokens Tokens() { return Tokens_119((void *)h);          }
         
         /// <summary> Defines a transform rule</summary> 
         /// <param name="FromStr"> Pattern that describes the text to match</param>
         /// <param name="ToStr"> New text that the match should be changed to</param>
         /// <returns> Object for rule being defined</returns>
         /// <remarks>
         /// <para>This defines something like a search/replace operation.  Whenever a piece of text matches the pattern described in FromStr,
         ///    it is replaced with the text found in ToStr.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule FromTo(const string & FromStr ,const string & ToStr  = "") { return FromTo_120((void *)h, FromStr.data(), FromStr.size() , ToStr.data(), ToStr.size());          }
         
         /// <summary> Sets the uCalc object that performs calculations related to the transform</summary> 
         /// <param name="uCalcHandle"> Handle of a uCalc object</param>
         /// <returns> uCalc object associated with current transform</returns>
         /// <remarks>
         /// <para>If your transform inovlves calcuations, then they will be performed using the
         ///    uCalc object the transform was defined in.  However, you can change the uCalc object with this function.
         ///    Each uCalc object can have its own sets of variables, functions, and other definitions.
         ///    For instance when {@Eval: ...} is used in the pattern or ~Eval() in the text.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms uCalc(void * uCalcHandle) { return uCalc_121((void *)h, uCalcHandle);          }
         
         /// <summary> Restricts the range of text where the search or search/replace operations will be performed</summary> 
         /// <param name="Start"> Starting position</param>
         /// <param name="Size"> Size of string to operate on</param>
         /// <returns> The current transform</returns>
         /// <remarks>
         /// <para>
         ///   Generally transform operations are performed on the entire text that is passed to the transform.  Use this if you only want to
         ///   operate on a subset of the text, starting at Start, with size Size.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms Range(size_t Start ,size_t Size) { return Range_122((void *)h, Start , Size);          }
         
         /// <summary> Creates a new Pass object</summary> 
         /// <param name="Share"> Share</param>
         /// <returns> Transform object</returns>
         /// <remarks>
         /// <para>
         ///    It is possible to create a transform with multiple passes.  Each pass can have a different set of rules.  The transform
         ///    of each pass is executed in sequential order.  Passes can later be referenced by index.  Calling NewPass the first
         ///    time returns the same as Pass(0), the next call to NewPass refers to Pass(1), etc.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms NewPass(bool Share  = false) { return NewPass_123((void *)h, Share);          }
         
         /// <summary> Creates a new rule</summary> 
         /// <returns> Returns the new rule object</returns>
         /// <remarks>
         /// <para>
         ///   This creates a new rule.  A rule typically consists of a string to find and/or modify, along with properties associted with that rule.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule NewRule() { return NewRule_124((void *)h);          }
         
         /// <summary> Returns a Pass object</summary> 
         /// <param name="Index"> Index of pass to select</param>
         /// <param name="Share"> Share</param>
         /// <returns> Transform object</returns>
         /// <remarks>
         /// <para>
         ///    It is possible to create a transform with multiple passes.  Each pass can have a different set of rules.  Each pass
         ///    is transformed in sequential order, starting with Pass(0).  A pass consists of a transform.</para>
         /// <para>   This function returns a pass object, which typically was defined with NewPass().
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms Pass(size_t Index ,bool Share  = false) { return Pass_125((void *)h, Index , Share);          }
         
         /// <summary> Returns the number of passes</summary> 
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the number of passes
         /// </para>
         /// </remarks> 
         size_t PassCount() { return PassCount_126((void *)h);          }
         
         /// <summary> Resets the transform object</summary> 
         /// <param name="ResetOption"> Reset option</param>
         /// <returns> Current transform object</returns>
         /// <remarks>
         /// <para>This clears the input string, transform rules, token definitions, and match list.  It also resets properties to default settings.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms Reset(TransformResetOption ResetOption  = TransformResetOption::All) { return Reset_127((void *)h, ResetOption);          }
         
         /// <summary> Performs the transformation on some text</summary> 
         /// <returns> The current transform object</returns>
         /// <remarks>
         /// <para>Once all the rules and parameters are set, you can call this function in order to
         ///    perform the text transformation.</para>
         /// <para>   Prior to calling Transform, you must set the text to be transformed, with Str
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms Transform() { return Transform_128((void *)h);          }
         
         /// <summary> Filters the text</summary> 
         /// <returns> Current object</returns>
         /// <remarks>
         /// <para>
         ///    This performs a filter operation on text.  This is similar to a Transform() operation, however, the result is text consisisting of
         ///    just the matches.  The operation is performed on text that was previously passed to Str().
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms Filter() { return Filter_129((void *)h);          }
         
         /// <summary> Performs a Filter operation on the given text</summary> 
         /// <param name="expression"> Expression to be transformed</param>
         /// <returns> Current transform</returns>
         /// <remarks>
         /// <para>
         ///    This performs a filter operation on text.  This is similar to a Transform() operation, however, the result is text consisisting of
         ///    just the matches.  The operation is performed on the string passed to this Filter() function.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms Filter(const string & expression) { return Filter_130((void *)h, expression.data(), expression.size());          }
         
         /// <summary> Performs a text search operation</summary> 
         /// <param name="Str"> Text to search through</param>
         /// <returns> The current Transforms object</returns>
         /// <remarks>
         /// <para>Once all the rules and parameters are set, you can call this function in order to
         ///    perform the text search.</para>
         /// <para>   You may either set the string to Str() prior to calling Find(), or you can pass a string argument directly to this function.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms Find(const string & Str  = "") { return Find_131((void *)h, Str.data(), Str.size());          }
         
         /// <summary> Transforms an expression in one quick step</summary> 
         /// <param name="expression"> Expression to be transformed</param>
         /// <returns> Current object</returns>
         /// <remarks>
         /// <para>
         ///    When you would like to set text to be transformed, and transform the text, and return a result (with .Str()), all in one step,
         ///    you can use this function, instead of calling Str, then Transform, then Str.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms Transform(const string & expression) { return Transform_132((void *)h, expression.data(), expression.size());          }
         
         /// <summary> Returns each step leading to an expression expansion</summary> 
         /// <param name="expression"> Expression to be transformed</param>
         /// <returns> String list containg steps leading up to transform result</returns>
         /// <remarks>
         /// <para>
         ///    Transforms an expression and returns intermediate steps
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String TransformWithSteps(const string & expression) { return TransformWithSteps_133((void *)h, expression.data(), expression.size());          }
         
         /// <summary> Returns string result of transformed text</summary> 
         /// <returns> String result of transformed text</returns>
         /// <remarks>
         /// <para>After performing a text transformation, you may obtain the transformed result with this
         ///    function.
         /// </para>
         /// </remarks> 
         string Str() {
            size_t ReturnStrSize;
            const char * ReturnStr = Str_134((void *)h, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Sets the text input that is to be transformed or searched</summary> 
         /// <param name="InputStr"> Text that mast be parsed and modified or searched</param>
         /// <returns> Current transform object</returns>
         /// <remarks>
         /// <para>The text that you want to transform or search should be passed to this function.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms Str(const string & InputStr) { return Str_135((void *)h, InputStr.data(), InputStr.size());          }
         
         /// <summary> Returns actual character base pointer of string</summary> 
         /// <returns> Pointer to uint8_t array</returns>
         /// <remarks>
         /// <para>
         ///    Note: Should never be used alone.  Always use in pair with StrLength
         /// </para>
         /// </remarks> 
         const char * StrData() { return StrData_136((void *)h);          }
         
         /// <summary> Specifically intended for use with handling Unicode utf8 vs utf16 difference</summary> 
         /// <param name="Start"> Starting character position</param>
         /// <param name="Count"> Number of characters</param>
         /// <returns> String</returns>
         /// <remarks>
         /// <para>
         /// This returns a SubStr of the text
         /// </para>
         /// </remarks> 
         string SubStr(size_t Start ,size_t Count  = -1) {
            size_t ReturnStrSize;
            const char * ReturnStr = SubStr_137((void *)h, ReturnStrSize , Start , Count);
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Returns the length of the text string being operated on</summary> 
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    This returns the length of the text string being operated on
         /// </para>
         /// </remarks> 
         size_t StrLength() { return StrLength_138((void *)h);          }
         
         /// <summary> Returns the Matches object for the current Transform</summary> 
         /// <param name="Options"> Option for subset of matches to return</param>
         /// <returns> Matches object</returns>
         /// <remarks>
         /// <para>
         ///    This returns the Matches object for the current Transform.  The Matches object contains the list of matches, the
         ///    location and lengh of each match, etc.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Matches Matches(MatchesOption Options  = MatchesOption::FocusableOnly) { return Matches_139((void *)h, Options);          }
         
         /// <summary> Sets the index base used in various functions</summary> 
         /// <param name="BaseValue"> value representing index of 1st element</param>
         /// <remarks>
         /// <para>
         ///    By default, uCalc String Library functions use a 0-based index.  So the first element is 0 (not 1), followed by 2, 3, 4 etc.
         ///    Use this if you prefer to start by 1 or another non-zero value.  This value is passed on to string objects derived from a parent string.
         ///    Each string may have a different index base.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms IndexBase(size_t BaseValue) { return IndexBase_140((void *)h, BaseValue);          }
         
         /// <summary> Gets the index base used in various functions</summary> 
         /// <remarks>
         /// <para>
         ///    By default, uCalc String Library functions use a 0-based index.  So the first element is 0 (not 1), followed by 2, 3, 4 etc.
         ///    Use this to check to see which base is being used.
         /// </para>
         /// </remarks> 
         size_t IndexBase() { return IndexBase_141((void *)h);          }
         
         /// <summary> Indicates whether text was modified in the most recent Transform operation</summary> 
         /// <returns> True if text was modified in the most recent transformation operation or false otherwise</returns>
         /// <remarks>
         /// <para>
         ///    If text was modified after a call to Transform, then this function returns True.  When transforming large amounts of text,
         ///    transferring strings back and forth with Str() can be expensive.  Checking this property allows you to skip the unnecessary
         ///    transfer of text when the text has not been modified.
         /// </para>
         /// </remarks> 
         bool WasModified() { return WasModified_142((void *)h);          }
         
         /// <summary> Returns the defaults settings for rules</summary> 
         /// <returns> Default rule object</returns>
         /// <remarks>
         /// <para>
         ///    Each FromTo or Find rule has many configurable properties that can be set.  Instead of excplicitely setting all of these many properties
         ///    with each rule definition, you may define default settings that will automatically be assigned to new rules.  Any of these default settings
         ///    can be overridden by explicitely setting the desired property for a given rule.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule RulesDefault() { return RulesDefault_143((void *)h);          }
         
         /// <summary> Returns the overall settings for a transform</summary> 
         /// <returns> Overall rule object</returns>
         /// <remarks>
         /// <para>
         ///    This returns an object containing the properties for a transform or search object.  This object is a Rules object, and conatains the
         ///    same properties listed for individual rules, but works in a slightly different context.  For instance, if you set the StopAfter property
         ///    of FromTo("a", "b") to 5, then it will only change the first five occurences of "a" to "b", whereas if you set the RulesOverall StopAfter
         ///    property to 5, and you have rules FromTo("a", "b") and FromTo("x", "y") then it will stop changing any occurence of either rule, after the
         ///    combined total of changes reaches 5; "a" might change to "b" 2 times and "x" to "y" 3 times, or whichever combination does not total more
         ///    than 5.  You might set QuoteSensitive to "true" overall, while you may set it to false within a given rule.  In certain combinations,
         ///    overall rules may clash with individual rules.  Although all combinations are allowed, not all combinations make sense.  When setting
         ///    the case sensitivity property of individual rules differently from the overall property, keep in mind that if the overall case sensitivity
         ///    is false, then all symbols are stored in the symbol table in all upper (or lower) case.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule RulesOverall() { return RulesOverall_144((void *)h);          }
         
         operator string() { return Str();          }
         
         Transforms() { h = nullptr; hb = h;          }
         
         /// <summary> Releases a transform</summary> 
         /// <remarks>
         /// <para>
         ///    This releases a transform from memory along with all the objects defined under it.
         /// </para>
         /// </remarks> 
         void Release() { if ((void *)h != nullptr) Release_145((void *)h); h = nullptr;          }
         
         /// <summary> Returns the handle of an object</summary> 
         /// <remarks>
         /// <para>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload
         ///    (when applicable), you may check Handle() to see when it becomes NULL (or 0) to know when to stop.</para>
         /// </remarks> 
         void * Handle() { return Handle_146((void *)h);          }
         
         Transforms(uCalcPtr PassedHandle) { h = PassedHandle; hb = h;          }
      }; // Transforms
      
      /// <summary> Class for rules</summary> 
      struct  Rule : Base {
         
         /// <summary> Handle for Rule</summary> 
         uCalcPtr h;
         
         /// <summary> Sets a given property</summary> 
         /// <param name="ItemProperty"> The property you want to set</param>
         /// <param name="TrueOrFalse"> Value to set the property to</param>
         /// <returns> Current rule</returns>
         /// <remarks>
         /// <para>Sets a given property for a rule.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule IsProperty(ItemIs ItemProperty ,bool TrueOrFalse) { return IsProperty_147((void *)h, ItemProperty , TrueOrFalse);          }
         
         /// <summary> Determines if a uCalc Item has the given properties</summary> 
         /// <param name="ItemProperty"> The property you want to check for</param>
         /// <returns> Returns True if the uCalc Item matches the given property, or False otherwise.</returns>
         /// <remarks>
         /// <para>Determines if a uCalc Item has the given properties.
         /// </para>
         /// </remarks> 
         bool IsProperty(ItemIs ItemProperty) { return IsProperty_148((void *)h, ItemProperty);          }
         
         /// <summary> Determines whether a rule is currently active</summary> 
         /// <returns> True or False</returns>
         /// <remarks>
         /// <para>
         ///    This lets you know if a rule is currently active or not.
         /// </para>
         /// </remarks> 
         bool Active() { return Active_149((void *)h);          }
         
         /// <summary> Sets whether a rule is active</summary> 
         /// <param name="TrueFalse"> Value that sets a rule as active or not</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    This sets whether a rule is active or not.
         ///    When you no longer need a rule, you can release it entirely from memory with Release().  However, at times you may
         ///    want to only temporarily deactivate a rule, and then later reactivate it for further use.  That is when you use
         ///    this function.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule Active(bool TrueFalse) { return Active_150((void *)h, TrueFalse);          }
         
         /// <summary> Defines a transform rule</summary> 
         /// <param name="FromStr"> Pattern that describes the text to match</param>
         /// <param name="ToStr"> New text that the match should be changed to</param>
         /// <returns> Object for rule being defined</returns>
         /// <remarks>
         /// <para>Whenever a piece of text matches the pattern described in FromStr, it is replaced with the text
         ///    found in ToStr.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule FromTo(const string & FromStr ,const string & ToStr  = "") { return FromTo_151((void *)h, FromStr.data(), FromStr.size() , ToStr.data(), ToStr.size());          }
         
         /// <summary> Lets you know weather a rule is focusable</summary> 
         /// <returns> True or false</returns>
         /// <remarks>
         /// <para>
         ///    This lets you know weather a rule is focusable.  You may use this property to decide wheather a match shouild be highlighted for instance.
         /// </para>
         /// </remarks> 
         bool Focusable() { return Focusable_152((void *)h);          }
         
         /// <summary> Sets the focusable property of a Rule to True or False</summary> 
         /// <param name="TrueFalse"> Value that determines rule is focusable</param>
         /// <returns> Current object</returns>
         /// <remarks>
         /// <para>
         ///    This sets the focusable property of a Rule to True or False.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule Focusable(bool TrueFalse) { return Focusable_153((void *)h, TrueFalse);          }
         
         /// <summary> Returns the Find part of a rule in a search or transform operation</summary> 
         /// <returns> A string</returns>
         /// <remarks>
         /// <para>
         ///    This returns the Find part of a rule.
         /// </para>
         /// </remarks> 
         string FindPart() {
            size_t ReturnStrSize;
            const char * ReturnStr = FindPart_154((void *)h, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Sets the Find part of a rule</summary> 
         /// <param name="Pattern"> Pattern for the Find part of a rule</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    This sets the Find part of a rule in a search.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule FindPart(const string & Pattern) { return FindPart_155((void *)h, Pattern.data(), Pattern.size());          }
         
         /// <summary> Returns the Find part of a rule</summary> 
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    This returns the Find part of a rule.
         /// </para>
         /// </remarks> 
         string FromPart() {
            size_t ReturnStrSize;
            const char * ReturnStr = FromPart_156((void *)h, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Sets the From part of a rule</summary> 
         /// <param name="Pattern"> Pattern for the From part of a rule</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    This sets the From part of a rule.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule FromPart(const string & Pattern) { return FromPart_157((void *)h, Pattern.data(), Pattern.size());          }
         
         /// <summary> Returns the To part of a rule</summary> 
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    This sets the To part of a rule.
         /// </para>
         /// </remarks> 
         string ToPart() {
            size_t ReturnStrSize;
            const char * ReturnStr = ToPart_158((void *)h, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Sets the To part of a rule</summary> 
         /// <param name="Pattern"> Pattern for the From part of a rule</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    This sets the To part of a rule.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule ToPart(const string & Pattern) { return ToPart_159((void *)h, Pattern.data(), Pattern.size());          }
         
         /// <summary> Lets you know if the rule is case-sensitive</summary> 
         /// <returns> True or false</returns>
         /// <remarks>
         /// <para>
         ///    Lets you know if the rule is case-sensitive.
         /// </para>
         /// </remarks> 
         bool CaseSensitive() { return CaseSensitive_160((void *)h);          }
         
         /// <summary> Sets whether a rule is case-sensitive</summary> 
         /// <param name="TrueFalse"> Value that determines case sensitivity</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>This sets whether a rule is case-sensitive or not.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule CaseSensitive(bool TrueFalse) { return CaseSensitive_161((void *)h, TrueFalse);          }
         
         /// <summary> Lets you know if SkipPatternMatching is set</summary> 
         /// <returns> True/False</returns>
         /// <remarks>
         /// <para>
         ///    Lets you know if SkipPatternMatching is set.
         /// </para>
         /// </remarks> 
         bool IgnoreInnerPatterns() { return IgnoreInnerPatterns_162((void *)h);          }
         
         /// <summary> Sets whether uCalc should ignore pattern matching for a rule</summary> 
         /// <param name="TrueFalse"> True or false</param>
         /// <returns> This Rule</returns>
         /// <remarks>
         /// <para>Sets whether uCalc should ignore pattern matching for a rule.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule IgnoreInnerPatterns(bool TrueFalse) { return IgnoreInnerPatterns_163((void *)h, TrueFalse);          }
         
         /// <summary> Lets you konw if the rule is Whitespace sensitive</summary> 
         /// <returns> True/False</returns>
         /// <remarks>
         /// <para>
         ///    Lets you know if WhitespaceSensitive is set.
         /// </para>
         /// </remarks> 
         bool WhitespaceSensitive() { return WhitespaceSensitive_164((void *)h);          }
         
         /// <summary> Sets whether a rule is whitespace-sensitive</summary> 
         /// <param name="TrueFalse"> Value that determines whitespace sensitivity</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>This sets whether a rule is whitespace-sensitive or not.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule WhitespaceSensitive(bool TrueFalse) { return WhitespaceSensitive_165((void *)h, TrueFalse);          }
         
         /// <summary> Lets you konw if the rule is Bracket sensitive</summary> 
         /// <returns> True/False</returns>
         /// <remarks>
         /// <para>
         ///    Lets you konw if the rule is Bracket sensitive.
         /// </para>
         /// </remarks> 
         bool BracketSensitive() { return BracketSensitive_166((void *)h);          }
         
         /// <summary> Sets bracket sensitivity</summary> 
         /// <param name="TrueFalse"> Value that determines bracket sensitivity</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>This sets whether a rule is bracket sensitive or not.  When set to True, a pattern will not match if it crosses the boundaries of
         ///    a bracket grouping.</para>
         /// <para>   If set to False, then clode block delimiters are treated as ordinary generic tokens, with no special property as delimiters.</para>
         /// <para>   A bracketed group is a pattern enclosed between tokens defined as a bracket pair.  By default square brackets, [],
         ///    parenthesis, (), and curly braces, {}, are defined as bracket pairs.  However, any string pairs can be defined as
         ///    bracket tokens.</para>
         /// <para>   So, if you define a pattern "a {etc} b", it will match "a x y z b c", "a (x y z) b c", "a (x) (y z) b c", but not "a x y z (b c)",
         ///    because the bracket grouping (b c) crosses the boundary of the "a {etc} b" pattern; the opening parenthesis is found in the would-be match,
         ///    but the closing parenthesis is beyond it, whereas in the other examples, both opening and closing parenthesis (when present) are
         ///    fully within "a {etc} b".</para>
         /// <para>   Note: Brackets used in pattern definitions take precedence over bracket token pairs.  Therefore you may define a pattern that gives
         ///    brackets a different behavior than the default.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule BracketSensitive(bool TrueFalse) { return BracketSensitive_167((void *)h, TrueFalse);          }
         
         /// <summary> Lets you know if a rule has a local transform</summary> 
         /// <returns> True/False</returns>
         /// <remarks>
         /// <para>
         ///    This lets you know if a rule has a local transform.
         /// </para>
         /// </remarks> 
         bool HasLocalTransform() { return HasLocalTransform_168((void *)h);          }
         
         /// <summary> Returns a local transform object</summary> 
         /// <returns> Local transform object</returns>
         /// <remarks>
         /// <para>
         ///    When a pattern is matched, you can zoom into the text and perform a local search just within the matched text, independantly
         ///    from the rest of the text outside of the match.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms LocalTransform() { return LocalTransform_169((void *)h);          }
         
         /// <summary> Lets you know if a rule is statement sensitive</summary> 
         /// <returns> True/False</returns>
         /// <remarks>
         /// <para>
         ///    This lets you know if a rule is statement sensitive.  If a rule is statement sensitive, it will not allow a match to cross statement boundaries.
         /// </para>
         /// </remarks> 
         bool StatementSensitive() { return StatementSensitive_170((void *)h);          }
         
         /// <summary> Sets a rule as statement Sensitive</summary> 
         /// <param name="TrueFalse"> Value that determines bracket sensitivity</param>
         /// <returns> Current Rule</returns>
         /// <remarks>
         /// <para>
         ///    Sets a rule as statement Sensitive
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule StatementSensitive(bool TrueFalse) { return StatementSensitive_171((void *)h, TrueFalse);          }
         
         /// <summary> Lets you know the maximum number of matches that will be looked for with this rule</summary> 
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Lets you know the maximum number of matches that will be looked for with this rule.
         /// </para>
         /// </remarks> 
         size_t Maximum() { return Maximum_172((void *)h);          }
         
         /// <summary> Sets the maximum number of matches allowed for this rule to count</summary> 
         /// <param name="Value"> Maximum number of matches of this rule allowed</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    This sets the maximum number of individual matches that this rule should find in order for any matches
         ///    of this rule to count.  If more than this number of matches are found in the text, then this rule will be
         ///    considered to have no matches at all.  For instance if a rule searches for matches of "a" in "a a r d v a r k",
         ///    and the maximum is set to 2, then no matches of "a" will be counted at all, because there are more than
         ///    2 occurences of "a" in the text.  On the other hand, if Max is set to 3 or 4 or any larger value, then
         ///    all the matches would count.</para>
         /// <para>   The default Maximum value is -1, which is the equivalent of no maximum limit.</para>
         /// <para>   If you want to stop counting after a given number of matches of a rule, without invalidating the matches
         ///    that were already found, use StopAfter instead.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule Maximum(size_t Value) { return Maximum_173((void *)h, Value);          }
         
         /// <summary> Returns the maximum number of matches of this rule allowed for the whole search to count</summary> 
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the maximum number of matches of this rule allowed for the whole search to count.
         /// </para>
         /// </remarks> 
         size_t MaximumAND() { return MaximumAND_174((void *)h);          }
         /// <summary> Sets the maximum number of matches of this rule for the whole search to count</summary> 
         /// <param name="Value"> Maximum number of matches of this rule allowed</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    This sets the maximum number of individual matches that this rule should find in order for any matches
         ///    of the transform to count.  If more than this number of matches are found in the text, then no matches from
         ///    any rule in the local transform will count (whereas in a maximum, only matches from the current rule will be
         ///    cancelled).  For instance if a rule searches for matches of "a" in "a a r d v a r k",
         ///    and the maximum AND is set to 2, then the entire transform (including other rules) will be invalidated and
         ///    return no matches, because there are more than 2 occurences of "a" in the text.  On the other hand, if
         ///    Max is set to 3 or 4 or any larger value, then all the matches would count.</para>
         /// <para>   This differs from Maximum in that if Maximum is exceeded, then only the Rule with the maximum will return
         ///    0 matches, whereas this one will cause the whole transform to return 0 matches.  This affects only the
         ///    current transform and does not propagate to a parent transform or a transform in a separate pass.</para>
         /// <para>   The default Maximum value is -1, which is the equivalent of no maximum limit.</para>
         /// <para>   If you want to stop counting after a given number of matches of a rule, without invalidating any matches, use StopAfter instead.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule MaximumAND(size_t Value) { return MaximumAND_175((void *)h, Value);          }
         
         /// <summary> Returns the minimum number of matches that must be found in this rule</summary> 
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the minimum number of matches that must be found in this rule
         /// </para>
         /// </remarks> 
         size_t Minimum() { return Minimum_176((void *)h);          }
         
         /// <summary> Sets the minimum number of matches that must be found in this rule</summary> 
         /// <param name="Value"> Minimum number of matches required of this rule</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    This sets the minimum number of individual matches that this rule should find.</para>
         /// <para>   This differs from a MinimumAND in that if this Minimum is not
         ///    reached, then it is considered that there are no matches for this particular rule.  Other rules
         ///    are not affected.  Whereas in a MinimumAND if this rule does not reach the required minimum,
         ///    then the entire local search or transform is invalidated, and it's considered that no match of any
         ///    kind is found.</para>
         /// <para>   If less than this number of matches are found in the text, then none of the
         ///    matches in this rule will count.  For instance if a rule searches for matches of "a" in "a a r d v a r k",
         ///    and the minimum is set to 5, then no matches of "a" will be counted at all, because there are not at least
         ///    5 occurences of "a" in the text.  On the other hand, if Min is set to 3 or less, then all the matches count.</para>
         /// <para>   Default: 0.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule Minimum(size_t Value) { return Minimum_177((void *)h, Value);          }
         
         /// <summary> Lets you know the minimum number of matches that must be found in this rule</summary> 
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Lets you know the minimum number of matches that must be found in this rule.
         /// </para>
         /// </remarks> 
         size_t MinimumAND() { return MinimumAND_178((void *)h);          }
         
         /// <summary> Sets the minimum number of matches that must be found in this rule</summary> 
         /// <param name="Value"> Minimum number of matches required of this rule</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    This sets the minimum number of individual matches that this rule should find.</para>
         /// <para>   This differs from a Minimum in that if Minimum is not
         ///    reached, then it is considered that there are no matches for that particular rule.  Other rules
         ///    are not affected.  Whereas in a MinimumAND, if this rule does not reach the required minimum,
         ///    then the entire search or transform is invalidated, and it's considered that no match of any
         ///    kind is found.</para>
         /// <para>   If less than this number of matches are found in the text, then the total match count will be 0.
         ///    For instance if a rule searches for matches of "a" in "a a r d v a r k", and the minimum is set
         ///    to 5, the the whole transform operation will return 0 matches, because there are not at least
         ///    5 occurences of "a" in the text.  On the other hand, if it is set to 3 or less, then all the matches count.</para>
         /// <para>   Default: 0.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule MinimumAND(size_t Value) { return MinimumAND_179((void *)h, Value);          }
         
         /// <summary> Returns the transform this rule belongs to</summary> 
         /// <returns> Transform object</returns>
         /// <remarks>
         /// <para>
         ///    This returns the transform this rule belongs to.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Transforms ParentTransform() { return ParentTransform_180((void *)h);          }
         
         /// <summary> Returns the precedence level for a rule</summary> 
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    Returns the precedence level for a rule.
         /// </para>
         /// </remarks> 
         size_t Precedence() { return Precedence_181((void *)h);          }
         /// <summary> Sets the precedence level for a rule</summary> 
         /// <param name="Value"> Precedence level</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    Patterns that start (and/or end) with a pattern variable instead of a token or identifier can
         ///    (and probably should) be assigned a precedence level.  You can choose any arbitrary numerical
         ///    value.  The meaning of the precedence level depends on whether the value is greater, equal, or
         ///    less than adjacent patterns with a precedence level.  It works similarly to precedence levels for
         ///    operators.  For instance, if you define:</para>
         /// <para>   FromTo("{a} + {b}", "({a} + {b})").Precedence(10);
         ///    FromTo("{a} * {b}", "({a} * {b})").Precedence(20);</para>
         /// <para>   Then</para>
         /// <para>   "1 + 2 * 3 + 4" transforms into "(1 + (2 * 3) + 4)"
         ///    "1 * 2 + 3 * 4" transforms into "(1 * 2) + (3 * 4)"
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule Precedence(size_t Value) { return Precedence_182((void *)h, Value);          }
         
         /// <summary> Sets a rule property</summary> 
         /// <param name="SelectedProperty"> Selected Property</param>
         /// <param name="TrueFalse"> True/False</param>
         /// <returns> Current object</returns>
         /// <remarks>
         /// <para>
         ///    Sets a rule property
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule Property(ItemIs SelectedProperty ,bool TrueFalse) { return Property_183((void *)h, SelectedProperty , TrueFalse);          }
         
         /// <summary> Lets you know if the rule is quote sensitive</summary> 
         /// <returns> True/False</returns>
         /// <remarks>
         /// <para>
         ///    This lets you know if the rule is quote sensitive.  The other version of this function sets or unsets quote sensitivity.
         /// </para>
         /// </remarks> 
         bool QuoteSensitive() { return QuoteSensitive_184((void *)h);          }
         
         /// <summary> Sets quote sensitivity</summary> 
         /// <param name="TrueFalse"> Value that determines quote sensitivity</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>This sets whether a rule is quote-sensitive or not.
         ///    Quote sensitivity is generally designed for use across all rules in a transform (or sub transform).  However,
         ///    it can also be set in the context of an individual pattern.</para>
         /// <para>   When set to true, it means that a block of text within quotes is treated as one token.  As such, patterns are not searched for
         ///    within the quotes.  The contents of the quotes are skipped over.  You may still get around this and parse within quotes when
         ///    necessary, by capturing the quote character itself, then parsing subsequent text.  When doing this be mindful of capturing
         ///    quote characters in pairs.</para>
         /// <para>   The actual character(s) used as quotes are configurable with Tokens().Add().
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule QuoteSensitive(bool TrueFalse) { return QuoteSensitive_185((void *)h, TrueFalse);          }
         
         /// <summary> Returns the replacement text of a FromTo rule</summary> 
         /// <returns> String representing the second argument that was used in FromTo</returns>
         /// <remarks>
         /// <para>This allows you to keep track of the replacement text of any given rule, even after it has been defined.</para>
         /// </remarks> 
         string ReplacementText() {
            size_t ReturnStrSize;
            const char * ReturnStr = ReplacementText_186((void *)h, ReturnStrSize );
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Lets you know the number of matches this rule must find before matches start counting</summary> 
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///   Lets you know the number of matches this rule must find before matches start counting.  The other version
         ///   of StartAfter() lets you set this value.
         /// </para>
         /// </remarks> 
         size_t StartAfter() { return StartAfter_187((void *)h);          }
         
         /// <summary> Sets the number of matches this rule must find before matches start counting</summary> 
         /// <param name="Value"> Number of matches that must be found before matches start counting</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    This sets the number of individual matches that this rule should find first before matches start counting.
         ///    These first few matches are not counted in the results.  Onlny matches starting after this number of initial
         ///    matches are counted.  If less than this number of matches are found in the text, no match will count for this rule.
         ///    For instance if a rule searches for matches of "a" in "a a r d v a r k", starting after 2, then only the 3rd "a"
         ///    (the one between "v" and "r") will count.  If StartAfter is set to 3, there will be no matches.  If StartAfter
         ///    is 1, then then the 2nd and 3rd occurences of "a" will count as matches.</para>
         /// <para>   The default StartAfter value is 0.  This means all matches that are found will count.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule StartAfter(size_t Value) { return StartAfter_188((void *)h, Value);          }
         
         /// <summary> Lets you know the number of matches after which this rule should stop finding matches</summary> 
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    Lets you know the number of matches after which this rule should stop finding matches.  The other
         ///    version of this function lets you set the value.
         /// </para>
         /// </remarks> 
         size_t StopAfter() { return StopAfter_189((void *)h);          }
         
         /// <summary> Sets the number of matches after which this rule should stop finding matches</summary> 
         /// <param name="Value"> Number of matches after which it should stop finding matches</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>
         ///    This sets the number of individual matches that this rule should find.  After this number of matches
         ///    are found, all other matches for this rule are ignored.  For instance if a rule searches for matches
         ///    of "a" in "a a r d v a r k", and stops after 2, then only the first two letters here wil match; the
         ///    third "a" (the one between "v" and "r") will not count as a match.</para>
         /// <para>   The default StopAfter value is -1, which means it will not stop counting matches for this rule.</para>
         /// <para>   This function has some similarities with Maximum.  However, with Maximum, if the maximum value is not attained,
         ///    none of the matches for the rule will count.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule StopAfter(size_t Value) { return StopAfter_190((void *)h, Value);          }
         
         /// <summary> Lets you know if matches for the current pattern will be skipped over</summary> 
         /// <returns> True/False</returns>
         /// <remarks>
         /// <para>
         ///    Lets you know if matches for the current pattern will be skipped over.  The other version of
         ///    SkipOver lets you set the value.
         /// </para>
         /// </remarks> 
         bool SkipOver() { return SkipOver_191((void *)h);          }
         
         /// <summary> Determines whether a given match is skipped over</summary> 
         /// <param name="TrueFalse"> Value that determines if a matched pattern is skipped over</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>If SkipOver is set to true, then text matches this pattern will be skipped over.
         ///    This is useful for preventing uCalc from searching for matches of any other rules within
         ///    certain text you want to skip over.</para>
         /// <para>   For instance if you skip over "({etc})", then everything in parenthesis will be skipped over.
         ///    So if you want to find occurences of "a" in "a b c (a b c a b c) a b c", it will find all occurences
         ///    of "a", except those within parenthesis, because everything in parenthesis is being skipped over.</para>
         /// <para>   Default: False
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule SkipOver(bool TrueFalse) { return SkipOver_192((void *)h, TrueFalse);          }
         
         /// <summary> Lets you know if the parser should rewind and parse again after a change</summary> 
         /// <returns> True/False</returns>
         /// <remarks>
         /// <para>
         ///    Lets you know if the parser should rewind and parse again after a change.  The other version of
         ///    this function sets the value.
         /// </para>
         /// </remarks> 
         bool RewindOnChange() { return RewindOnChange_193((void *)h);          }
         
         /// <summary> Determines whether to rewind back to the start of the modified match for further parsing</summary> 
         /// <param name="TrueFalse"> Value that determines if the transfrom should rewind and parse again after a change</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>After a match is modified with the replacement text of a rule, if RewindOnChange is false (default),
         ///    then uCalc will continue parsing from the position just after the insertion of the modified text.
         ///    If this property is set to true, then the parser will rewind to the location where the match was
         ///    found, and parse again, which will allow it to potentially find additional matches (of any rule,
         ///    not just the current one) within the already modified text.</para>
         /// <para>   This property can be extremely useful in many situations.  However, special care should be taken
         ///    so that an infinite reparsing session doesn't take place.  For instance, if a rule changes A to B,
         ///    and another rule changes B to A, and this property is set ot true, and a Maximum value is not set,
         ///    and there is an occurence of A, then A will change to B.  B will be parsed again and changed to A.
         ///    The cycle will repeat itself indefinitely.</para>
         /// <para>   Default: False
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule RewindOnChange(bool TrueFalse) { return RewindOnChange_194((void *)h, TrueFalse);          }
         
         /// <summary> Lets you know the associativity property of a rule</summary> 
         /// <returns> True/Flase</returns>
         /// <remarks>
         /// <para>
         ///    Lets you know the associativity property of a rule; true means it goes right to left; false means it's left to right (default)
         /// </para>
         /// </remarks> 
         bool RightToLeft() { return RightToLeft_195((void *)h);          }
         
         /// <summary> Sets whether a rule is right-to-left associative</summary> 
         /// <param name="TrueFalse"> Value that determines case sensitivity</param>
         /// <returns> The current rule object</returns>
         /// <remarks>
         /// <para>This sets whether a rule is is right-to-left associative.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule RightToLeft(bool TrueFalse) { return RightToLeft_196((void *)h, TrueFalse);          }
         
         /// <summary> Returns an object that contains the list of matches along with associated properties for each match</summary> 
         /// <param name="Options"> Option for subset of matches to return</param>
         /// <returns> Matches object</returns>
         /// <remarks>
         /// <para>
         ///    Returns an object that contains the list of matches along with associated properties for each match.  With this object, you
         ///    can for instance, determine the location and length of each match, which rule the match applies to etc.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Matches Matches(MatchesOption Options  = MatchesOption::SameAsBefore) { return Matches_197((void *)h, Options);          }
         
         /// <summary> Lets you know if this rule is the child of another rule</summary> 
         /// <returns> True/False</returns>
         /// <remarks>
         /// <para>
         ///    Lets you know if this rule is the child of another rule
         /// </para>
         /// </remarks> 
         bool IsChildRule() { return IsChildRule_198((void *)h);          }
         
         /// <summary> Returns the number of matches associated with this particular rule</summary> 
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the number of matches associated with this particular rule
         /// </para>
         /// </remarks> 
         size_t MatchesCount() { return MatchesCount_199((void *)h);          }
         
         /// <summary> Returns the index value associated with a rule</summary> 
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the index value associated with a rule.
         /// </para>
         /// </remarks> 
         size_t Index() { return Index_200((void *)h);          }
         
         /// <summary> Sets an index value associated with this rule</summary> 
         /// <param name="Value"> Index to associate with this rule</param>
         /// <returns> This same rule</returns>
         /// <remarks>
         /// <para>
         ///    This sets an index value to be associated with this rule.  Each rule should have a unique index.  For instance, if you want to
         ///    highlight the matches from one rule in Red, and the matches from another rule in Blue, by obtaining Matches(n).Rule().Index(),
         ///    you can determine which color to choose for match n.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Rule Index(size_t Value) { return Index_201((void *)h, Value);          }
         
         Rule() { h = nullptr; hb = h;          }
         
         /// <summary> Releases a transform rule.  Transform rules are defined with FromTo.</summary> 
         /// <remarks>
         /// <para>
         ///    This releases a transform rule from memory.
         /// </para>
         /// </remarks> 
         void Release() { if ((void *)h != nullptr) Release_202((void *)h); h = nullptr;          }
         
         /// <summary> Returns the handle of an object</summary> 
         /// <remarks>
         /// <para>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload
         ///    (when applicable), you may check Handle() to see when it becomes NULL (or 0) to know when to stop.</para>
         /// </remarks> 
         void * Handle() { return Handle_203((void *)h);          }
         
         Rule(uCalcPtr PassedHandle) { h = PassedHandle; hb = h;          }
      }; // Rule
      
      /// <summary> Class for collection of tokens</summary> 
      struct  Tokens : Base {
         
         /// <summary> Handle for tokens</summary> 
         uCalcPtr h;
         
         /// <summary> Sets a given property</summary> 
         /// <param name="ItemProperty"> The property you want to set</param>
         /// <param name="TrueOrFalse"> Value to set the property to</param>
         /// <returns> Current object</returns>
         /// <remarks>
         /// <para>Sets a given property
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Tokens IsProperty(ItemIs ItemProperty ,bool TrueOrFalse) { return IsProperty_204((void *)h, ItemProperty , TrueOrFalse);          }
         
         /// <summary> Determines if a token has the given property</summary> 
         /// <param name="ItemProperty"> The property you want to check for</param>
         /// <returns> Returns True if the token matches the given property, or False otherwise.</returns>
         /// <remarks>
         /// <para>Determines if a token has the given property
         /// </para>
         /// </remarks> 
         bool IsProperty(ItemIs ItemProperty) { return IsProperty_205((void *)h, ItemProperty);          }
         
         /// <summary> Imports a list of tokens from another token list</summary> 
         /// <param name="TokenCollection"> Pre-existing token list</param>
         /// <returns> Object containing list of tokens</returns>
         /// <remarks>
         /// <para>
         ///    Instead of defining a set of tokens each time, you may import a pre-existing token list.  This function imports from
         ///    another token list object.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Tokens Add(Tokens TokenCollection) { return Add_206((void *)h, (void *)TokenCollection.h );          }
         
         /// <summary> Imports an individual token item</summary> 
         /// <param name="TokenItem"> Pre-existing token item</param>
         /// <returns> Current token list</returns>
         /// <remarks>
         /// <para>
         ///    Imports an individual token item into the token list.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Tokens Add(Item TokenItem) { return Add_207((void *)h, (void *)TokenItem.h );          }
         
         /// <summary> Defines a new token</summary> 
         /// <param name="TokenDef"> Token definition</param>
         /// <param name="Type"> The type of token</param>
         /// <param name="CloseBracket"> String representing closing bracket, such as "}", ")", "]", etc.</param>
         /// <param name="SubMatch"> Submatch</param>
         /// <param name="Grammar"> Type of RegEx grammar</param>
         /// <param name="InsertionPoint"> Where to insert the token into the token list</param>
         /// <returns> Returns the newly created token item object</returns>
         /// <remarks>
         /// <para>
         ///    uCalc comes with a default set of tokens.  These are general tokens for things like numbers, alphanumeric words, whitespace, quotes,
         ///    parenthesis, etc.  However you may define your own tokens to the exact specifications that you need.</para>
         /// <para>   A token is defined using a Regular Expression, which is passed as the first argument.</para>
         /// <para>   A token type can be set to any of those listed under the TokenType enum (except for TokenType::None).</para>
         /// <para>   Bracket tokens (such as parenthesis, square brackets, or curly braces) are typically defined in pairs.  So if "(", ")"
         ///    are defined as a token pair, then CloseBracket would be ")".</para>
         /// <para>   In cases where you want a submatch within the token, and not the whole token, use the SubMatch argument with a numerical
         ///    value represent the numerical index used in the regular expression.  Useful for instance in matching quoted text without the quotes themselves.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item Add(const string & TokenDef ,TokenType Type  = TokenType::Generic,const string & CloseBracket  = "",size_t SubMatch  = 0,RegExGrammar Grammar  = RegExGrammar::Default,size_t InsertionPoint  = -1) { return Add_208((void *)h, TokenDef.data(), TokenDef.size() , Type, CloseBracket.data(), CloseBracket.size(), SubMatch, Grammar, InsertionPoint);          }
         
         /// <summary> Defines a token</summary> 
         /// <param name="TokenDef"> Token definition</param>
         /// <param name="Type"> The type of token</param>
         /// <param name="SubMatch"> Submatch</param>
         /// <param name="Grammar"> Type of RegEx grammar</param>
         /// <returns> Current object</returns>
         /// <remarks>
         /// <para>
         /// Defines a token.  Similar to Add.  Add is more complete, but calls an internal version of Define().
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item Define(const string & TokenDef ,TokenType Type  = TokenType::Generic,size_t SubMatch  = 0,RegExGrammar Grammar  = RegExGrammar::Default) { return Define_209((void *)h, TokenDef.data(), TokenDef.size() , Type, SubMatch, Grammar);          }
         
         /// <summary> Defines a new token</summary> 
         /// <param name="InsertionPoint"> Where to insert the token into the token list</param>
         /// <param name="TokenDef"> Token definition</param>
         /// <param name="Type"> The type of token</param>
         /// <param name="CloseBracket"> String representing closing bracket, such as "}", ")", "]", etc.</param>
         /// <param name="SubMatch"> Submatch</param>
         /// <param name="Grammar"> Type of RegEx grammar</param>
         /// <returns> Token item object</returns>
         /// <remarks>
         /// <para>
         ///    This is similar to Add(), however add appends the token to the end of the token list by default, whereas Insert() lets you choose where in the list to insert
         ///    the token definition with the first parameter (TokenListIndex).  Tokens most recently defined have precedence over tokens defined earlier.  However,
         ///    this allows a new token definition to have less precedence than the default.
         ///    0 inserts at the beginning, 1 inserts at the 2nd location etc.  Negative numbers insert starting at the end of the list, with -1 at the last position
         ///    (default), -2 inserts at the next to last location, etc.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item Insert(int32_t InsertionPoint ,const string & TokenDef ,TokenType Type  = TokenType::Generic,const string & CloseBracket  = "",size_t SubMatch  = 0,RegExGrammar Grammar  = RegExGrammar::Default) { return Insert_210((void *)h, InsertionPoint , TokenDef.data(), TokenDef.size() , Type, CloseBracket.data(), CloseBracket.size(), SubMatch, Grammar);          }
         
         /// <summary> Returns the nth token from the token list</summary> 
         /// <param name="Index"> nth Index</param>
         /// <returns> Token Item</returns>
         /// <remarks>
         /// <para>
         /// Returns the nth token from the token list.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Item Token(size_t Index) { return Token_211((void *)h, Index);          }
         
         /// <summary> Returns a token's position in the token list</summary> 
         /// <param name="GivenToken"> Token Item</param>
         /// <returns> Integer value</returns>
         /// <remarks>
         /// <para>
         /// Returns a token's position in the token list.
         /// </para>
         /// </remarks> 
         int32_t IndexOf(Item GivenToken) { return IndexOf_212((void *)h, (void *)GivenToken.h );          }
         
         /// <summary> Removes a token from the token list</summary> 
         /// <param name="TokenItem"> Token to be removed from the token list</param>
         /// <remarks>
         /// <para>
         /// Removes the specified token from the token list.
         /// </para>
         /// </remarks> 
         void Remove(Item TokenItem) { Remove_213((void *)h, (void *)TokenItem.h );          }
         
         /// <summary> Sets a token pattern that will cause a temporary switch to another token list</summary> 
         /// <param name="OtherTokenGroup"> Other token list to switch to</param>
         /// <param name="StartToken"> Regex pattern that triggers the context switch</param>
         /// <param name="EndToken"> Regex pattern that causes a return to the original token list</param>
         /// <returns> Current token list</returns>
         /// <remarks>
         /// <para>
         ///    This causes a context switch to happen when a given token pattern is reached.  This is useful for instance if are
         ///    parsing source code, and you want comments to be parsed based on a different set of tokens from the rest of the code.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Tokens ContextSwitch(Tokens OtherTokenGroup ,const string & StartToken ,const string & EndToken) { return ContextSwitch_214((void *)h, (void *)OtherTokenGroup.h  , StartToken.data(), StartToken.size() , EndToken.data(), EndToken.size());          }
         
         /// <summary> Sets a token pattern that will cause a temporary switch to another token list with configurable insertion points</summary> 
         /// <param name="OtherTokenGroup"> Other token list to switch to</param>
         /// <param name="StartToken"> Regex pattern that triggers the context switch</param>
         /// <param name="InsertionPointA"> Where in the token list to insert the new token</param>
         /// <param name="EndToken"> Regex pattern that causes a return to the original token list</param>
         /// <param name="InsertionPointB"> Where in the other token list (a copy of it) to insert the ending token</param>
         /// <returns> Current token list</returns>
         /// <remarks>
         /// <para>
         /// This causes a context switch to happen when a given token pattern is reached.  This is useful for instance if are
         /// parsing source code, and you want comments to be parsed based on a different set of tokens from the rest of the code.</para>
         /// <para>This version of the function lets you decide where in the token list to insert the context switching token.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Tokens ContextSwitch(Tokens OtherTokenGroup ,const string & StartToken ,int32_t InsertionPointA  = -1,const string & EndToken  = "",int32_t InsertionPointB  = -1) { return ContextSwitch_215((void *)h, (void *)OtherTokenGroup.h  , StartToken.data(), StartToken.size() , InsertionPointA, EndToken.data(), EndToken.size(), InsertionPointB);          }
         
         /// <summary> Sets a token that will cause a temporary switch to another token list with configurable insertion points</summary> 
         /// <param name="OtherTokenGroup"> Other token list to switch to</param>
         /// <param name="StartToken"> Existing token that will cause the context switch</param>
         /// <param name="InsertionPointA"> // Where in the token list to insert the token</param>
         /// <param name="EndToken"> Regex pattern that causes a return to the original token list</param>
         /// <param name="InsertionPointB"> Where in the other token list (a copy of it) to insert the ending token</param>
         /// <returns> Current token</returns>
         /// <remarks>
         /// <para>
         /// This causes a context switch to happen when a given token pattern is reached.  This is useful for instance if are
         /// parsing source code, and you want comments to be parsed based on a different set of tokens from the rest of the code.</para>
         /// <para>This version of the function inserts a pre-existing token for the context switch, and lets you decide where in the token list to insert it.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::Tokens ContextSwitch(Tokens OtherTokenGroup ,Item StartToken ,int32_t InsertionPointA  = -1,const string & EndToken  = "",int32_t InsertionPointB  = -1) { return ContextSwitch_216((void *)h, (void *)OtherTokenGroup.h  , (void *)StartToken.h  , InsertionPointA, EndToken.data(), EndToken.size(), InsertionPointB);          }
         
         /// <summary> Clears the token list</summary> 
         /// <remarks>
         /// <para>
         /// Removes all tokens from the token list.  When a token list is cleared, it is typically prior to replacing it with a new
         /// token list.  A token list should generally not remain empty when doing a transform.  A token list should at minimum contain a catch-all
         /// pattern, which might capture one character.
         /// </para>
         /// </remarks> 
         void Clear() { Clear_217((void *)h);          }
         
         /// <summary> Returns the number of tokens in a token list</summary> 
         /// <returns> Integer representing number of tokens</returns>
         /// <remarks>
         /// <para>
         /// This returns the number of tokens in a token list.
         /// </para>
         /// </remarks> 
         size_t Count() { return Count_218((void *)h);          }
         
         Tokens() { h = nullptr; hb = h;          }
         
         /// <summary> Releases a token list from memory.  Transform rules are defined with FromTo.</summary> 
         /// <remarks>
         /// <para>
         ///    This releases a transform rule from memory.
         /// </para>
         /// </remarks> 
         void Release() { if ((void *)h != nullptr) Release_219((void *)h); h = nullptr;          }
         
         /// <summary> Returns the handle of an object</summary> 
         /// <remarks>
         /// <para>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload
         ///    (when applicable), you may check Handle() to see when it becomes NULL (or 0) to know when to stop.</para>
         /// </remarks> 
         void * Handle() { return Handle_220((void *)h);          }
         
         Tokens(uCalcPtr PassedHandle) { h = PassedHandle; hb = h;          }
      }; // Tokens
      
      //Product: uCalc String Library
      
      /// <summary> uCalc string library class</summary> 
      struct  String {
         
         /// <summary> Handle for uCalc String Library instance</summary> 
         uCalcPtr h;
         
         /// <summary> Returns the length (number of characters) of the nth token i a string</summary> 
         /// <param name="StartIndex"> Index of nth token</param>
         /// <param name="Count"> Number of consecutive tokens to count</param>
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the length (number of characters) of the nth token in a string.</para>
         /// <para>   Note: When Count > 1, white space characters between tokens are part of the length.
         /// </para>
         /// </remarks> 
         size_t TokenCharLen(size_t StartIndex ,size_t Count  = 1) { return TokenCharLen_221((void *)h, StartIndex , Count);          }
         
         /// <summary> Returns the numbers of tokens in the string</summary> 
         /// <param name="Index"> Index of nth token</param>
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    This returns the number of tokens that are found in the string.  If the Index argument is used, it returns the
         ///    number of tokens in the nth match.
         /// </para>
         /// </remarks> 
         size_t TokenCount(size_t Index  = -1) { return TokenCount_222((void *)h, Index);          }
         
         /// <summary> Returns the type of token the nth token is</summary> 
         /// <param name="Index"> Index of nth token</param>
         /// <returns> TokenType</returns>
         /// <remarks>
         /// <para>
         ///    Returns the type of token the nth token is.
         /// </para>
         /// </remarks> 
         TokenType TypeOfToken(size_t Index) { return TypeOfToken_223((void *)h, Index);          }
         
         /// <summary> Compares two strings</summary> 
         /// <param name="OtherString"> The other string to compare with</param>
         /// <param name="IndexA"> Starting point of first string</param>
         /// <param name="IndexB"> Starting point of next string</param>
         /// <param name="Length"> Length</param>
         /// <returns> Returns bigger than, equal, or less than (Comparison enum)</returns>
         /// <remarks>
         /// <para>
         ///    Compares two strings
         /// </para>
         /// </remarks> 
         Comparison Compare(const string & OtherString ,size_t IndexA  = 0,size_t IndexB  = 0,size_t Length  = -1) { return Compare_224((void *)h, OtherString.data(), OtherString.size() , IndexA, IndexB, Length);          }
         
         /// <summary> Returns the index of the token that contains the indicated character</summary> 
         /// <param name="CharacterPosition"> Character position</param>
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the index of the token that contains the indicated character.
         /// </para>
         /// </remarks> 
         size_t TokenIndexAtChar(size_t CharacterPosition) { return TokenIndexAtChar_225((void *)h, CharacterPosition);          }
         
         /// <summary> Applies a special format to the string</summary> 
         /// <param name="expression"> Expression</param>
         /// <param name="Variable"> Name of variable</param>
         /// <returns> Returns the current uCalc string object</returns>
         /// <remarks>
         /// <para>
         ///    This applies a special format to the string.  The format consists of an expression that evaluates to a string, where
         ///    the variable x contains the string.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Apply(const string & expression ,const string & Variable  = "") { return Apply_226((void *)h, expression.data(), expression.size() , Variable.data(), Variable.size());          }
         
         /// <summary> Returns the nth token</summary> 
         /// <param name="Index"> Nth otken Index</param>
         /// <returns> String</returns>
         /// <remarks>
         /// <para>
         ///     Returns the nth token.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String At(size_t Index) { return At_227((void *)h, Index);          }
         
         /// <summary> Returns the character at the given location</summary> 
         /// <param name="CharacterPosition"> Index of charactor location</param>
         /// <returns> String object</returns>
         /// <remarks>
         /// <para>
         ///    This returns the character at a given location.  The character is returned as a uCalc string.  If that string is modified, the modification is
         ///    reflected in the overall string.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String AtChar(size_t CharacterPosition) { return AtChar_228((void *)h, CharacterPosition);          }
         
         /// <summary> Returns the text string of the nth token</summary> 
         /// <param name="Index"> Index of token location</param>
         /// <returns> uCalc string</returns>
         /// <remarks>
         /// <para>
         ///    This returns the text string of the nth token.  Modifications to the returned uCalc string are reflected in the overall string.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String AtToken(size_t Index) { return AtToken_229((void *)h, Index);          }
         
         /// <summary> Returns the matching bracket for the indicated token</summary> 
         /// <param name="Index"> nth Token index</param>
         /// <returns> uCalc string object</returns>
         /// <remarks>
         /// <para>
         ///    This returns the matching bracket for the indicated token.  Manipulations to the returned uCalc string will be reflected in the overall
         ///    string.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String BracketMatch(size_t Index) { return BracketMatch_230((void *)h, Index);          }
         
         /// <summary> Returns the text corresponding to a bracket followed by text up to the next matching closing bracket</summary> 
         /// <param name="Index"> nth Token Index</param>
         /// <returns> uCalc string object</returns>
         /// <remarks>
         /// <para>
         ///    This returns the text corresponding to a bracket followed by text up to the next matching closing bracket.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String BracketedText(size_t Index) { return BracketedText_231((void *)h, Index);          }
         
         /// <summary> Returns the text between a bracket and the the next matching closing bracket</summary> 
         /// <param name="Index"> nth Token Index</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    This returns the text between a bracket and the the next matching closing bracket.  Brackets themselves are not included.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String BracketedTextInside(size_t Index) { return BracketedTextInside_232((void *)h, Index);          }
         
         /// <summary> Returns the nth index of the closing bracket token</summary> 
         /// <param name="Index"> nth token index</param>
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns nth index of the closing bracket token.
         /// </para>
         /// </remarks> 
         size_t BracketMatchTokenIndex(size_t Index) { return BracketMatchTokenIndex_233((void *)h, Index);          }
         
         /// <summary> Returns the character position of the nth match</summary> 
         /// <param name="Nth"> Index of nth match</param>
         /// <returns> Integer that represents the character position</returns>
         /// <remarks>
         /// <para>
         ///    This returns the character position of the nth match.
         /// </para>
         /// </remarks> 
         size_t CharPos(size_t Nth  = -1) { return CharPos_234((void *)h, Nth);          }
         
         /// <summary> Returns the number of characters of the nth match</summary> 
         /// <param name="Nth"> Index of nth match</param>
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    This returns the number of characters of the nth match.
         /// </para>
         /// </remarks> 
         size_t CharCount(size_t Nth  = -1) { return CharCount_235((void *)h, Nth);          }
         
         /// <summary> Returns the nth token position</summary> 
         /// <param name="Nth"> Index of nth match</param>
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the nth token position.
         /// </para>
         /// </remarks> 
         size_t TokenPos(size_t Nth  = -1) { return TokenPos_236((void *)h, Nth);          }
         
         /// <summary> Returns the character position of the nth token</summary> 
         /// <param name="Index"> Index of nth match</param>
         /// <remarks>
         /// <para>
         ///    Returns the character position of the nth token
         /// </para>
         /// </remarks> 
         size_t TokenCharPos(size_t Index) { return TokenCharPos_237((void *)h, Index);          }
         
         /// <summary> Retrieves the string value of a uCalc String object</summary> 
         /// <param name="Nth"> Index of match</param>
         /// <param name="TokenCount"> Number of tokens</param>
         /// <returns> String value of the uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    This retrieves the string value associated with a uCalc String object.
         ///    If no argument is passed then the entire string is returned.
         ///    Use Str(n) when you just want the string representation of a match.
         ///    Use Match(n) when you want uCalc String Object that you can manipulate further.
         /// </para>
         /// </remarks> 
         string Str(size_t Nth  = -1,size_t TokenCount  = 1) {
            size_t ReturnStrSize;
            const char * ReturnStr = Str_238((void *)h, ReturnStrSize , Nth, TokenCount);
            return string(ReturnStr, ReturnStrSize);
         }
         
         /// <summary> Assigns a string value to the uCalc String object</summary> 
         /// <param name="StringValue"> New string value to assign to the uCalc string object</param>
         /// <remarks>
         /// <para>
         ///    This assigns a new string to the uCalc String object.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Str(const string & StringValue) { return Str_239((void *)h, StringValue.data(), StringValue.size());          }
         
         /// <summary> Searches for occurences of a pattern in the string</summary> 
         /// <param name="Pattern"> Pattern</param>
         /// <param name="Nth"> Nth match</param>
         /// <returns> This uCalc string object</returns>
         /// <remarks>
         /// <para>
         ///      Searches for occurences of a pattern in the string.  Once this operation is executed, you may retrieve any of the matches with Match(n).
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Find(const string & Pattern ,size_t Nth  = 1) { return Find_240((void *)h, Pattern.data(), Pattern.size() , Nth);          }
         
         /// <summary> Finds occurences of all text other than the matching pattern</summary> 
         /// <param name="Pattern"> Pattern</param>
         /// <param name="Nth"> Nth match</param>
         /// <returns> This uCalc string object</returns>
         /// <remarks>
         /// <para>
         ///    Finds occurences of all text other than the matching pattern.  This allows you to perform a transform on all text other than text that matches
         ///    the given pattern.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String FindOtherThan(const string & Pattern ,size_t Nth  = 1) { return FindOtherThan_241((void *)h, Pattern.data(), Pattern.size() , Nth);          }
         
         /// <summary> Returns the immediate parent of the current string</summary> 
         /// <returns> uCalc string object that represents the parent of this string</returns>
         /// <remarks>
         /// <para>
         ///    Returns the immediate parent of the current string.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Parent() { return Parent_242((void *)h);          }
         
         /// <summary> Returns the root string of the current string</summary> 
         /// <returns> uCalc String object representing the root string</returns>
         /// <remarks>
         /// <para>
         ///    Returns the root string of the current string
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Root() { return Root_243((void *)h);          }
         
         /// <summary> Sets the index base used in various functions</summary> 
         /// <param name="BaseValue"> value representing index of 1st element</param>
         /// <remarks>
         /// <para>
         ///    By default, uCalc String Library functions use a 0-based index.  So the first element is 0 (not 1), followed by 1, 2, 3, 4 etc.
         ///    Use this if you prefer to start by 1 or another non-zero value.  This value is passed on to string objects derived from a parent string.
         ///    Each string may have a different index base.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String IndexBase(size_t BaseValue) { return IndexBase_244((void *)h, BaseValue);          }
         
         /// <summary> Gets the index base used in various functions</summary> 
         /// <remarks>
         /// <para>
         ///    By default, uCalc String Library functions use a 0-based index.  So the first element is 0 (not 1), followed by 1, 2, 3, 4 etc.
         ///    Use this to check to see which base is being used.
         /// </para>
         /// </remarks> 
         size_t IndexBase() { return IndexBase_245((void *)h);          }
         
         /// <summary> Returns a substring with all text up to and including the matched pattern</summary> 
         /// <param name="Pattern"> Pattern to match</param>
         /// <param name="Nth"> nth occurence</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Returns all text up to and including the matched pattern.  By default it goes up to the first occurence of a match, but you may elect to have it go
         ///    up to the nth occurence.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String UpTo(const string & Pattern ,size_t Nth  = 1) { return UpTo_246((void *)h, Pattern.data(), Pattern.size() , Nth);          }
         
         /// <summary> Retruns a substring starting from the given occurence of a match</summary> 
         /// <param name="Pattern"> Pattern to match</param>
         /// <param name="Nth"> Nth occurence</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Retruns the text starting from the given occurence of a match until the end.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String StartingFrom(const string & Pattern ,size_t Nth  = 1) { return StartingFrom_247((void *)h, Pattern.data(), Pattern.size() , Nth);          }
         
         /// <summary> Returns a substring that starts with a given pattern and ends with another pattern</summary> 
         /// <param name="StartPattern"> Start pattern</param>
         /// <param name="EndPattern"> End pattern</param>
         /// <param name="StartNth"> nth occurence for Start pattern</param>
         /// <param name="EndNth"> nth occurence for End pattern</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Returns text that starts with a given pattern and ends with another pattern.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String StartingFromEndingWith(const string & StartPattern ,const string & EndPattern ,size_t StartNth  = 1,size_t EndNth  = 1) { return StartingFromEndingWith_248((void *)h, StartPattern.data(), StartPattern.size() , EndPattern.data(), EndPattern.size() , StartNth, EndNth);          }
         
         /// <summary> Returns a substring of all text up to (but not including) a given pattern</summary> 
         /// <param name="Pattern"> Pattern</param>
         /// <param name="Nth"> Nth match</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Returns a substring of all text up to (but not including) a given pattern
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Before(const string & Pattern ,size_t Nth  = 1) { return Before_249((void *)h, Pattern.data(), Pattern.size() , Nth);          }
         
         /// <summary> Returns a substring of all text that comes after the occurence of a pattern</summary> 
         /// <param name="Pattern"> Pattern</param>
         /// <param name="Nth"> Nth match</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Returns a substring of all text that comes after the occurence of a pattern.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String After(const string & Pattern ,size_t Nth  = 1) { return After_250((void *)h, Pattern.data(), Pattern.size() , Nth);          }
         
         /// <summary> Returns a substring of text in between (but not including) occurences of two given patterns</summary> 
         /// <param name="StartPattern"> Pattern</param>
         /// <param name="StopPattern"> Pattern 2</param>
         /// <param name="StartNth"> Nth match of starting pattern</param>
         /// <param name="StopNth"> nth match of ending pattern</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Returns a substring of text in between (but not including) occurences of two given patterns.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Between(const string & StartPattern ,const string & StopPattern ,size_t StartNth  = 1,size_t StopNth  = 1) { return Between_251((void *)h, StartPattern.data(), StartPattern.size() , StopPattern.data(), StopPattern.size() , StartNth, StopNth);          }
         
         /// <summary> Returns a list of tokens</summary> 
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Returns a list of tokens.  Each token can be accessed with Match(n) or other indexed function.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String ListOfTokens() { return ListOfTokens_252((void *)h);          }
         
         /// <summary> Returns list of characters</summary> 
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Returns a list of characters.  Each characters can be accessed as a uCalc string with Match(n) or other indexed function.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String ListOfChars() { return ListOfChars_253((void *)h);          }
         
         /// <summary> Returns substring</summary> 
         /// <param name="StartPos"> Starting position</param>
         /// <param name="Length"> Length of substring</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Returns a substring of the current string.</para>
         /// <para>   Note: the StartPos and Length parameters are in terms of tokens, not characters.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String SubString(size_t StartPos ,size_t Length  = -1) { return SubString_254((void *)h, StartPos , Length);          }
         
         /// <summary> Sets rules for when a child object is created</summary> 
         /// <param name="Option"> Option</param>
         /// <returns> String object</returns>
         /// <remarks>
         /// <para>
         ///    Many functions create a new string object derived from another given object.  This option determins the persistence of previously
         ///    derived objects.  The right option depends on what you want to do.  If you're looking to conserve memory, then ClearAll may be the
         ///    option you need.  However, with this option, you can no longer refer to previously derived strings, otherwise the behavior will be
         ///    undefined (such as crashing).  You may delay object deletetion be removing only the ones that overlap in location with the newly derived
         ///    string.  The decision to delete may be delayed based on when an object is being written to.   ChildStringOptions::ClearOverlapOnWrite, allows
         ///    you to have multiple overlapping views of the same parent string without any conflict.  Only when modifying a derived string are overlapped
         ///    derived strings removed.  Corresponding Detach options are similar, except that instead of actually erasing an overlapping object, it
         ///    is disassassiated from the parent string.  It can continue to be used as an independent string without undefined behavior.  With this
         ///    option, the independent strings are erased only when the parent string itself is erased.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String ChildStringOption(ChildStringOptions Option) { return ChildStringOption_255((void *)h, Option);          }
         
         /// <summary> Gets the option</summary> 
         /// <returns> ChildStringOptions</returns>
         /// <remarks>
         /// <para>
         ///    Retrieves the option for  ChildStringOption.
         /// </para>
         /// </remarks> 
         ChildStringOptions ChildStringOption() { return ChildStringOption_256((void *)h);          }
         
         /// <summary> Clears derived strings from memory</summary> 
         /// <returns> String object</returns>
         /// <remarks>
         /// <para>
         ///    Clears derived strings from memory.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String ClearDerivedStrings() { return ClearDerivedStrings_257((void *)h);          }
         
         /// <summary> Removes occurences of a pattern in the string</summary> 
         /// <param name="From_"> pattern of occurences to remove from a string</param>
         /// <returns> uCalc String</returns>
         /// <remarks>
         /// <para>
         ///    Removes occurences of a pattern in the string.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Remove(const string & From_) { return Remove_258((void *)h, From_.data(), From_.size());          }
         
         /// <summary> Replaces occurences of a pattern with something else</summary> 
         /// <param name="From_"> patter of occurence to match</param>
         /// <param name="To_"> replacement of occurence</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Replaces occurences of a pattern with something else.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Replace(const string & From_ ,const string & To_) { return Replace_259((void *)h, From_.data(), From_.size() , To_.data(), To_.size());          }
         
         /// <summary> Skips</summary> 
         /// <param name="Pattern"> Pattern</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Skips
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Skip(const string & Pattern) { return Skip_260((void *)h, Pattern.data(), Pattern.size());          }
         
         /// <summary> Changes occurences of a pattern to upper case</summary> 
         /// <param name="Pattern"> Pattern</param>
         /// <param name="Nth"> nth occrence of pattern</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Changes occurences of a pattern to upper case.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String ToUpper(const string & Pattern  = "",size_t Nth  = -1) { return ToUpper_261((void *)h, Pattern.data(), Pattern.size(), Nth);          }
         
         /// <summary> Changes occurences of a pattern to lower case</summary> 
         /// <param name="Pattern"> Pattern</param>
         /// <param name="Nth"> nth occrence of pattern</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Changes occurences of a pattern to lower case.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String ToLower(const string & Pattern  = "",size_t Nth  = -1) { return ToLower_262((void *)h, Pattern.data(), Pattern.size(), Nth);          }
         
         /// <summary> Returns a list of matches, consisting of characters, tokens, elements, statements, or lines</summary> 
         /// <param name="ListType"> Type of list</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Returns a list of matches, consisting of characters, tokens, elements, statements, or lines.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String List(StringListType ListType  = StringListType::Matches) { return List_263((void *)h, ListType);          }
         
         /// <summary> Sets a format for the List() function</summary> 
         /// <param name="Separator"> Separator</param>
         /// <param name="Prefix"> Prefix</param>
         /// <param name="Postfix"> Postfix</param>
         /// <param name="UserFunc"> UserFunc</param>
         /// <param name="Element"> Element</param>
         /// <param name="Index"> Index</param>
         /// <param name="Count"> Count</param>
         /// <remarks>
         /// <para>
         ///    This sets a format that determines how the value of Str() is returned.  Separator will be inserted between
         ///    each match; Prefix will be added before the string; Postfix is appended at the end of the string.  UserFunc
         ///    determines how each match is displayed.  Element is the name of the variable to use in the function (x by default).
         ///    Index is the name of the current match (Index by default).  Count is the number of matches (Count by default).</para>
         /// <para>   For instance:
         ///    Given this list of matches: "a", "b", "c", "d"
         ///    ListOfChars().ListFormat(", ", "{", "}", "$'{Index} of {Count}:{x}'").Str()
         ///    would return:
         ///    {1 of 4:a, 2 of 4:b, 3 of 4:c, 4 of 4:d}
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String ListFormat(const string & Separator ,const string & Prefix  = "",const string & Postfix  = "",const string & UserFunc  = "",const string & Element  = "",const string & Index  = "",const string & Count  = "") { return ListFormat_264((void *)h, Separator.data(), Separator.size() , Prefix.data(), Prefix.size(), Postfix.data(), Postfix.size(), UserFunc.data(), UserFunc.size(), Element.data(), Element.size(), Index.data(), Index.size(), Count.data(), Count.size());          }
         
         /// <summary> Sets the separator that will delimite the elements of a list</summary> 
         /// <param name="Separator"> Separator that will delimite each element of a list</param>
         /// <returns> Current uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items
         ///    within curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String ListSeparator(const string & Separator) { return ListSeparator_265((void *)h, Separator.data(), Separator.size());          }
         
         /// <summary> Sets the prefix that is inserted at the start of lists</summary> 
         /// <param name="Prefix"> Prefix that will be inserted at the start of a list</param>
         /// <returns> Current uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items
         ///    within curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String ListPrefix(const string & Prefix) { return ListPrefix_266((void *)h, Prefix.data(), Prefix.size());          }
         
         /// <summary> Sets the postfix that is appended at the end of a list</summary> 
         /// <param name="Postfix"> Postfix that will be appended at the end of a list</param>
         /// <returns> Current uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items
         ///    within curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String ListPostfix(const string & Postfix) { return ListPostfix_267((void *)h, Postfix.data(), Postfix.size());          }
         
         /// <summary> Sets the formatting function for displaying the elements of a list</summary> 
         /// <param name="UserFunc"> UserFunc</param>
         /// <param name="Element"> Element</param>
         /// <param name="Index"> Index</param>
         /// <param name="MatchCount"> MatchCount</param>
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items
         ///    within curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.
         ///    See ListFormat().
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String ListFunction(const string & UserFunc ,const string & Element  = "",const string & Index  = "",const string & MatchCount  = "") { return ListFunction_268((void *)h, UserFunc.data(), UserFunc.size() , Element.data(), Element.size(), Index.data(), Index.size(), MatchCount.data(), MatchCount.size());          }
         
         /// <summary> Creates a string from a programmatically constructed list</summary> 
         /// <param name="first"> first</param>
         /// <param name="last"> last</param>
         /// <param name="expression"> expression</param>
         /// <param name="Step"> Step</param>
         /// <param name="Condition"> Condition</param>
         /// <param name="VarIndex"> VarIndex</param>
         /// <param name="VarFirst"> VarFirst</param>
         /// <param name="VarLast"> VarLast</param>
         /// <returns> uCalc string object</returns>
         /// <remarks>
         /// <para>
         ///    For example:
         ///    Populate(2, 5, "$'{Index}:{First}:{Last} etc.'").Str();
         ///    would return:
         ///    "2:2:5 etc.3:2:5 etc.4:2:5 etc.5:2:5 etc."
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Populate(double first ,double last ,const string & expression ,double Step  = 1,const string & Condition  = "",const string & VarIndex  = "",const string & VarFirst  = "",const string & VarLast  = "") { return Populate_269((void *)h, first , last , expression.data(), expression.size() , Step, Condition.data(), Condition.size(), VarIndex.data(), VarIndex.size(), VarFirst.data(), VarFirst.size(), VarLast.data(), VarLast.size());          }
         
         /// <summary> Returns the nth match</summary> 
         /// <param name="Index"> Index of nth match</param>
         /// <returns> uCalc string object</returns>
         /// <remarks>
         /// <para>
         ///    Returns the nth match.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Match(size_t Index) { return Match_270((void *)h, Index);          }
         
         /// <summary> Returns the index of the nth match where a character location is</summary> 
         /// <param name="Index"> Index of the character who's match index you want</param>
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the index of the nth match where a character location is
         /// </para>
         /// </remarks> 
         size_t MatchIndexAtChar(size_t Index) { return MatchIndexAtChar_271((void *)h, Index);          }
         
         /// <summary> Returns the index of the nth match where a token location is</summary> 
         /// <param name="Index"> Index of the token who's match index you want</param>
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the index of the nth match where a token location is
         /// </para>
         /// </remarks> 
         size_t MatchIndexAtToken(size_t Index) { return MatchIndexAtToken_272((void *)h, Index);          }
         
         /// <summary> Returns a string with a list of matches consisting of everything other than occurences of the Find pattern.</summary> 
         /// <returns> uCalc String object</returns>
         /// <remarks>
         /// <para>
         ///    Returns a string with a list of matches consisting of everything other than occurences of the Find pattern.
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String NonMatches() { return NonMatches_273((void *)h);          }
         
         /// <summary> Returns the number of matches</summary> 
         /// <returns> Integer</returns>
         /// <remarks>
         /// <para>
         ///    Returns the number of matches
         /// </para>
         /// </remarks> 
         size_t MatchCount() { return MatchCount_274((void *)h);          }
         
         /// <summary> Transforms a string</summary> 
         /// <param name="tt"> Transforms</param>
         /// <returns> Transformed string object</returns>
         /// <remarks>
         /// <para>
         ///    Transforms a string
         /// </para>
         /// </remarks> 
         uCalcSoftware::uCalc::String Transform(Transforms tt) { return Transform_275((void *)h, (void *)tt.h );          }
         
         String() { h = Default().NewString().h;          }
         String(const String &Instance) { h = Instance.h;          } // +++ Check again
         String(const string & s) { h = Default().NewString(s).h;          }
         String(const char * s) { h = Default().NewString(s).h;          }
         operator string() { return Str();          }
         String& operator=(const string & other) { Str(other); return *this;          }
         String& operator=(const char * other) { Str(other); return *this;          }
         
         /// <summary> Releases a uCalc string</summary> 
         /// <remarks>
         /// <para>
         ///    This releases a uCalc string from memory along with all the objects dependent on it.
         /// </para>
         /// </remarks> 
         void Release() { if ((void *)h != nullptr) Release_276((void *)h); h = nullptr;          }
         
         /// <summary> Returns the handle of an object</summary> 
         /// <remarks>
         /// <para>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload
         ///    (when applicable), you may check Handle() to see when it becomes NULL (or 0) to know when to stop.</para>
         /// </remarks> 
         void * Handle() { return Handle_277((void *)h);          }
         
         String(uCalcPtr PassedHandle) { h = PassedHandle;          }
      }; // uCalcStringLib
      
      /// <summary> Handle for uCalc instance</summary> 
      uCalcPtr h;
      
      /// <summary> Adds an error handler</summary> 
      /// <param name="ErrorHandlerAddress"> Address of the callback function containing the error handler code</param>
      /// <param name="Position"> Position in which to insert the error handler</param>
      /// <returns> The error handler item, which can later be removed using this return object</returns>
      /// <remarks>
      /// <para>Use this to set a callback function that is called whenever there is an error.  Your callback can
      /// do things like alert the user of the error, abort the operation, correct the error and resume, keep a log of errors, etc.</para>
      /// <para>By default, if there are multiple error handlers, they are called starting with the most recently
      /// defined one first, ending with the first one to be defined last.  In order to define a handler in a different
      /// order, specify the Position value of where it should be inserted; 0 so that it is called first, -1 so it
      /// is called last, or a positive integer indicating where it should be inserted.  If the Postion value is out
      /// of bounds, it is adjusted into the maximum acceptable value.</para>
      /// <para>If you set ErrorResponse to ErrorHandlerResponse::Resume, subsequent error handler callbacks will
      /// be bypassed and uCalc will resume operations as though no error had taken place.  Note that the condition
      /// that caused the error must be corrected before resuming, otherwise uCalc may get stuck in an infinite loop.</para>
      /// <para>Some functions, such as ErrorExpression, ErrorLocation, and ErrorSymbol produce a meaningful result only
      /// during the parsing stage.  They do not cause other problems beyond meaningless data during the evaluation stage.
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Item AddErrorHandler(uCalcCallback ErrorHandlerAddress ,size_t Position  = 0) { return AddErrorHandler_278((void *)h, ErrorHandlerAddress , Position);       }
      
      /// <summary> Creates a new uCalc instance</summary> 
      /// <param name="InheritsFrom"> Other uCalc object to inherit from</param>
      /// <returns> Returns the newly created uCalc object</returns>
      /// <remarks>
      /// <para>This function is invoked when a uCalc object is constructed, but may be invoked explicitely as well; it's useful if you want to inherit from another uCalc object.</para>
      /// </remarks> 
      static  uCalcSoftware::uCalc Create(uCalc InheritsFrom  = nullptr) { return Create_279(0, (void *)InheritsFrom.h );       }
      
      /// <summary> Creates a new uCalc instance and makes it the default</summary> 
      /// <param name="InheritsFrom"> Other uCalc object to inherit from</param>
      /// <returns> uCalc object</returns>
      /// <remarks>
      /// <para>Creates a temporary new uCalc instance and makes it the default, from which other new uCalc instances will inherit from.  If released, it will revert back
      ///    to the previous default.</para>
      /// </remarks> 
      static  uCalcSoftware::uCalc CreateTempDefault(uCalc InheritsFrom  = nullptr) { return CreateTempDefault_280(0, (void *)InheritsFrom.h );       }
      
      /// <summary> Releases the current uCalc default and reverts to the previous default</summary> 
      /// <returns> uCalc object (the previous default)</returns>
      /// <remarks>
      /// <para>This releases the current uCalc default object, and reverts to the previous default.</para>
      /// </remarks> 
      static  uCalcSoftware::uCalc ReleaseTempDefault() { return ReleaseTempDefault_281(0);       }
      
      /// <summary> Adds an alias to an existing symbol</summary> 
      /// <param name="NewSymbol"> New symbol that will be set as equivalent to an existing symbol</param>
      /// <param name="ExistingSymbol"> Pre-existing symbol</param>
      /// <returns> Newly created alias Item</returns>
      /// <remarks>
      /// <para>
      /// This function creates a new symbol that is pegged to an existing symbol.  Whenever this new symbol is encountered by the parser,
      /// it will be the same as if it had encountered the pre-existing symbol this new symbol is an alias of.  This new symbol shares the same handle as the
      /// pre-existing symbol within the symbol table.  The new symbol actually has its own handle, but that handle can only be used to delete the item.
      /// The pre-existing handle is used for all other purposes.</para>
      /// <para>For simple situations where you just want two symbols to be considered equivalent by the parser, Alias is prefered as a more efficient option over
      /// ExpressionTransforms().FromTo(OldSymbol, NewSymbol), which would modify the expression in order for the symbols to be equal.
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Item Alias(const string & NewSymbol ,const string & ExistingSymbol) { return Alias_282((void *)h, NewSymbol.data(), NewSymbol.size() , ExistingSymbol.data(), ExistingSymbol.size());       }
      
      /// <summary> Adds an alias to an existing symbol</summary> 
      /// <param name="NewSymbol"> New symbol that will be set as equivalent to an existing symbol</param>
      /// <param name="ExistingItem"> Pre-existing item</param>
      /// <returns> Newly created alias Item</returns>
      /// <remarks>
      /// <para>
      /// This function creates a new symbol that is pegged to an existing symbol.  Whenever this new symbol is encountered by the parser,
      /// it will be the same as if it had encountered the pre-existing symbol this new symbol is an alias of.  This new symbol shares the same handle as the
      /// pre-existing symbol within the symbol table.  The new symbol actually has its own handle, but that handle can only be used to delete the item.
      /// The pre-existing handle is used for all other purposes.</para>
      /// <para>This version of Alias uses an Item object as second argument.  It is a little more efficient than the one that uses a string (the string version calls
      /// the internal version of this function).
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Item Alias(const string & NewSymbol ,Item ExistingItem) { return Alias_283((void *)h, NewSymbol.data(), NewSymbol.size() , (void *)ExistingItem.h );       }
      
      /// <summary> Adds an alias name to an existing command name for Define()</summary> 
      /// <param name="NewCommandName"> New command name that will be set as equivalent to an existing command name</param>
      /// <param name="ExistingCommandName"> Pre-existing command name</param>
      /// <remarks>
      /// <para>
      /// This creates an alternative command name for use with the Define() function.
      /// </para>
      /// </remarks> 
      void AliasForDefine(const string & NewCommandName ,const string & ExistingCommandName) { AliasForDefine_284((void *)h, NewCommandName.data(), NewCommandName.size() , ExistingCommandName.data(), ExistingCommandName.size());       }
      
      /// <summary> Returns the data type of the expression that was passed</summary> 
      /// <param name="expression"> Expression whose data type will be returned</param>
      /// <returns> Data type object</returns>
      /// <remarks>
      /// <para>
      ///    Returns the data type of the expression that was passed
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::DataType DataTypeOf(const string & expression) { return DataTypeOf_285((void *)h, expression.data(), expression.size());       }
      
      /// <summary> Returns a data type object</summary> 
      /// <param name="Index"> Index associated with the data type</param>
      /// <returns> Data type object</returns>
      /// <remarks>
      /// <para>Each instance of uCalc has its own separate handle & data type object for any given data type.  For
      /// instance, Item("Int64") in one instance of uCalc is not the same object as Item("Int64") from
      /// another uCalc instance.  This allows the same data type to have a different name and different formatting
      /// in different instances.  However, they share the same data type index across instances.  Given the index
      /// of a data type, you can directly obtain the proper data type object for a particular uCalc instance,
      /// faster than obtaining it by name as text with Item().
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::DataType DataTypeOf(BuiltInType Index) { return DataTypeOf_286((void *)h, Index);       }
      
      /// <summary> Defines a uCalc.Item object (a constunction, function, operator, variable, etc)</summary> 
      /// <param name="expression"> Text containing item to be defined.</param>
      /// <param name="Address"> Address a function or variable is attached to</param>
      /// <param name="dType"> Data type associated with the item</param>
      /// <returns> The defined item object</returns>
      /// <remarks>
      /// <para>Functions, operators, variables, constants, tokens, syntax constructs, etc., can be defined with
      /// this method.  Although there are specialized methods that do the same things, such as DefineFunction,
      /// DefineOperator, DefineVariable, and DefineConstant, this method may in same cases offer
      /// more flexibility.</para>
      /// <para>The expression must start with a keyword such as Function, Operator, Variable, Token, Syntax, etc., followed by a colon, before the expression.</para>
      /// <para>The expression must be preceded by a keyword and colon, such as "Variable:".  For a full list of keywords,
      /// see Definition Keywords.</para>
      /// <para>Sections are separated by ~~ (two tilda signs).
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Item Define(const string & expression ,void * Address  = 0,DataType dType  = nullptr) { return Define_287((void *)h, expression.data(), expression.size() , Address, (void *)dType.h );       }
      
      /// <summary> Defines a constant</summary> 
      /// <param name="Definition"> Text containing constant to be defined</param>
      /// <param name="VariableAddress"> Address the constant is attached to</param>
      /// <returns> The defined item object</returns>
      /// <remarks>
      /// <para>
      /// A constant definition is similar to that of a variable defined with DefineVariable, except that the end-user is not allowed to modify the value of a constant.
      /// You are, however, allowed to perform lower-level operations such as renaming a constant, or deleting it and re-defining a new constant or variable with the same
      /// name but different value.  If you attach the constant to the address of a variable in your host program, the host program may change the value.  However, when
      /// the end-user attempts to assign a value to the constant, an error is raised.</para>
      /// <para>DefineConstant("x") is the same as: Define("Lock ~~ Variable: x")
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Item DefineConstant(const string & Definition ,void * VariableAddress  = 0) { return DefineConstant_288((void *)h, Definition.data(), Definition.size() , VariableAddress);       }
      
      /// <summary> Defines a function</summary> 
      /// <param name="Definition"> Text containing function to be defined</param>
      /// <param name="FunctionAddr"> Address of callback function this function is attached to</param>
      /// <returns> The defined item object</returns>
      /// <remarks>
      /// <para>
      /// A function definition can either be self-contained, or it may be attached to the address of a callback function in your
      /// source code.</para>
      /// <para>DefineFunction("f(x) = ...") is the same as: Define("Function: ...")
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Item DefineFunction(const string & Definition ,uCalcCallback FunctionAddr  = 0) { return DefineFunction_289((void *)h, Definition.data(), Definition.size() , FunctionAddr);       }
      
      /// <summary> Defines an operator</summary> 
      /// <param name="Definition"> Text containing operator to be defined</param>
      /// <param name="PrecedenceLevel"> Precedence level of the operator</param>
      /// <param name="Direction"> Operator associativity; arithmetic operators are typically grouped left-to-right; special operators, such as the equal sign, =, assignment operator are right-to-left</param>
      /// <param name="FunctionAddr"> Address of callback function this operator is attached to</param>
      /// <returns> The defined item object</returns>
      /// <remarks>
      /// <para>
      /// Many details for defining an operator are similar to those of defining a function.  In additionn, all operators must be
      /// defined with a precedence level.  You may use the Precedence function to obtain precedence levels of existing operators,
      /// and select a precedence level relative to those operators.  By default operators are grouped left to right.  But you
      /// may also set it to right to left.  This means that when adjacent operators have the same precedence level, the operands on
      /// the right will be evaluated first.</para>
      /// <para>DefineOperator("...") is the same as: Define("Operator: ...")
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Item DefineOperator(const string & Definition ,size_t PrecedenceLevel ,Grouping Direction  = Grouping::LeftToRight,uCalcCallback FunctionAddr  = 0) { return DefineOperator_290((void *)h, Definition.data(), Definition.size() , PrecedenceLevel , Direction, FunctionAddr);       }
      
      /// <summary> Defines a variable</summary> 
      /// <param name="Definition"> Text containing variable to be defined</param>
      /// <param name="VariableAddress"> Address of variable in host program this uCalc variable is attached to</param>
      /// <returns> The defined item object</returns>
      /// <remarks>
      /// <para>
      /// A variable typically cotains a variable name, optionally followed by a data type, optionally followed by an equal sign and an
      /// expression whose result is the starting value for the variable.</para>
      /// <para>DefineVariable("...") is the same as: Define("Variable: x")
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Item DefineVariable(const string & Definition ,void * VariableAddress  = 0) { return DefineVariable_291((void *)h, Definition.data(), Definition.size() , VariableAddress);       }
      
      /// <summary> Returns the expression associated with an error that was raised</summary> 
      /// <returns> Text string</returns>
      string ErrorExpression() {
         size_t ReturnStrSize;
         const char * ReturnStr = ErrorExpression_292((void *)h, ReturnStrSize );
         return string(ReturnStr, ReturnStrSize);
      }
      
      /// <summary> Sets a new expression after an error is raised</summary> 
      /// <param name="expression"> Error expression</param>
      void ErrorExpression(const string & expression) { ErrorExpression_293((void *)h, expression.data(), expression.size());       }
      
      /// <summary> Returns the location of an error that was raised</summary> 
      size_t ErrorLocation() { return ErrorLocation_294((void *)h);       }
      
      /// <summary> Returns a message associated with the error that was raised</summary> 
      /// <param name="Index"> Index of error message to return</param>
      /// <returns> Text string</returns>
      /// <remarks>
      /// <para>If an error was raised, this returns a message describing the recent error.  If the Index argument is specified,
      /// then the error message associated with this Index is instead returned.  A value must be chosen from ErrorMsg.</para>
      /// <para>Note: Errors are cleared whenever a new expression is parsed or a new item is defined.
      /// </para>
      /// </remarks> 
      string ErrorMessage(ErrorMsg Index  = ErrorMsg::None) {
         size_t ReturnStrSize;
         const char * ReturnStr = ErrorMessage_295((void *)h, ReturnStrSize , Index);
         return string(ReturnStr, ReturnStrSize);
      }
      
      /// <summary> Returns the error number of the error that was raised</summary> 
      /// <returns> ErrorMsg enum</returns>
      /// <remarks>
      /// <para>
      /// Returns the error enum of the most recently raised error.</para>
      /// <para>Note: Errors are cleared whenever a new expression is parsed or a new item is defined.
      /// </para>
      /// </remarks> 
      ErrorMsg ErrorNumber() { return ErrorNumber_296((void *)h);       }
      
      /// <summary> Sets the error response for an error handler</summary> 
      /// <param name="Response"> Indicates what should happen now that an error is raised</param>
      /// <remarks>
      /// <para>When an error is raised, you have the option of aborting the rest of the operation (this is the default action),
      /// or you may correct the problem, and then resume the operation.  Or you may perform an action, and then raise the
      /// error again for a subsequent error handler to take care of.</para>
      /// <para>This method can only be called from within a callback of an error handler routine definined with AddErrorHandler.
      /// The options to use can be found at ErrorHandlerResponse.
      /// </para>
      /// </remarks> 
      void ErrorResponse(ErrorHandlerResponse Response) { ErrorResponse_297((void *)h, Response);       }
      
      /// <summary> Returns the symbol associated with the error that is raised</summary> 
      /// <returns> text string</returns>
      string ErrorSymbol() {
         size_t ReturnStrSize;
         const char * ReturnStr = ErrorSymbol_298((void *)h, ReturnStrSize );
         return string(ReturnStr, ReturnStrSize);
      }
      
      /// <summary> Parses and evaluates an expression in one step</summary> 
      /// <param name="expression"> Expression to be evaluated</param>
      /// <remarks>
      /// <para>
      /// This takes a math expression and returns a numeric result.  This is a convenient one-step alternative to
      /// ExprObj = Parse(Expressions);
      /// Result = ExprObj.Evaluate();
      /// </para>
      /// </remarks> 
      double Eval(const string & expression) { return Eval_299((void *)h, expression.data(), expression.size());       }
      
      /// <summary> Parses and evaluates an expression of any data type in one step, returning the result as a string</summary> 
      /// <param name="expression"> Expression to be evaluated</param>
      /// <returns> String containt result</returns>
      /// <remarks>
      /// <para>This is similar to Eval, however, it can return non-numeric results, such as strings.  Also, the expression can be of any type.  The result is converted
      /// to a string and then returned.</para>
      /// </remarks> 
      string EvalStr(const string & expression) {
         size_t ReturnStrSize;
         const char * ReturnStr = EvalStr_300((void *)h, ReturnStrSize , expression.data(), expression.size());
         return string(ReturnStr, ReturnStrSize);
      }
      
      /// <summary> Causes division by 0 errors to be raised</summary> 
      /// <param name="TrueOrFalse"> True or false</param>
      /// <remarks>
      /// <para>
      /// This causes an error to be raised instead of silently returning a special value when a division by 0 or pole error occurs.
      /// </para>
      /// </remarks> 
      void RaiseErrorOnDivideByZero(bool TrueOrFalse  = true) { RaiseErrorOnDivideByZero_301((void *)h, TrueOrFalse);       }
      
      /// <summary> Causes inexact results to raise an error</summary> 
      /// <param name="TrueOrFalse"> True or false</param>
      /// <remarks>
      /// <para>
      /// This causes an error to be raised instead returning an approximate value when an exact value cannot be represented.
      /// </para>
      /// </remarks> 
      void RaiseErrorOnInexact(bool TrueOrFalse  = true) { RaiseErrorOnInexact_302((void *)h, TrueOrFalse);       }
      
      /// <summary> Causes underflow to raise an error</summary> 
      /// <param name="TrueOrFalse"> True or false</param>
      /// <remarks>
      /// <para>
      /// This causes an error to be raised for underflow instead of just returning 0.
      /// </para>
      /// </remarks> 
      void RaiseErrorOnUnderflow(bool TrueOrFalse  = true) { RaiseErrorOnUnderflow_303((void *)h, TrueOrFalse);       }
      
      /// <summary> Causes overflow to raise an error</summary> 
      /// <param name="TrueOrFalse"> True or false</param>
      /// <remarks>
      /// <para>
      /// This causes an error to be raised instead returning infinity when there's an overflow.
      /// </para>
      /// </remarks> 
      void RaiseErrorOnOverflow(bool TrueOrFalse  = true) { RaiseErrorOnOverflow_304((void *)h, TrueOrFalse);       }
      
      /// <summary> Causes invalid value to raise an error</summary> 
      /// <param name="TrueOrFalse"> True or false</param>
      /// <remarks>
      /// <para>
      /// This causes an error to be raised instead returning NaN when an invalid value is encountered.
      /// </para>
      /// </remarks> 
      void RaiseErrorOnInvalid(bool TrueOrFalse  = true) { RaiseErrorOnInvalid_305((void *)h, TrueOrFalse);       }
      
      /// <summary> Returns the bits representing floating point errors that are cought & raised</summary> 
      /// <param name="Value"> True or false</param>
      /// <returns> Integer</returns>
      /// <remarks>
      /// <para>
      /// Returns the bits for the floating point errors that are to be raised
      /// </para>
      /// </remarks> 
      int32_t FloatingPointErrorsToCatch(int32_t Value  = -1) { return FloatingPointErrorsToCatch_306((void *)h, Value);       }
      
      /// <summary> Returns the uCalc.Item object associated with the given name</summary> 
      /// <param name="ItemName"> Name of uCalc.Item that you want to get</param>
      /// <param name="Properties"> Properties to help pinpoint the right Item in case several items share the same name</param>
      /// <returns> Item object</returns>
      /// <remarks>
      /// <para>
      /// If there is no match, an item whose ItemIs.NotFound property is true is returned.</para>
      /// <para>In case multiple items share the same name, you can narrow it down using properties.  For instance, the + operator has a
      /// binary and unary version.  You can specify Prefix if you wanted the unary one.</para>
      /// <para>Note: A ItemIs member should not be passed directly as the second argument.  Instead Properties, with one or more
      /// ItemIs members should be passed.
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Item ItemOf(const string & ItemName ,int64_t Properties  = 0) { return ItemOf_307((void *)h, ItemName.data(), ItemName.size() , Properties);       }
      
      /// <summary> Returns the nth uCalc.Item object that matches a property</summary> 
      /// <param name="Property_"> Property of the uCalc.Item object you're after</param>
      /// <param name="nth"> The nth object</param>
      /// <param name="DataTypeName"> The data type of the object</param>
      /// <returns> Item object</returns>
      /// <remarks>
      /// <para>
      /// This can be used to cycle through defined items.
      /// If there is no match, an item whose IsProperty ItemIs.NotFound value is true is returned.
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Item ItemOf(ItemIs Property_ ,size_t nth ,const string & DataTypeName  = "") { return ItemOf_308((void *)h, Property_ , nth , DataTypeName.data(), DataTypeName.size());       }
      
      /// <summary> Namespace symbol</summary> 
      /// <param name="nth"> Nth symbol</param>
      /// <returns> Nth symbol item object</returns>
      uCalcSoftware::uCalc::Item NamespaceSymbol(size_t nth) { return NamespaceSymbol_309((void *)h, nth);       }
      
      /// <summary> Number of symbols</summary> 
      size_t NamespaceSymbolCount() { return NamespaceSymbolCount_310((void *)h);       }
      
      /// <summary> Returns a new uCalc String object</summary> 
      /// <param name="InitialString"> String initial value</param>
      /// <returns> New uCalc string object</returns>
      /// <remarks>
      /// <para>
      /// Returns a new uCalc String object.
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::String NewString(const string & InitialString  = "") { return NewString_311((void *)h, InitialString.data(), InitialString.size());       }
      
      /// <summary> Returns a new transform object</summary> 
      /// <param name="TokensToInherit"> Existing token list to inherit from</param>
      /// <returns> Transform object</returns>
      /// <remarks>
      /// <para>
      /// Returns a new object that can be used to transform text.
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Transforms NewTransform(Tokens TokensToInherit  = nullptr) { return NewTransform_312((void *)h, (void *)TokensToInherit.h );       }
      
      /// <summary> Returns the transform object for parsing math expressions, associated with the current uCalc instance</summary> 
      /// <returns> Transform object</returns>
      /// <remarks>
      /// <para>
      /// Whenever a math expression is parsed, this transform is first called.  Use this when you want transforms that apply to math
      /// expressions parsed with Parse, Eval, or EvalStr.  To create a new transform that does not affect math expressions sent to
      /// Parse, use NewTransform instead.</para>
      /// <para>Independent transforms created with NewTransform can be released when no longer needed.  However, the object created with
      /// ExpressionTransforms should never be released explicitely, because the uCalc object requires it in order parse expressions.</para>
      /// <para>Note: For simple situations when you just want one keyword to be the equivalent of a pre-existing one, it is more efficient to
      /// use Alias().  With Alias, nothing is transformed.  Instead, an aliased symbol simply shares the same handle as a pre-existing
      /// one in the symbol table.  When there are no expression transforms at all, the transform step is skipped altogether, slightly
      /// speeding up parsing for all expressions in a uCalc instance.
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Transforms ExpressionTransforms() { return ExpressionTransforms_313((void *)h);       }
      
      /// <summary> Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary> 
      /// <param name="expression"> Expression to be parsed</param>
      /// <param name="DataType"> Data type of return value</param>
      /// <returns> Parsed expression</returns>
      uCalcSoftware::uCalc::Expression Parse(const string & expression ,DataType DataType  = nullptr) { return Parse_314((void *)h, expression.data(), expression.size() , (void *)DataType.h );       }
      
      /// <summary> Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary> 
      /// <param name="expression"> Expression to be parsed</param>
      /// <param name="DataType"> Data type of return value</param>
      /// <returns> Parsed expression</returns>
      uCalcSoftware::uCalc::Expression Parse(const string & expression ,const string & DataType) { return Parse_315((void *)h, expression.data(), expression.size() , DataType.data(), DataType.size());       }
      
      /// <summary> Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary> 
      /// <param name="expression"> Expression to be parsed</param>
      /// <param name="DataTypeIndex"> Data type index of return value</param>
      /// <returns> Parsed expression</returns>
      uCalcSoftware::uCalc::Expression Parse(const string & expression ,BuiltInType DataTypeIndex) { return Parse_316((void *)h, expression.data(), expression.size() , DataTypeIndex);       }
      
      /// <summary> Returns the precedence level of an operator or the level between two operators</summary> 
      /// <param name="Operator1"> Operator with lower (or equal) precedence</param>
      /// <param name="Operator2"> Operator with greater (or equal precedence)</param>
      /// <remarks>
      /// <para>
      /// This returns a precedence level based on levels of existing operators; useful when defining a new operator (or transform)
      /// with a level relative to levels of existing operators (or transforms).  If one argument is passed, then the precedence
      /// level of that item is returned.  This is useful for defining a new item at the same level as the specified item.  If two
      /// arguments are passed, then the average (mean) precedence level of the two items is returned.  This is useful for defining
      /// a level that's in between that of two existing items.
      /// </para>
      /// </remarks> 
      size_t Precedence(const string & Operator1 ,const string & Operator2  = "") { return Precedence_317((void *)h, Operator1.data(), Operator1.size() , Operator2.data(), Operator2.size());       }
      
      /// <summary> Combines properties into a numeric value that can be passed to Item</summary> 
      /// <param name="p1"> First property</param>
      /// <param name="p2"> Second property</param>
      /// <param name="p3"> Third property</param>
      /// <param name="p4"> Fourth property</param>
      /// <returns> Integer representation of combined properties</returns>
      /// <remarks>
      /// <para>
      /// ItemIs properties are numbered sequentially (0, 1, 2, 3, 4, ...).  However the Properties argument
      /// of the Item function requires properties in the 2^x sequence (1, 2, 4, 8, 16, ...).  So you
      /// must pass one or more ItemIs to Properties before passing it to Item.
      /// </para>
      /// </remarks> 
      static int64_t Properties(ItemIs p1 ,ItemIs p2  = ItemIs::SelectAny,ItemIs p3  = ItemIs::SelectAny,ItemIs p4  = ItemIs::SelectAny) { return Properties_318(0, p1 , p2, p3, p4);       }
      
      /// <summary> Returns default settings</summary> 
      /// <returns> Default Item</returns>
      /// <remarks>
      /// <para>The Item returned contains the default settings for the current uCalc instance.  To define new settings,
      /// use Define("Default ~~ ...").
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Item DefaultItem() { return DefaultItem_319((void *)h);       }
      
      /// <summary> Default data type</summary> 
      /// <returns> Default data type</returns>
      /// <remarks>
      /// <para>This returns the default data type.  The default type is Double unless explicitely changed to something else.
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::DataType DefaultDataType() { return DefaultDataType_320((void *)h);       }
      
      /// <summary> Sets a new default data type</summary> 
      /// <param name="dType"> The type you want to set as default</param>
      /// <remarks>
      /// <para>This sets a new default data type.  This new default will be used instead of Double for all definitions where a type is relevant, when an explicit
      /// type is not specified.</para>
      /// <para>The default is set using a data type object.
      /// </para>
      /// </remarks> 
      void DefaultDataType(DataType dType) { DefaultDataType_321((void *)h, (void *)dType.h );       }
      
      /// <summary> Sets a new default data type</summary> 
      /// <param name="DataTypeIndex"> Enum of type you want to set as default</param>
      /// <remarks>
      /// <para>This sets a new default data type.  This new default will be used instead of Double for all definitions where a type is relevant, when an explicit
      /// type is not specified.</para>
      /// <para>The default is set using a BuiltInType enum.
      /// </para>
      /// </remarks> 
      void DefaultDataType(BuiltInType DataTypeIndex) { DefaultDataType_322((void *)h, DataTypeIndex);       }
      
      /// <summary> Sets a new default data type</summary> 
      /// <param name="dTypeStr"> Name of existing data type</param>
      /// <remarks>
      /// <para>This sets a new default data type.  This new default will be used instead of Double for all definitions where a type is relevant, when an explicit
      /// type is not specified.</para>
      /// <para>The default is set using the name of a data type.
      /// </para>
      /// </remarks> 
      void DefaultDataType(const string & dTypeStr) { DefaultDataType_323((void *)h, dTypeStr.data(), dTypeStr.size());       }
      
      /// <summary> Checks to see if this uCalc object instance is the default</summary> 
      /// <returns> Boolean value</returns>
      /// <remarks>
      /// <para>
      /// Definitions from this instance will automatically be inherited by new instances unless you specify otherwise.
      /// </para>
      /// </remarks> 
      bool IsDefault() { return IsDefault_324((void *)h);       }
      
      /// <summary> Sets this uCalc object instance as the default</summary> 
      /// <param name="Condition"> True or False</param>
      /// <remarks>
      /// <para>
      /// Definitions from this instance will automatically be inherited by new instances unless you specify otherwise.
      /// Set this to True.  When set to False, nothing happens.
      /// </para>
      /// </remarks> 
      void IsDefault(bool Condition) { IsDefault_325((void *)h, Condition);       }
      
      /// <summary> Returns the default uCalc instance</summary> 
      /// <returns> uCalc object</returns>
      /// <remarks>
      /// <para>
      /// Returns the default uCalc instance.  New uCalc instances inherit from this by default.
      /// </para>
      /// </remarks> 
      static  uCalcSoftware::uCalc Default() { return Default_326(0);       }
      
      /// <summary> Sets a uCalc instance as the default</summary> 
      /// <param name="uCalcObj"> uCalc object to set as default</param>
      /// <remarks>
      /// <para>
      /// Sets the new uCalc default object that new uCalc instances will inherit from by default.
      /// </para>
      /// </remarks> 
      static void Default(uCalc uCalcObj) { Default_327(0, (void *)uCalcObj.h );       }
      
      /// <summary> Allows a callback to decorate the output of EvalStr</summary> 
      /// <param name="FunctionAddr"> Address of callback function that formats the output</param>
      /// <param name="Properties"> Name of datatype that will have its output formated and/or ranking among other output formats; omit this to affect all data types and use default ranking.</param>
      uCalcSoftware::uCalc::Item Format(uCalcCallback FunctionAddr ,const string & Properties  = "") { return Format_328((void *)h, FunctionAddr , Properties.data(), Properties.size());       }
      
      /// <summary> Allows a callback to decorate the output of EvalStr</summary> 
      /// <param name="FunctionAddr"> Address of callback function that formats the output</param>
      /// <param name="dType"> Data type that will have its output formated; omit this to affect all data types and use default ranking</param>
      uCalcSoftware::uCalc::Item Format(uCalcCallback FunctionAddr ,DataType dType) { return Format_329((void *)h, FunctionAddr , (void *)dType.h );       }
      
      /// <summary> Formats the appearance of results returned by EvalStr or EvaluateStr</summary> 
      /// <param name="Definition"> Format definition for the output</param>
      /// <param name="dType"> Data type that will have its output formated; omit this to affect all data types and use default ranking.</param>
      /// <remarks>
      /// <para>
      /// This function lets you modify the output returned by EvalStr or EvaluateStr.  The definition may contain one or more
      /// of the following:</para>
      /// <para>Datatype:  If you specify a data type, then the modification will affect only the evaluation result of expressions
      /// that evaluate to the specified data type.</para>
      /// <para>InsertAt:  By default, if you define multiple outputs, then the one defined most recently is applied first, followed by
      /// the other outputs in the reverse order in which they were defined, ending with the output that was defined first.  You may
      /// specify a different insertion point than the default with this parameter.</para>
      /// <para>Expression:  This represents what the modified output should look like.  The expression consists of an identifier
      /// (any temporary variable name), followed by an equal sign, followed by a string expression representing the value
      /// that will be returned.  The variable holds the original value that would have been returned.
      /// </para>
      /// </remarks> 
      uCalcSoftware::uCalc::Item Format(const string & Definition ,DataType dType  = nullptr) { return Format_330((void *)h, Definition.data(), Definition.size() , (void *)dType.h );       }
      
      /// <summary> Removes any output formatting</summary> 
      /// <param name="dType"> Data type</param>
      /// <remarks>
      /// <para>
      /// Removes formatting for EvalStr or EvaluateStr that was defined with Format.
      /// </para>
      /// </remarks> 
      void FormatRemove(DataType dType  = nullptr) { FormatRemove_331((void *)h, (void *)dType.h );       }
      
      /// <summary> Returns the value located at the specified pointer address</summary> 
      /// <param name="ValuePtr"> Pointer address of the value</param>
      /// <param name="dType"> Data type of the value; helps determine in which form to return the value at the given location</param>
      /// <returns> String representation of the value at the given address</returns>
      /// <remarks>
      /// <para>This returns the value located at the given address.  The value is first converted to a string.
      /// </para>
      /// </remarks> 
      string ValueAt(void * ValuePtr ,DataType dType  = nullptr) {
         size_t ReturnStrSize;
         const char * ReturnStr = ValueAt_332((void *)h, ReturnStrSize , ValuePtr , (void *)dType.h );
         return string(ReturnStr, ReturnStrSize);
      }
      
      /// <summary> Returns the value located at the specified pointer address</summary> 
      /// <param name="ValuePtr"> Pointer address containing the data</param>
      /// <param name="ValueDataType"> Data type of the data you need, which helps to determine in which form to return the value at the given location</param>
      /// <returns> String representation of the value at the given address</returns>
      /// <remarks>
      /// <para>This returns the value located at the given address.  The value is first converted to a string.
      /// </para>
      /// </remarks> 
      string ValueAt(void * ValuePtr ,const string & ValueDataType) {
         size_t ReturnStrSize;
         const char * ReturnStr = ValueAt_333((void *)h, ReturnStrSize , ValuePtr , ValueDataType.data(), ValueDataType.size());
         return string(ReturnStr, ReturnStrSize);
      }
      
      /// <summary> Sets the license code</summary> 
      /// <param name="Type"> Type of license</param>
      /// <param name="L1"> Value 1</param>
      /// <param name="L2"> Value 2</param>
      /// <param name="L3"> Value 3</param>
      /// <param name="L4"> Value 4</param>
      /// <param name="L5"> Value 5</param>
      /// <remarks>
      /// <para>
      /// The license code prevents the message box from showing.  A license can be purchsased from www.uCalc.com
      /// </para>
      /// </remarks> 
      static void License(LicenseType Type ,size_t L1  = 0,size_t L2  = 0,size_t L3  = 0,size_t L4  = 0,size_t L5  = 0) { License_334(0, Type , L1, L2, L3, L4, L5);       }
      
      void Reset() { Release(); h = Create().h;       }
      void Reset(uCalc uCalcObj) { Release(); h = Create(uCalcObj).h;       }
      static String Copy(String s) { String sRet(s.Str()); return sRet.h;       }
      static String Share(String s) { return s.h;       }
      static String Move(String s) { return s.h;       }
      //+++static String Borrow(String s) { }
      uCalc() { h = Create().h;       }
      
      /// <summary> Releases a uCalc instance</summary> 
      /// <remarks>
      /// <para>When a uCalc instance is no longer needed, it can be released with this function.  This releases all items
      /// associated with the uCalc instance.</para>
      /// <para>Note: The uCalc object named uc, which is instantiated at startup (except in C#) should not be explicitely released.
      /// In C#, if there is only 1 uCalc object left, do not explicitely release it.</para>
      /// <para>Note: A uCalc object must explicitely be released with Release for it to be released.  A uCalc object is not
      /// automatically released from memory when the object goes out of scope.</para>
      /// </remarks> 
      void Release() { if ((void *)h != nullptr) Release_335((void *)h); h = nullptr;       }
      
      /// <summary> Returns the handle of an object</summary> 
      /// <remarks>
      /// <para>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload
      ///    (when applicable), you may check Handle() to see when it becomes NULL (or 0) to know when to stop.</para>
      /// </remarks> 
      void * Handle() { return Handle_336((void *)h);       }
      
      uCalc(uCalcPtr PassedHandle) { h = PassedHandle;       }
      
   };
} // Namespace End

using namespace uCalcSoftware;

// End of Template
